@@ -24,7 +24,7 @@
 
 ;;; The DEFSTRUCT-DESCRIPTION structure holds compile-time information
 ;;; about a structure type.
-;;; It is defined prior to WRAPPER because WRAPPER-INFO
+;;; It is defined prior to LAYOUT because a LAYOUT-INFO slot
 ;;; is declared to hold a DEFSTRUCT-DESCRIPTION.
 (def!struct (defstruct-description
              (:conc-name dd-)
@@ -111,6 +111,65 @@
 ;;; ID must be an fixnum for either value of n-word-bits.
 (def!type layout-id () '(signed-byte 30))
 
+(declaim (start-block))
+
+;;; The CLASSOID structure is a supertype of all classoid types.  A
+;;; CLASSOID is also a CTYPE structure as recognized by the type
+;;; system.  (FIXME: It's also a type specifier, though this might go
+;;; away as with the merger of SB-PCL:CLASS and CL:CLASS it's no
+;;; longer necessary)
+(defstruct (classoid
+             (:include ctype)
+             (:constructor nil)
+             (:copier nil)
+             (:print-object
+              (lambda (class stream)
+                (let ((name (classoid-name class)))
+                  (print-unreadable-object (class stream
+                                                  :type t
+                                                  :identity (not name))
+                    (format stream
+                            ;; FIXME: Make sure that this prints
+                            ;; reasonably for anonymous classes.
+                            "~:[anonymous~;~:*~S~]~@[ (~(~A~))~]"
+                            name
+                            (classoid-state class))))))
+             #-sb-xc-host (:pure nil))
+  ;; the value to be returned by CLASSOID-NAME.
+  (name nil :type symbol)
+  ;; the current LAYOUT for this class, or NIL if none assigned yet
+  (layout nil :type (or null layout))
+  ;; How sure are we that this class won't be redefined?
+  ;;   :READ-ONLY = We are committed to not changing the effective
+  ;;                slots or superclasses.
+  ;;   :SEALED    = We can't even add subclasses.
+  ;;   NIL        = Anything could happen.
+  (state nil :type (member nil :read-only :sealed))
+  ;; direct superclasses of this class. Always NIL for CLOS classes.
+  (direct-superclasses () :type list)
+  ;; Definition location
+  ;; Not used for standard-classoid, because pcl has its own mechanism.
+  (source-location nil)
+  ;; representation of all of the subclasses (direct or indirect) of
+  ;; this class. This is NIL if no subclasses or not initalized yet;
+  ;; otherwise, it's an EQ hash-table mapping CLASSOID objects to the
+  ;; subclass layout that was in effect at the time the subclass was
+  ;; created.
+  ;; Initially an alist, and changed to a hash-table at some threshold.
+  (subclasses nil :type (or list hash-table))
+  (%lock nil) ; install it just-in-time, similar to hash-table-lock
+  ;; the PCL class (= CL:CLASS, but with a view to future flexibility
+  ;; we don't just call it the CLASS slot) object for this class, or
+  ;; NIL if none assigned yet
+  (pcl-class nil))
+;;; For low-level C code if it wants to display a classoid
+(defconstant classoid-name-wordindex
+  (+ 2 #-compact-instance-header 1))  ; <Header, Layout (if needed), %BITS, NAME>
+#-sb-xc-host
+(eval-when (:compile-toplevel)
+  (assert (= (+ sb-vm:instance-slots-offset (get-dsd-index classoid name))
+             classoid-name-wordindex)))
+
 ;;; The LAYOUT structure is pointed to by the first cell of instance
 ;;; (or structure) objects. It represents what we need to know for
 ;;; type checking and garbage collection. Whenever a class is
@@ -146,11 +205,9 @@
 
 ;;; 32-bit is not done yet. Three slots are still used, instead of two.
 
-#-metaspace
-(progn
-(sb-xc:defstruct (wrapper (:copier nil)
+(sb-xc:defstruct (layout (:copier nil)
                          ;; Parsing DEFSTRUCT uses a temporary layout
-                         (:constructor make-temporary-wrapper
+                         (:constructor make-temporary-layout
                              (clos-hash classoid inherits &aux (invalid nil))))
 
   ;; A packed field containing the DEPTHOID, LENGTH, and FLAGS
@@ -219,44 +276,9 @@
   #-64-bit (id-word3 0 :type word)
   #-64-bit (id-word4 0 :type word)
   #-64-bit (id-word5 0 :type word))
-(declaim (freeze-type wrapper)))
+(declaim (freeze-type layout))
 
-#+metaspace
-(progn
-;;; Separating the pointer and nonpointer slots of LAYOUT satisfies the
-;;; requirement of a certain garbage collector (WIP), but also opens up the
-;;; possibility of absorbing at least one of the vectorish slots into the
-;;; WRAPPER since it could be reallocated with variable size.
-;;; The SLOT-TABLE slot might be a good candidate for trailing tagged slots.
-(sb-xc:defstruct (wrapper (:copier nil) (:constructor %make-wrapper))
-  ;; !!! The FRIEND slot in WRAPPER *MUST* BE FIRST !!! (Wired-in assumption in genesis)
-  (friend nil :type sb-vm:layout)
-  (clos-hash (missing-arg) :type (and fixnum unsigned-byte)) ; redundant
-  (classoid (missing-arg) :type classoid)
-  (inherits #() :type simple-vector)
-  (equalp-impl #'equalp-err :type (sfunction (t t) boolean) :read-only t)
-  (slot-table #(1 nil) :type simple-vector)
-  (%info nil :type (or list defstruct-description))
-  (invalid :uninitialized :type (or cons (member nil t :uninitialized))))
-;;; See #-metaspace structure definition for remarks about each slot.
-;;; LAYOUT points to WRAPPER and vice-versa.
-;;; The most common LAYOUT is 8 words.
-;;; Needing >64 words would be quite unusual - the layout would have
-;;; an enormous depthoid or bitmap or both.
-(sb-xc:defstruct (sb-vm:layout (:copier nil)
-                         ;; Parsing DEFSTRUCT uses a temporary layout
-                         (:constructor %make-temporary-layout (friend clos-hash)))
-  ;; !!! The FRIEND slot in LAYOUT *MUST* BE FIRST !!!
-  (friend nil :type wrapper)
-  (clos-hash (missing-arg) :type (and fixnum unsigned-byte))
-  (flags 0 :type (signed-byte #.sb-vm:n-word-bits))
-  (id-word0 0 :type word)
-  (id-word1 0 :type word)
-  (id-word2 0 :type word)
-  ;; There are zero or more raw words if a type needs to store additional layout-ids,
-  ;; and there are one or more raw words for the GC bitmap.
-  )
-(declaim (freeze-type wrapper sb-vm:layout)))
+(declaim (end-block))
 
 ;;; The cross-compiler representation of a LAYOUT omits several things:
 ;;;   * BITMAP - obtainable via (DD-BITMAP (LAYOUT-INFO layout)).
@@ -269,11 +291,9 @@
 ;;; FLAGS are computed on demand, and not stored.
 #+sb-xc-host
 (progn
-  (defstruct (wrapper (:constructor host-make-wrapper
+  (defstruct (layout (:constructor host-make-layout
                          (id clos-hash classoid
-                          &key ((:info %info)) depthoid inherits length invalid
-                          #+metaspace friend)))
-    #+metaspace (friend)
+                          &key ((:info %info)) depthoid inherits length invalid)))
     (id nil :type (or null fixnum))
     ;; Cross-compiler-only translation from slot index to symbol naming
     ;; the accessor to call. (Since access by position is not a thing)
@@ -289,12 +309,10 @@
     (depthoid -1 :type layout-depthoid)
     (length 0 :type layout-length)
     (%info nil :type (or null defstruct-description)))
-  #+metaspace (defstruct (sb-vm:layout (:constructor %make-layout))
-                friend)
-  (defun make-temporary-wrapper (clos-hash classoid inherits)
-    (host-make-wrapper nil clos-hash classoid :inherits inherits :invalid nil))
-  (defun wrapper-flags (wrapper)
-    (declare (type wrapper wrapper))
+  (defun make-temporary-layout (clos-hash classoid inherits)
+    (host-make-layout nil clos-hash classoid :inherits inherits :invalid nil))
+  (defun layout-flags (layout)
+    (declare (type layout layout))
     (let ((mapping `((structure-object  ,+structure-layout-flag+)
                      (standard-object   ,+pcl-object-layout-flag+)
                      (pathname          ,+pathname-layout-flag+)
@@ -304,10 +322,10 @@
                      (stream            ,+stream-layout-flag+)
                      (sequence          ,+sequence-layout-flag+)))
           (flags 0))
-      (dolist (x (cons wrapper (coerce (wrapper-inherits wrapper) 'list)))
-        (let ((cell (assoc (wrapper-classoid-name x) mapping)))
+      (dolist (x (cons layout (coerce (layout-inherits layout) 'list)))
+        (let ((cell (assoc (layout-classoid-name x) mapping)))
           (when cell (setq flags (logior flags (second cell))))))
-      (let ((dd (wrapper-%info wrapper)))
+      (let ((dd (layout-%info layout)))
         (when (or (logtest flags (logior +pathname-layout-flag+ +condition-layout-flag+))
                   (and (logtest flags +structure-layout-flag+)
                        dd
@@ -317,37 +335,36 @@
       ;; any more complicated than necessary. It is unable to express that
       ;; these funcallable instances can go on pure boxed pages.
       ;; (The trampoline is always an assembler routine, thus ignorable)
-      (when (member (wrapper-classoid-name wrapper)
+      (when (member (layout-classoid-name layout)
                     '(sb-pcl::ctor sb-pcl::%method-function))
         (setf flags (logior flags +strictly-boxed-flag+)))
       flags))
-  (defun wrapper-bitmap (wrapper)
-    (acond ((wrapper-%info wrapper) (dd-bitmap it))
+  (defun layout-bitmap (layout)
+    (acond ((layout-%info layout) (dd-bitmap it))
            ;; Give T a 0 bitmap. It's arbitrary, but when we need some layout
            ;; that has this bitmap we can use the layout of T.
-           ((eq wrapper (find-layout t)) 0)
+           ((eq layout (find-layout t)) 0)
            (t
-            +layout-all-tagged+))))
+            +layout-all-tagged+)))
+  (defun %layout-bitmap (layout) (layout-bitmap layout))
+) ; end PROGN #+sb-xc-host
 
 (defun equalp-err (a b)
   (bug "EQUALP ~S ~S" a b))
 
-(defmacro name->dd (name)
-  ;; This wants to be a toplevel macrolet but can't be, because the body of
-  ;; the macro (which is run in the host) wouldn't see NAME->DD as a macro
-  ;; when expanding for the target. And it can't be a toplevel FLET because
-  ;; that would demote the two using macros from toplevel.
-  `(find-defstruct-description (cond #-metaspace ((eq ,name 'sb-vm:layout) 'wrapper)
-                                     (t ,name))))
-(defmacro type-dd-length (type-name) (dd-length (name->dd type-name)))
 (defmacro get-dsd-index (type-name slot-name)
   (declare (notinline dsd-index)) ; avoid later inlining failure style-warning
-  (dsd-index (find slot-name (dd-slots (name->dd type-name)) :key #'dsd-name)))
+  (dsd-index (find slot-name
+                   (dd-slots (find-defstruct-description type-name))
+                   :key #'dsd-name)))
 
 ;;; Applicable only if bit-packed (for 64-bit architectures)
 (defmacro pack-layout-flags (depthoid length flags)
   `(logior (ash ,depthoid (+ 32 sb-vm:n-fixnum-tag-bits)) (ash ,length 16) ,flags))
 
+(defmacro type-dd-length (type-name)
+  (dd-length (find-defstruct-description type-name)))
+
 (defconstant layout-id-vector-fixed-capacity 7)
 (defmacro calculate-extra-id-words (depthoid)
   ;; There are 1 or 2 ids per word depending on n-word-bytes.
@@ -357,34 +374,48 @@
   `(ceiling (max 0 (- ,depthoid ,layout-id-vector-fixed-capacity))
             ,(/ sb-vm:n-word-bytes 4)))
 
-(declaim (inline wrapper-info wrapper-dd))
-(defun wrapper-info (wrapper)
-  (let ((info (wrapper-%info wrapper))) (unless (listp info) info)))
-(defun (setf wrapper-info) (newval wrapper)
+(declaim (inline layout-dd layout-info))
+;; Use LAYOUT-DD to read LAYOUT-INFO if you want to assert that it is non-nil.
+(defun layout-dd (layout)
+  (the defstruct-description (layout-%info layout)))
+(defun layout-info (layout)
+  (let ((info (layout-%info layout)))
+    (unless (listp info) info)))
+(defun (setf layout-info) (newval layout)
   ;; The current value must be nil or a defstruct-description,
   ;; otherwise we'd clobber a non-nil slot list.
-  (aver (not (consp (wrapper-%info wrapper))))
-  (setf (wrapper-%info wrapper) newval))
-;; Use WRAPPER-DD to read WRAPPER-INFO and assert that it is non-nil.
-(defun wrapper-dd (wrapper)
-  (the defstruct-description (wrapper-%info wrapper)))
+  (aver (not (consp (layout-%info layout))))
+  (setf (layout-%info layout) newval))
 
 ;;; See the pictures above DD-BITMAP in src/code/defstruct for the details.
 (defconstant standard-gf-primitive-obj-layout-bitmap
-  #+compact-instance-header  6
+  ;; layout has no bitmap bit. The other slots all look tagged even though
+  ;; it's an accident that the raw instruction words resemble fixnums.
+  ;; I should probably find a more principled way to express this,
+  ;; but the old - technically correct - way based on a bitmap really made
+  ;; the logic look like crap. But now that the variable-length part of a
+  ;; funcallable-instance follows its embedded trampoline, it would be simple
+  ;; to alter the GC to skip a fixed number of slots.
+  ;; I think that is preferable to using a bitmap.
+  #+compact-instance-header -1
+  ;; the layout is considered untagged so that we don't doubly process it
+  ;; if walking tagged slots. The assumption is that layout is always
+  ;; handled separately, so therefore we don't want to process it again
+  ;; in a bitmap-based scan. And the entry point is considered untagged
+  ;; because it points to an assembler routine, thus ignorable.
   #-compact-instance-header -4)
 
 #-sb-xc-host
 (progn
 (declaim (inline bitmap-start bitmap-nwords bitmap-all-taggedp))
 (defun bitmap-start (layout)
-  (+ (type-dd-length sb-vm:layout)
+  (+ (type-dd-length layout)
      (calculate-extra-id-words (layout-depthoid layout))))
 (defun bitmap-nwords (layout)
-  (declare (sb-vm:layout layout))
+  (declare (layout layout))
   (- (%instance-length layout)
      (calculate-extra-id-words (layout-depthoid layout))
-     (type-dd-length sb-vm:layout)))
+     (type-dd-length layout)))
 (defun bitmap-all-taggedp (layout)
   ;; All bitmaps have at least 1 word; read that first.
   (and (= (%raw-instance-ref/signed-word layout (bitmap-start layout))
@@ -392,100 +423,62 @@
        ;; Then check that there are no additional words.
        (= (bitmap-nwords layout) 1)))
 
-#+metaspace ; If metaspace, then WRAPPER has no flags; they're in the LAYOUT.
-(defmacro wrapper-flags (x) `(layout-flags (wrapper-friend ,x)))
+;;; Extract the bitmap from 1 or more words of bits that have the same format
+;;; as a BIGNUM - least-significant word first, native bit order within each word,
+;;; all but the last are unsigned, and the last is signed.
+(defun %layout-bitmap (layout)
+  (declare (type layout layout))
+  (do ((accumulator 0)
+       (shift 0)
+       (index (bitmap-start layout) (1+ index))
+       (last (1- (%instance-length layout))))
+      ((> index last) accumulator)
+    (setf accumulator (logior accumulator
+                              (ash (if (= index last)
+                                       (%raw-instance-ref/signed-word layout index)
+                                       (%raw-instance-ref/word layout index))
+                                   shift)))
+    (incf shift sb-vm:n-word-bits)))
 
-;; 32-bit has the depthoid as a slot, 64-bit has it is part of FLAGS which are
-;; in the LAYOUT
-#+64-bit
-(defun wrapper-depthoid (wrapper) (layout-depthoid (wrapper-friend wrapper)))
-
-(defun wrapper-bitmap (wrapper)
-  (declare (type wrapper wrapper))
-  (acond ((wrapper-info wrapper) (dd-bitmap it))
+(defun layout-bitmap (layout)
+  (declare (type layout layout))
+  ;; Whenever we call LAYOUT-BITMAP on a structure-object subtype,
+  ;; it's supposed to have the INFO slot populated, linking it to a defstruct-description.
+  (acond ((layout-info layout) (dd-bitmap it))
          ;; Instances lacking DD-INFO are CLOS objects, which can't generally have
          ;; raw slots, except that funcallable-instances may have 2 raw slots -
          ;; the trampoline and the layout. The trampoline can have a tag, depending
          ;; on the platform, and the layout is tagged but a special case.
          ;; In any event, the bitmap is always 1 word, and there are no "extra ID"
          ;; words preceding it.
-         (t (the fixnum
-                 (%raw-instance-ref/signed-word (wrapper-friend wrapper)
-                                                (type-dd-length sb-vm:layout))))))
+         (t
+          (aver (not (logtest +structure-layout-flag+ (layout-flags layout))))
+          (the fixnum
+               (%raw-instance-ref/signed-word layout
+                                              (type-dd-length layout))))))
 #+64-bit
-(defmacro wrapper-length (wrapper) ; SETFable
-  `(ldb (byte 16 16) (layout-flags (wrapper-friend ,wrapper))))
+(defmacro layout-length (layout) ; SETFable
+  `(ldb (byte 16 16) (layout-flags ,layout)))
 
-) ; end PROGN
+) ; end PROGN #-sb-xc-host
 
 ;;; True of STANDARD-OBJECT, which include generic functions.
 (declaim (inline layout-for-pcl-obj-p))
-(defun layout-for-pcl-obj-p (wrapper)
-  (declare (type wrapper wrapper))
-  (logtest (wrapper-flags wrapper) +pcl-object-layout-flag+))
+(defun layout-for-pcl-obj-p (layout)
+  (declare (type layout layout))
+  (logtest (layout-flags layout) +pcl-object-layout-flag+))
 
-;;; The CLASSOID structure is a supertype of all classoid types.  A
-;;; CLASSOID is also a CTYPE structure as recognized by the type
-;;; system.  (FIXME: It's also a type specifier, though this might go
-;;; away as with the merger of SB-PCL:CLASS and CL:CLASS it's no
-;;; longer necessary)
-(def!struct (classoid
-             (:include ctype)
-             (:constructor nil)
-             (:copier nil)
-             (:print-object
-              (lambda (class stream)
-                (let ((name (classoid-name class)))
-                  (print-unreadable-object (class stream
-                                                  :type t
-                                                  :identity (not name))
-                    (format stream
-                            ;; FIXME: Make sure that this prints
-                            ;; reasonably for anonymous classes.
-                            "~:[anonymous~;~:*~S~]~@[ (~(~A~))~]"
-                            name
-                            (classoid-state class))))))
-             #-sb-xc-host (:pure nil))
-  ;; the value to be returned by CLASSOID-NAME.
-  (name nil :type symbol)
-  ;; the current WRAPPER for this class, or NIL if none assigned yet
-  (wrapper nil :type (or null wrapper))
-  ;; How sure are we that this class won't be redefined?
-  ;;   :READ-ONLY = We are committed to not changing the effective
-  ;;                slots or superclasses.
-  ;;   :SEALED    = We can't even add subclasses.
-  ;;   NIL        = Anything could happen.
-  (state nil :type (member nil :read-only :sealed))
-  ;; direct superclasses of this class. Always NIL for CLOS classes.
-  (direct-superclasses () :type list)
-  ;; Definition location
-  ;; Not used for standard-classoid, because pcl has its own mechanism.
-  (source-location nil)
-  ;; representation of all of the subclasses (direct or indirect) of
-  ;; this class. This is NIL if no subclasses or not initalized yet;
-  ;; otherwise, it's an EQ hash-table mapping CLASSOID objects to the
-  ;; subclass layout that was in effect at the time the subclass was
-  ;; created.
-  ;; Initially an alist, and changed to a hash-table at some threshold.
-  (subclasses nil :type (or list hash-table))
-  (%lock nil) ; install it just-in-time, similar to hash-table-lock
-  ;; the PCL class (= CL:CLASS, but with a view to future flexibility
-  ;; we don't just call it the CLASS slot) object for this class, or
-  ;; NIL if none assigned yet
-  (pcl-class nil))
-
-(defun wrapper-classoid-name (x)
-  (classoid-name (wrapper-classoid x)))
+(defun layout-classoid-name (x)
+  (classoid-name (layout-classoid x)))
 
 ;;;; object types to represent classes
 
 ;;; An UNDEFINED-CLASSOID is a cookie we make up to stick in forward
 ;;; referenced layouts. Users should never see them.
-(def!struct (undefined-classoid
-             (:include classoid)
-             (:copier nil)
-             (:constructor make-undefined-classoid
-                 (name &aux (%bits (pack-ctype-bits classoid name))))))
+(defstruct (undefined-classoid
+            (:include classoid)
+            (:constructor !alloc-undefined-classoid (%bits name))
+            (:copier nil)))
 
 ;;; BUILT-IN-CLASS is used to represent the standard classes that
 ;;; aren't defined with DEFSTRUCT and other specially implemented
@@ -497,7 +490,7 @@
 ;;; This translation is done when type specifiers are parsed. Type
 ;;; system operations (union, subtypep, etc.) should never encounter
 ;;; translated classes, only their translation.
-(def!struct (built-in-classoid (:include classoid) (:copier nil)
+(defstruct (built-in-classoid (:include classoid) (:copier nil)
                                     (:constructor !make-built-in-classoid))
   ;; the type we translate to on parsing. If NIL, then this class
   ;; stands on its own. Only :INITIALIZING for a period during cold
@@ -505,10 +498,9 @@
   (translation nil :type (or null ctype (member :initializing)))
   (predicate (missing-arg) :type (sfunction (t) boolean) :read-only t))
 
-(def!struct (condition-classoid (:include classoid)
-                                (:copier nil)
-                                (:constructor make-condition-classoid
-                                    (&key name &aux (%bits (pack-ctype-bits classoid name)))))
+(defstruct (condition-classoid (:include classoid)
+                                (:constructor !alloc-condition-classoid (%bits name))
+                                (:copier nil))
   ;; list of CONDITION-SLOT structures for the direct slots of this
   ;; class
   (slots nil :type list)
@@ -536,17 +528,16 @@
 ;;; STRUCTURE-CLASSOID represents what we need to know about structure
 ;;; classes. Non-structure "typed" defstructs are a special case, and
 ;;; don't have a corresponding class.
-(def!struct (structure-classoid
+(defstruct (structure-classoid
              (:include classoid)
-             (:copier nil)
-             (:constructor make-structure-classoid
-                           (&key name &aux (%bits (pack-ctype-bits classoid name))))))
+             (:constructor !alloc-structure-classoid (%bits name))
+             (:copier nil)))
 
 ;;;; classoid namespace
 
 ;;; We use an indirection to allow forward referencing of class
 ;;; definitions with load-time resolution.
-(def!struct (classoid-cell
+(defstruct (classoid-cell
              (:copier nil)
              (:constructor make-classoid-cell (name &optional classoid))
              (:print-object (lambda (s stream)
@@ -570,18 +561,32 @@
 ;;; STANDARD-CLASS and FUNCALLABLE-STANDARD-CLASS.  The type system
 ;;; side does not need to distinguish between STANDARD-CLASS and
 ;;; FUNCALLABLE-STANDARD-CLASS.
-(def!struct (standard-classoid (:include classoid)
-                               (:copier nil)
-                               (:constructor make-standard-classoid
-                                   (&key name pcl-class
-                                    &aux (%bits (pack-ctype-bits classoid name)))))
+(defstruct (standard-classoid
+             (:include classoid)
+             (:constructor !alloc-standard-classoid (%bits name pcl-class))
+             (:copier nil))
   old-layouts)
 ;;; a metaclass for classes which aren't standardlike but will never
 ;;; change either.
-(def!struct (static-classoid (:include classoid)
-                             (:copier nil)
-                             (:constructor make-static-classoid
-                                 (&key name &aux (%bits (pack-ctype-bits classoid name))))))
+(defstruct (static-classoid (:include classoid)
+                             (:constructor !alloc-static-classoid (%bits name))
+                             (:copier nil)))
 
 (declaim (freeze-type built-in-classoid condition-classoid
                       standard-classoid static-classoid))
+
+;;; Return the name of the global hashset that OBJ (a CTYPE instance)
+;;; would be stored in, if it were stored in one.
+;;; This is only for bootstrap, and not 100% precise as it does not know
+;;; about the other MEMBER type containers.
+(defun ctype->hashset-sym (obj)
+  (macrolet ((generate  ()
+               (collect ((clauses))
+                 (dolist (type-class *type-class-list*
+                                     `(etypecase obj ,@(clauses)))
+                   (dolist (instance-type (cdr type-class))
+                     (clauses
+                      (list instance-type
+                            (unless (member instance-type '(classoid named-type))
+                              `',(symbolicate "*" instance-type "-HASHSET*")))))))))
+    (generate)))
