@@ -28,7 +28,7 @@
 ;;; Then we can implement XSET in terms of this hashset.
 
 (defstruct (robinhood-hashset-storage
-            (:constructor make-hashset-storage (cells psl-vector hash-vector hv-inexact))
+            (:constructor !make-hs-storage (cells psl-vector hash-vector hv-inexact))
             (:conc-name hss-)
             (:copier nil)
             (:predicate nil))
@@ -53,7 +53,9 @@
   ;; It _doesn't_ mean that FIND returns the right answer if writes occur while reading.
   ;; It _does_ mean that the algorithm won't crash.
   (storage (missing-arg) :type robinhood-hashset-storage)
-  (hash-function #'error :type (sfunction (t) fixnum))
+  ;; need to allow maximum number of bits in either host or target fixnum
+  ;; because the code runs on either. Will be efficient in the target at least.
+  (hash-function #'error :type (sfunction (t) (or fixnum sb-xc:fixnum)))
   (test-function #'error :type function)
   #+hashset-metrics (count-find-hits 0 :type sb-vm:word)
   #+hashset-metrics (count-finds 0 :type sb-vm:word)
@@ -79,8 +81,9 @@
 (defmacro hs-chain-terminator-p (val) `(eq ,val 0))
 
 (defun allocate-hashset-storage (capacity weakp)
+  (declare (type (unsigned-byte 28) capacity)) ; 256M cells maximum
   (declare (ignorable weakp))
-  (declare (sb-c::tlab :system))
+  (declare (sb-c::tlab :system) (inline !make-hs-storage))
   (let* ((len (+ capacity hs-storage-trailer-cells))
          (cells
           #+sb-xc-host (make-array len :initial-element 0)
@@ -102,7 +105,7 @@
     ;; I might instead want the hash-vector's type to
     ;;   (OR (SIMPLE-ARRAY (UNSIGNED-BYTE 16) (*)) (SIMPLE-ARRAY (UNSIGNED-BYTE 32) (*)))
     ;; but I'm not going there yet.
-    (make-hashset-storage cells psl-vector hash-vector (> capacity 65536))))
+    (!make-hs-storage cells psl-vector hash-vector (> capacity 65536))))
 
 (defun hashset-weakp (hashset)
   #+sb-xc-host (declare (ignore hashset))
@@ -117,7 +120,7 @@
          (storage (allocate-hashset-storage capacity weakness)))
     (make-robinhood-hashset :hash-function hash-function
                             :test-function test-function
-                            :mutex (if synchronized #-sb-xc-host (sb-thread:make-mutex)
+                            :mutex (if synchronized #-sb-xc-host (sb-thread:make-mutex :name "hashset")
                                                     #+sb-xc-host nil)
                             :storage storage)))
 
@@ -412,6 +415,20 @@
             (or (hashset-find hashset key)
                 (hashset-insert hashset (funcall copier key)))))))
 
+(defun map-hashset (function hashset)
+  (declare (dynamic-extent function))
+  (macrolet ((dovect ()
+               `(let ((vec (hss-cells (hashset-storage hashset))))
+                  (do ((index 0 (1+ index))
+                       (length (- (length vec) hs-storage-trailer-cells)))
+                      ((>= index length))
+                    (let ((elt (svref vec index)))
+                      (unless (or (null elt) (eql elt 0))
+                        (funcall function elt)))))))
+    (if (hashset-mutex hashset)
+        (with-system-mutex ((hashset-mutex hashset)) (dovect))
+        (dovect))))
+
 (defun hashset-count (hashset &aux (cells (hss-cells (hashset-storage hashset))))
   (count-if (lambda (x) (and x (not (hs-chain-terminator-p x))))
             cells :end (hs-cells-capacity cells)))
