@@ -307,7 +307,7 @@
          :operation '/
          :operands (list this that)))
 
-(defun restart-type-error (type condition pc-offset)
+(defun restart-type-error (type condition &optional pc-offset)
   (let ((tn-offset (car *current-internal-error-args*)))
     (labels ((retry-value (value)
                (if (typep value type)
@@ -320,8 +320,9 @@
                (sb-di::sub-set-debug-var-slot
                 nil tn-offset (retry-value value)
                 *current-internal-error-context*)
-               (sb-vm::incf-context-pc *current-internal-error-context*
-                                       pc-offset)
+               (when pc-offset
+                 (sb-vm::incf-context-pc *current-internal-error-context*
+                                         pc-offset))
                (return-from restart-type-error))
              (try (condition)
                (restart-case (error condition)
@@ -335,12 +336,15 @@
 (defun object-not-type-error (object type &optional (context nil context-p))
   (if (invalid-array-p object)
       (invalid-array-error object)
-      (let* ((context (if context-p
-                          context
-                          (sb-di:error-context)))
+      (let* ((context (cond (context-p
+                             context)
+                            ((= *current-internal-trap-number* sb-vm:cerror-trap)
+                             'cerror)
+                            (t
+                             (sb-di:error-context))))
              (condition
                (make-condition (if (and (%instancep object)
-                                        (wrapper-invalid (%instance-wrapper object)))
+                                        (layout-invalid (%instance-layout object)))
                                    ;; Signaling LAYOUT-INVALID is dubious, but I guess it provides slightly
                                    ;; more information in that it says that the object may have at some point
                                    ;; been TYPE. Anyway, it's not wrong - it's a subtype of TYPE-ERROR.
@@ -350,15 +354,19 @@
                                :expected-type (typecase type
                                                 (classoid-cell
                                                  (classoid-cell-name type))
-                                                (wrapper
-                                                 (wrapper-proper-name type))
+                                                (layout
+                                                 (layout-proper-name type))
                                                 (t
                                                  type))
                                :context (and (not (integerp context))
+                                             (not (eq context 'cerror))
                                              context))))
-        (if (integerp context)
-            (restart-type-error type condition context)
-            (error condition)))))
+        (cond ((integerp context)
+               (restart-type-error type condition context))
+              ((eq context 'cerror)
+               (restart-type-error type condition))
+              (t
+               (error condition))))))
 
 (macrolet ((def (errname fun-name)
              `(setf (svref **internal-error-handlers**
@@ -434,10 +442,7 @@
     (declare (ignorable raw-low raw-high))
     (let ((type (or (sb-di:error-context)
                     'fixnum)))
-      (object-not-type-error #+x86-64
-                             (* low high)
-                             #-x86-64
-                             (if (memq (sb-c:sc+offset-scn raw-low) `(,sb-vm:any-reg-sc-number
+      (object-not-type-error (if (memq (sb-c:sc+offset-scn raw-low) `(,sb-vm:any-reg-sc-number
                                                                       ,sb-vm:descriptor-reg-sc-number))
                                  (ash (logior
                                        (ash high sb-vm:n-word-bits)
@@ -449,7 +454,7 @@
                              type
                              nil))))
 
-(sb-c::when-vop-existsp (:translate sb-c::unsigned+)
+(sb-c::when-vop-existsp (:translate overflow+)
   (flet ((err (x of cf)
            (let* ((raw-x (car *current-internal-error-args*))
                   (signed (= (sb-c:sc+offset-scn raw-x) sb-vm:signed-reg-sc-number)))
@@ -461,7 +466,7 @@
                                 (of
                                  (ldb (byte sb-vm:n-word-bits 0) x))
                                 (t
-                                 (bug "flags")))
+                                 x))
                           (cond (cf
                                  (dpb 1 (byte 1 sb-vm:n-word-bits) x))
                                 (of
@@ -495,10 +500,26 @@
                                        (dpb 1 (byte 1 sb-vm:n-word-bits) x))))
                              type nil)))
 
+  (deferr add-overflow2-error (x y)
+    (let ((type (or (sb-di:error-context)
+                    'fixnum)))
+      (if (numberp x)
+          (object-not-type-error (+ x y) type nil)
+          (object-not-type-error x 'number nil))))
+
   (deferr sub-overflow2-error (x y)
     (let ((type (or (sb-di:error-context)
                     'fixnum)))
-      (object-not-type-error (- x y) type nil))))
+      (if (numberp x)
+          (object-not-type-error (- x y) type nil)
+          (object-not-type-error x 'number nil))))
+
+  (deferr mul-overflow2-error (x y)
+    (let ((type (or (sb-di:error-context)
+                    'fixnum)))
+      (if (numberp x)
+          (object-not-type-error (* x y) type nil)
+          (object-not-type-error x 'number nil)))))
 
 ;;;; INTERNAL-ERROR signal handler
 
