@@ -62,6 +62,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;; duplicate defglobal because this file is compiled before "reader"
 (define-load-time-global *standard-readtable* nil)
 
+(define-load-time-global sb-pretty::*standard-pprint-dispatch-table* nil)
 (defun %with-standard-io-syntax (function)
   (declare (type function function))
   (declare (dynamic-extent function))
@@ -193,8 +194,7 @@ variable: an unreadable object representing the error is printed instead.")
          (and *print-pretty* (pprint-dispatch object))
        (if pretty
            (%with-output-to-string (stream)
-              (sb-pretty::with-pretty-stream (stream)
-                (funcall fun stream object)))
+              (sb-pretty:output-pretty-object stream fun object))
            (let ((buffer-size (approx-chars-in-repr object)))
              (let* ((string (make-string buffer-size :element-type 'base-char))
                     (stream (%make-finite-base-string-output-stream string)))
@@ -284,6 +284,208 @@ variable: an unreadable object representing the error is printed instead.")
                (print-description)
                (write-char #\> stream)))))
   nil)
+
+
+;;;; circularity detection stuff
+
+;;; When *PRINT-CIRCLE* is T, this gets bound to a hash table that
+;;; (eventually) ends up with entries for every object printed. When
+;;; we are initially looking for circularities, we enter a T when we
+;;; find an object for the first time, and a 0 when we encounter an
+;;; object a second time around. When we are actually printing, the 0
+;;; entries get changed to the actual marker value when they are first
+;;; printed.
+(defvar *circularity-hash-table* nil)
+
+;;; When NIL, we are just looking for circularities. After we have
+;;; found them all, this gets bound to 0. Then whenever we need a new
+;;; marker, it is incremented.
+(defvar *circularity-counter* nil)
+
+;;; Check to see whether OBJECT is a circular reference, and return
+;;; something non-NIL if it is. If ASSIGN is true, reference
+;;; bookkeeping will only be done for existing entries, no new
+;;; references will be recorded. If ASSIGN is true, then the number to
+;;; use in the #n= and #n# noise is assigned at this time.
+;;;
+;;; Note: CHECK-FOR-CIRCULARITY must be called *exactly* once with
+;;; ASSIGN true, or the circularity detection noise will get confused
+;;; about when to use #n= and when to use #n#. If this returns non-NIL
+;;; when ASSIGN is true, then you must call HANDLE-CIRCULARITY on it.
+;;; If CHECK-FOR-CIRCULARITY returns :INITIATE as the second value,
+;;; you need to initiate the circularity detection noise, e.g. bind
+;;; *CIRCULARITY-HASH-TABLE* and *CIRCULARITY-COUNTER* to suitable values
+;;; (see #'OUTPUT-OBJECT for an example).
+;;;
+;;; Circularity detection is done in two places, OUTPUT-OBJECT and
+;;; WITH-CIRCULARITY-DETECTION (which is used from PPRINT-LOGICAL-BLOCK).
+;;; These checks aren't really redundant (at least I can't really see
+;;; a clean way of getting by with the checks in only one of the places).
+;;; This causes problems when mixed with pprint-dispatching; an object is
+;;; marked as visited in OUTPUT-OBJECT, dispatched to a pretty printer
+;;; that uses PPRINT-LOGICAL-BLOCK (directly or indirectly), leading to
+;;; output like #1=#1#. The MODE parameter is used for detecting and
+;;; correcting this problem.
+(defun check-for-circularity (object &optional assign (mode t))
+  (when (null *print-circle*)
+         ;; Don't bother, nobody cares.
+    (return-from check-for-circularity nil))
+  (let ((circularity-hash-table *circularity-hash-table*))
+    (cond
+        ((null circularity-hash-table)
+          (values nil :initiate))
+        ((null *circularity-counter*)
+         (ecase (gethash object circularity-hash-table)
+           ((nil)
+            ;; first encounter
+            (setf (gethash object circularity-hash-table) mode)
+            ;; We need to keep looking.
+            nil)
+           ((:logical-block)
+            (setf (gethash object circularity-hash-table)
+                  :logical-block-circular)
+            t)
+           ((t)
+            (cond ((eq mode :logical-block)
+                   ;; We've seen the object before in output-object, and now
+                   ;; a second time in a PPRINT-LOGICAL-BLOCK (for example
+                   ;; via pprint-dispatch). Don't mark it as circular yet.
+                   (setf (gethash object circularity-hash-table)
+                         :logical-block)
+                   nil)
+                  (t
+                   ;; second encounter
+                   (setf (gethash object circularity-hash-table) 0)
+                   ;; It's a circular reference.
+                   t)))
+           ((0 :logical-block-circular)
+            ;; It's a circular reference.
+            t)))
+        (t
+         (let ((value (gethash object circularity-hash-table)))
+           (case value
+             ((nil t :logical-block)
+              ;; If NIL, we found an object that wasn't there the
+              ;; first time around. If T or :LOGICAL-BLOCK, this
+              ;; object appears exactly once. Either way, just print
+              ;; the thing without any special processing. Note: you
+              ;; might argue that finding a new object means that
+              ;; something is broken, but this can happen. If someone
+              ;; uses the ~@<...~:> format directive, it conses a new
+              ;; list each time though format (i.e. the &REST list),
+              ;; so we will have different cdrs.
+              nil)
+             ;; A circular reference to something that will be printed
+             ;; as a logical block. Wait until we're called from
+             ;; PPRINT-LOGICAL-BLOCK with ASSIGN true before assigning the
+             ;; number.
+             ;;
+             ;; If mode is :LOGICAL-BLOCK and assign is false, return true
+             ;; to indicate that this object is circular, but don't assign
+             ;; it a number yet. This is necessary for cases like
+             ;; #1=(#2=(#2# . #3=(#1# . #3#))))).
+             (:logical-block-circular
+              (cond ((and (not assign)
+                          (eq mode :logical-block))
+                     t)
+                    ((and assign
+                          (eq mode :logical-block))
+                     (let ((value (incf *circularity-counter*)))
+                       ;; first occurrence of this object: Set the counter.
+                       (setf (gethash object circularity-hash-table) value)
+                       value))
+                    (t
+                     nil)))
+             (0
+              (if (eq assign t)
+                  (let ((value (incf *circularity-counter*)))
+                    ;; first occurrence of this object: Set the counter.
+                    (setf (gethash object circularity-hash-table) value)
+                    value)
+                  t))
+             (t
+              ;; second or later occurrence
+              (- value))))))))
+
+;;; Handle the results of CHECK-FOR-CIRCULARITY. If this returns T then
+;;; you should go ahead and print the object. If it returns NIL, then
+;;; you should blow it off.
+(defun handle-circularity (marker stream)
+  (case marker
+    (:initiate
+     ;; Someone forgot to initiate circularity detection.
+     (let ((*print-circle* nil))
+       (error "trying to use CHECK-FOR-CIRCULARITY when ~
+               circularity checking isn't initiated")))
+    ((t :logical-block)
+     ;; It's a second (or later) reference to the object while we are
+     ;; just looking. So don't bother groveling it again.
+     nil)
+    (t
+     (write-char #\# stream)
+     (output-integer (abs marker) stream 10 nil)
+     (cond ((minusp marker)
+            (write-char #\# stream)
+            nil)
+           (t
+            (write-char #\= stream)
+            t)))))
+
+(defmacro with-circularity-detection ((object stream) &body body)
+  (with-unique-names (marker body-name)
+    `(labels ((,body-name ()
+                ,@body))
+       (cond ((or (not *print-circle*)
+                  (uniquely-identified-by-print-p ,object))
+              (,body-name))
+             (*circularity-hash-table*
+              (let ((,marker (check-for-circularity ,object t :logical-block)))
+                (if ,marker
+                    (when (handle-circularity ,marker ,stream)
+                      (,body-name))
+                    (,body-name))))
+             (t
+              (let ((*circularity-hash-table* (make-hash-table :test 'eq)))
+                (output-object ,object *null-broadcast-stream*)
+                (let ((*circularity-counter* 0))
+                  (let ((,marker (check-for-circularity ,object t
+                                                        :logical-block)))
+                    (when ,marker
+                      (handle-circularity ,marker ,stream)))
+                  (,body-name))))))))
+
+;;;; level and length abbreviations
+
+;;; The current level we are printing at, to be compared against
+;;; *PRINT-LEVEL*. See the macro DESCEND-INTO for a handy interface to
+;;; depth abbreviation.
+(defvar *current-level-in-print* 0)
+(declaim (index *current-level-in-print*))
+
+;;; Automatically handle *PRINT-LEVEL* abbreviation. If we are too
+;;; deep, then a #\# is printed to STREAM and BODY is ignored.
+(defmacro descend-into ((stream) &body body)
+  (let ((flet-name (gensym "DESCEND")))
+    `(flet ((,flet-name ()
+              ,@body))
+       (cond ((and (null *print-readably*)
+                   (let ((level *print-level*))
+                     (and level (>= *current-level-in-print* level))))
+              (write-char #\# ,stream))
+             (t
+              (let ((*current-level-in-print* (1+ *current-level-in-print*)))
+                (,flet-name)))))))
+
+;;; Punt if INDEX is equal or larger then *PRINT-LENGTH* (and
+;;; *PRINT-READABLY* is NIL) by outputting \"...\" and returning from
+;;; the block named NIL.
+(defmacro punt-print-if-too-long (index stream)
+  `(when (and (not *print-readably*)
+              (let ((len *print-length*))
+                (and len (>= ,index len))))
+     (write-string "..." ,stream)
+     (return)))
+
 
 ;;;; OUTPUT-OBJECT -- the main entry point
 
@@ -308,8 +510,7 @@ variable: an unreadable object representing the error is printed instead.")
              (multiple-value-bind (fun pretty)
                  (and *print-pretty* (pprint-dispatch object))
                (if pretty
-                   (sb-pretty::with-pretty-stream (stream)
-                     (funcall fun stream object))
+                   (sb-pretty:output-pretty-object stream fun object)
                    (output-ugly-object stream object))))
            (handle-it (stream)
              (if *suppress-print-errors*
@@ -377,12 +578,11 @@ variable: an unreadable object representing the error is printed instead.")
         (return-from output-ugly-object
           (print-unreadable-object (object stream :identity t)
             (prin1 'instance stream))))
-      (let* ((wrapper (layout-friend layout))
-             (classoid (wrapper-classoid wrapper)))
+      (let ((classoid (layout-classoid layout)))
         ;; Additionally, don't crash if the object is an obsolete thing with
         ;; no update protocol.
         (when (or (sb-kernel::undefined-classoid-p classoid)
-                  (and (wrapper-invalid wrapper)
+                  (and (layout-invalid layout)
                        (logtest (layout-flags layout)
                                 (logior +structure-layout-flag+
                                         +condition-layout-flag+))))
@@ -405,14 +605,14 @@ variable: an unreadable object representing the error is printed instead.")
       (output-symbol object (sb-xc:symbol-package object) stream)
       ;; Write only the characters of the name, never the package
       (let ((rt *readtable*))
-        (funcall (truly-the function
-                  (choose-symbol-out-fun *print-case* (%readtable-case rt)))
-                 (symbol-name object) stream rt))))
+        (output-symbol-case-dispatch *print-case* (readtable-case rt)
+                                     (symbol-name object) stream rt))))
 
 (defun output-symbol (symbol package stream)
   (let* ((readably *print-readably*)
          (readtable (if readably *standard-readtable* *readtable*))
-         (out-fun (choose-symbol-out-fun *print-case* (%readtable-case readtable))))
+         (print-case *print-case*)
+         (readtable-case (readtable-case readtable)))
     (flet ((output-token (name)
              (declare (type simple-string name))
              (cond ((or (and (readtable-normalization readtable)
@@ -431,7 +631,8 @@ variable: an unreadable object representing the error is printed instead.")
                         (write-char char stream)))
                     (write-char #\| stream))
                    (t
-                    (funcall (truly-the function out-fun) name stream readtable)))))
+                    (output-symbol-case-dispatch print-case readtable-case
+                                                 name stream readtable)))))
       (let ((name (symbol-name symbol))
             (current (sane-package)))
         (cond
@@ -458,7 +659,7 @@ variable: an unreadable object representing the error is printed instead.")
             (unless (and accessible (eq found symbol))
               (output-token (or (package-local-nickname package current)
                                 (package-name package)))
-              (write-string (if (eql (find-external-symbol name package) 0) "::" ":")
+              (write-string (if (symbol-externalp symbol package) ":" "::")
                             stream)))))
         (output-token name)))))
 
@@ -497,7 +698,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; For each character, the value of the corresponding element is the
 ;;; lowest base in which that character is a digit.
 (defconstant-eqx +digit-bases+
-  #.(let ((a (sb-xc:make-array 128 ; FIXME
+  #.(let ((a (sb-xc:make-array base-char-code-limit
                                :retain-specialization-for-after-xc-core t
                                :element-type '(unsigned-byte 8)
                                :initial-element 36)))
@@ -576,13 +777,13 @@ variable: an unreadable object representing the error is printed instead.")
                      (< (the fixnum (aref bases code)) base))))
 
     (prog ((len (length name))
-           (attributes +character-attributes+)
-           (bases +digit-bases+)
+           (attributes #.+character-attributes+)
+           (bases #.+digit-bases+)
            (base *print-base*)
            (letter-attribute
-            (case (%readtable-case readtable)
-              (#.+readtable-upcase+ uppercase-attribute)
-              (#.+readtable-downcase+ lowercase-attribute)
+            (case (readtable-case readtable)
+              (:upcase uppercase-attribute)
+              (:downcase lowercase-attribute)
               (t (logior lowercase-attribute uppercase-attribute))))
            (index 0)
            (bits 0)
@@ -706,6 +907,8 @@ variable: an unreadable object representing the error is printed instead.")
 ;;;; case hackery: One of these functions is chosen to output symbol
 ;;;; names according to the values of *PRINT-CASE* and READTABLE-CASE.
 
+(declaim (start-block output-symbol-case-dispatch))
+
 ;;; called when:
 ;;; READTABLE-CASE      *PRINT-CASE*
 ;;; :UPCASE             :UPCASE
@@ -740,7 +943,7 @@ variable: an unreadable object representing the error is printed instead.")
 (defun output-capitalize-symbol (pname stream readtable)
   (declare (simple-string pname))
   (let ((prev-not-alphanum t)
-        (up (eql (%readtable-case readtable) +readtable-upcase+)))
+        (up (eq (readtable-case readtable) :upcase)))
     (dotimes (i (length pname))
       (let ((char (char pname i)))
         (write-char (if up
@@ -771,37 +974,23 @@ variable: an unreadable object representing the error is printed instead.")
           (t
            (write-string pname stream)))))
 
-(defun choose-symbol-out-fun (print-case readtable-case)
-  (macrolet
-      ((compute-fun-vector (&aux (vector (make-array 12)))
-         ;; Pack a 2D array of functions into a simple-vector.
-         ;; Major axis is *PRINT-CASE*, minor axis is %READTABLE-CASE.
-         (dotimes (readtable-case-index 4)
-           (dotimes (print-case-index 3)
-             (let ((readtable-case
-                    (elt '(:upcase :downcase :preserve :invert) readtable-case-index))
-                   (print-case
-                    (elt '(:upcase :downcase :capitalize) print-case-index)))
-               (setf (aref vector (logior (ash print-case-index 2)
-                                          readtable-case-index))
-                     (case readtable-case
-                       (:upcase
-                        (case print-case
-                          (:upcase 'output-preserve-symbol)
-                          (:downcase 'output-lowercase-symbol)
-                          (:capitalize 'output-capitalize-symbol)))
-                       (:downcase
-                        (case print-case
-                          (:upcase 'output-uppercase-symbol)
-                          (:downcase 'output-preserve-symbol)
-                          (:capitalize 'output-capitalize-symbol)))
-                       (:preserve 'output-preserve-symbol)
-                       (:invert 'output-invert-symbol))))))
-         `(load-time-value (vector ,@(map 'list (lambda (x) `(function ,x)) vector))
-                           t)))
-    (aref (compute-fun-vector)
-          (logior (case print-case (:upcase 0) (:downcase 4) (t 8))
-                  (truly-the (mod 4) readtable-case)))))
+;;; Call an output function based on PRINT-CASE and READTABLE-CASE.
+(defun output-symbol-case-dispatch (print-case readtable-case name stream readtable)
+  (ecase readtable-case
+    (:upcase
+     (ecase print-case
+       (:upcase (output-preserve-symbol name stream readtable))
+       (:downcase (output-lowercase-symbol name stream readtable))
+       (:capitalize (output-capitalize-symbol name stream readtable))))
+    (:downcase
+     (ecase print-case
+       (:upcase (output-uppercase-symbol name stream readtable))
+       (:downcase (output-preserve-symbol name stream readtable))
+       (:capitalize (output-capitalize-symbol name stream readtable))))
+    (:preserve (output-preserve-symbol name stream readtable))
+    (:invert (output-invert-symbol name stream readtable))))
+
+(declaim (end-block))
 
 ;;;; recursive objects
 
@@ -871,22 +1060,35 @@ variable: an unreadable object representing the error is printed instead.")
             (t
              (output-unreadable-array-readably vector stream))))))
 
-;;; This function outputs a string quoting characters sufficiently
-;;; so that someone can read it in again. Basically, put a slash in
-;;; front of an character satisfying NEEDS-SLASH-P.
+;;; Output a string, quoting characters to be readable by putting a slash in front
+;;; of any character satisfying NEEDS-SLASH-P.
 (defun quote-string (string stream)
   (macrolet ((needs-slash-p (char)
                ;; KLUDGE: We probably should look at the readtable, but just do
                ;; this for now. [noted by anonymous long ago] -- WHN 19991130
-               `(or (char= ,char #\\)
-                 (char= ,char #\"))))
+               `(let ((c ,char)) (or (char= c #\\) (char= c #\"))))
+             (scan (type)
+               ;; Pre-test for any escaping, and if needed, do char-at-a-time output.
+               ;; For 1 or 0 characters, always take the WRITE-CHAR branch.
+               `(let ((data (truly-the ,type data)))
+                  (declare (optimize (sb-c:insert-array-bounds-checks 0)))
+                  (when (or (<= (- end start) 1)
+                            (do ((index start (1+ index)))
+                                ((>= index end))
+                              (when (needs-slash-p (schar data index)) (return t))))
+                    (do ((index start (1+ index)))
+                        ((>= index end) (return-from quote-string))
+                      (let ((char (schar data index)))
+                        (when (needs-slash-p char) (write-char #\\ stream))
+                        (write-char char stream)))))))
     (with-array-data ((data string) (start) (end)
                       :check-fill-pointer t)
-      (do ((index start (1+ index)))
-          ((>= index end))
-        (let ((char (schar data index)))
-          (when (needs-slash-p char) (write-char #\\ stream))
-          (write-char char stream))))))
+      (if (simple-base-string-p data)
+          (scan simple-base-string)
+          #+sb-unicode (scan simple-character-string)))
+    ;; If no escaping needed, WRITE-STRING is way faster, up to 2x in my testing,
+    ;; than WRITE-CHAR because the stream layer will get so many fewer calls.
+    (write-string string stream)))
 
 (defun array-readably-printable-p (array)
   (and (eq (array-element-type array) t)
@@ -1123,7 +1325,7 @@ variable: an unreadable object representing the error is printed instead.")
   ;;   "The function SB-KERNEL:SIMPLE-CHARACTER-STRING-P is undefined."
   ;; but a symbol-macrolet is ok. This is a FIXME except I don't care.
   (symbol-macrolet ((chars "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"))
-    (declare (optimize (sb-c::insert-array-bounds-checks 0) speed))
+    (declare (optimize (sb-c:insert-array-bounds-checks 0) speed))
     (macrolet ((iterative-algorithm ()
                  `(loop (multiple-value-bind (q r)
                             (truncate (truly-the word integer) base)
@@ -1690,10 +1892,7 @@ variable: an unreadable object representing the error is printed instead.")
 (defmethod print-object ((component code-component) stream)
   (print-unreadable-object (component stream :identity t)
     (let (dinfo)
-      (cond ((code-obj-is-filler-p component)
-             (format stream "filler ~dw"
-                     (ash (code-object-size component) (- sb-vm:word-shift))))
-            ((eq (setq dinfo (%code-debug-info component)) :bpt-lra)
+      (cond ((eq (setq dinfo (%code-debug-info component)) :bpt-lra)
              (write-string "bpt-trap-return" stream))
             ((functionp dinfo)
              (format stream "trampoline ~S" dinfo))
@@ -1711,7 +1910,7 @@ variable: an unreadable object representing the error is printed instead.")
                       (write-string ", " stream)
                       (output-object (sb-c::debug-info-name dinfo) stream)))))))))
 
-#-(or x86 x86-64 arm64)
+#-(or x86 x86-64 arm64 riscv)
 (defmethod print-object ((lra lra) stream)
   (print-unreadable-object (lra stream :identity t)
     (write-string "return PC object" stream)))
@@ -1728,92 +1927,109 @@ variable: an unreadable object representing the error is printed instead.")
 #+sb-simd-pack
 (defmethod print-object ((pack simd-pack) stream)
   (cond ((and *print-readably* *read-eval*)
-         (multiple-value-bind (format maker extractor)
-             (etypecase pack
-               ((simd-pack double-float)
-                (values "#.(~S ~S ~S)"
-                        '%make-simd-pack-double #'%simd-pack-doubles))
-               ((simd-pack single-float)
-                (values "#.(~S ~S ~S ~S ~S)"
-                        '%make-simd-pack-single #'%simd-pack-singles))
-               (t
-                (values "#.(~S #X~16,'0X #X~16,'0X)"
-                        '%make-simd-pack-ub64 #'%simd-pack-ub64s)))
-           (multiple-value-call
-               #'format stream format maker (funcall extractor pack))))
+         (format stream "#.(~S #b~3,'0b #x~16,'0X #x~16,'0X)"
+                 '%make-simd-pack
+                 (%simd-pack-tag pack)
+                 (%simd-pack-low pack)
+                 (%simd-pack-high pack)))
         (*print-readably*
          (print-not-readable-error pack stream))
         (t
          (print-unreadable-object (pack stream)
-           (flet ((all-ones-p (value start end &aux (mask (- (ash 1 end) (ash 1 start))))
-                    (= (logand value mask) mask))
-                  (split-num (value start)
-                    (loop
-                       for i from 0 to 3
-                       and v = (ash value (- start)) then (ash v -8)
-                       collect (logand v #xFF))))
-             (multiple-value-bind (low high)
-                 (%simd-pack-ub64s pack)
-               (etypecase pack
-                 ((simd-pack double-float)
-                  (multiple-value-bind (v0 v1) (%simd-pack-doubles pack)
-                    (format stream "~S~@{ ~:[~,13E~;~*TRUE~]~}"
-                            'simd-pack
-                            (all-ones-p low 0 64) v0
-                            (all-ones-p high 0 64) v1)))
-                 ((simd-pack single-float)
-                  (multiple-value-bind (v0 v1 v2 v3) (%simd-pack-singles pack)
-                    (format stream "~S~@{ ~:[~,7E~;~*TRUE~]~}"
-                            'simd-pack
-                            (all-ones-p low 0 32) v0
-                            (all-ones-p low 32 64) v1
-                            (all-ones-p high 0 32) v2
-                            (all-ones-p high 32 64) v3)))
-                 (t
-                  (format stream "~S~@{ ~{ ~2,'0X~}~}"
-                          'simd-pack
-                          (split-num low 0) (split-num low 32)
-                          (split-num high 0) (split-num high 32))))))))))
+           (etypecase pack
+             ((simd-pack double-float)
+              (multiple-value-call #'format stream "~S~@{ ~,13E~}" 'simd-pack
+                (%simd-pack-doubles pack)))
+             ((simd-pack single-float)
+              (multiple-value-call #'format stream "~S~@{ ~,7E~}" 'simd-pack
+                (%simd-pack-singles pack)))
+             ((simd-pack (unsigned-byte 8))
+              (multiple-value-call #'format stream "~S~@{ ~3D~}" 'simd-pack
+                (%simd-pack-ub8s pack)))
+             ((simd-pack (unsigned-byte 16))
+              (multiple-value-call #'format stream "~S~@{ ~5D~}" 'simd-pack
+                (%simd-pack-ub16s pack)))
+             ((simd-pack (unsigned-byte 32))
+              (multiple-value-call #'format stream "~S~@{ ~10D~}" 'simd-pack
+                (%simd-pack-ub32s pack)))
+             ((simd-pack (unsigned-byte 64))
+              (multiple-value-call #'format stream "~S~@{ ~20D~}" 'simd-pack
+                (%simd-pack-ub64s pack)))
+             ((simd-pack (signed-byte 8))
+              (multiple-value-call #'format stream "~S~@{ ~4,@D~}" 'simd-pack
+                (%simd-pack-sb8s pack)))
+             ((simd-pack (signed-byte 16))
+              (multiple-value-call #'format stream "~S~@{ ~6,@D~}" 'simd-pack
+                (%simd-pack-sb16s pack)))
+             ((simd-pack (signed-byte 32))
+              (multiple-value-call #'format stream "~S~@{ ~11@D~}" 'simd-pack
+                (%simd-pack-sb32s pack)))
+             ((simd-pack (signed-byte 64))
+              (multiple-value-call #'format stream "~S~@{ ~20@D~}" 'simd-pack
+                (%simd-pack-sb64s pack))))))))
 
 #+sb-simd-pack-256
 (defmethod print-object ((pack simd-pack-256) stream)
   (cond ((and *print-readably* *read-eval*)
-         (multiple-value-bind (format maker extractor)
-             (etypecase pack
-               ((simd-pack-256 double-float)
-                (values "#.(~@{~S~^ ~})"
-                        '%make-simd-pack-256-double #'%simd-pack-256-doubles))
-               ((simd-pack-256 single-float)
-                (values "#.(~@{~S~^ ~})"
-                        '%make-simd-pack-256-single #'%simd-pack-256-singles))
-               (t
-                (values "#.(~S~@{ #X~16,'0X~})"
-                        '%make-simd-pack-256-ub64 #'%simd-pack-256-ub64s)))
-           (multiple-value-call
-               #'format stream format maker (funcall extractor pack))))
+         (format stream "#.(~S #b~3,'0B #x~16,'0D #x~16,'0D #x~16,'0D #x~16,'0D)"
+                 '%make-simd-pack-256
+                 (%simd-pack-256-tag pack)
+                 (%simd-pack-256-0 pack)
+                 (%simd-pack-256-1 pack)
+                 (%simd-pack-256-2 pack)
+                 (%simd-pack-256-3 pack)))
         (*print-readably*
          (print-not-readable-error pack stream))
         (t
          (print-unreadable-object (pack stream)
            (etypecase pack
              ((simd-pack-256 double-float)
-              (multiple-value-call #'format stream "~S~@{ ~,13E~}"
-                'simd-pack-256
+              (multiple-value-call #'format stream "~S~@{ ~,13E~}" 'simd-pack-256
                 (%simd-pack-256-doubles pack)))
              ((simd-pack-256 single-float)
-              (multiple-value-call #'format stream "~S~@{ ~,7E~}"
-                'simd-pack-256
+              (multiple-value-call #'format stream "~S~@{ ~,7E~}" 'simd-pack-256
                 (%simd-pack-256-singles pack)))
-             (t
-              (multiple-value-bind (p0 p1 p2 p3)
-                  (%simd-pack-256-ub64s pack)
-                (format stream "~S~@{ ~16,'0X~}"
-                        'simd-pack-256
-                        p0 p1 p2 p3))))))))
+             ((simd-pack-256 (unsigned-byte 8))
+              (multiple-value-call #'format stream "~S~@{ ~3D~}" 'simd-pack-256
+                (%simd-pack-256-ub8s pack)))
+             ((simd-pack-256 (unsigned-byte 16))
+              (multiple-value-call #'format stream "~S~@{ ~5D~}" 'simd-pack-256
+                (%simd-pack-256-ub16s pack)))
+             ((simd-pack-256 (unsigned-byte 32))
+              (multiple-value-call #'format stream "~S~@{ ~10D~}" 'simd-pack-256
+                (%simd-pack-256-ub32s pack)))
+             ((simd-pack-256 (unsigned-byte 64))
+              (multiple-value-call #'format stream "~S~@{ ~20D~}" 'simd-pack-256
+                (%simd-pack-256-ub64s pack)))
+             ((simd-pack-256 (signed-byte 8))
+              (multiple-value-call #'format stream "~S~@{ ~4@D~}" 'simd-pack-256
+                (%simd-pack-256-sb8s pack)))
+             ((simd-pack-256 (signed-byte 16))
+              (multiple-value-call #'format stream "~S~@{ ~6@D~}" 'simd-pack-256
+                (%simd-pack-256-sb16s pack)))
+             ((simd-pack-256 (signed-byte 32))
+              (multiple-value-call #'format stream "~S~@{ ~11@D~}" 'simd-pack-256
+                (%simd-pack-256-sb32s pack)))
+             ((simd-pack-256 (signed-byte 64))
+              (multiple-value-call #'format stream "~S~@{ ~20@D~}" 'simd-pack-256
+                (%simd-pack-256-sb64s pack))))))))
 
 ;;;; functions
 
 (defmethod print-object ((object function) stream)
+  (macrolet ((unprintable-instance-p (x)
+               ;; Guard against calling %FUN-FUN if it would return 0.
+               ;; %FUNCALLABLE-INSTANCE-FUN is known to return FUNCTION so determining
+               ;; whether it is actually assigned requires a low-level trick.
+               (let ((s (sb-vm::primitive-object-slot
+                         (sb-vm:primitive-object 'funcallable-instance)
+                         'function)))
+                 `(and (funcallable-instance-p ,x)
+                       (eql 0 (%primitive sb-alien:slot ,x 'function ,(sb-vm:slot-offset s)
+                                          ,sb-vm:fun-pointer-lowtag))))))
+    (when (unprintable-instance-p object)
+      (return-from print-object
+        (print-unreadable-object (object stream :type t :identity t)))))
   (let* ((name (%fun-name object))
          (proper-name-p (and (legal-fun-name-p name) (fboundp name)
                              (eq (fdefinition name) object))))
@@ -1827,16 +2043,20 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;;; catch-all for unknown things
 
-(declaim (inline lowtag-of))
-(defun lowtag-of (x) (logand (get-lisp-obj-address x) sb-vm:lowtag-mask))
-
 (defmethod print-object ((object t) stream)
   (when (eq object sb-pcl:+slot-unbound+)
     ;; If specifically the unbound marker with 0 data,
     ;; as opposed to any other unbound marker.
     (print-unreadable-object (object stream) (write-string "unbound" stream))
     (return-from print-object))
-  (when (eql (get-lisp-obj-address object) sb-vm:no-tls-value-marker-widetag)
+  ;; NO-TLS-VALUE was added here as a printable object type for #+ubsan which,
+  ;; among other things, detects read-before-write on a per-array-element basis.
+  ;; Git rev 22d8038118 caused uninitialized SIMPLE-VECTORs to get prefilled
+  ;; with NO_TLS_VALUE_MARKER, but a better choice would be
+  ;; (logior (mask-field (byte (- n-word-bits 8) 8) -1) unbound-marker-widetag).
+  ;; #+ubsan has probably bitrotted for other reasons, so this is untested.
+  #+ubsan
+  (when (eql (get-lisp-obj-address object) unwritten-vector-element-marker)
     (print-unreadable-object (object stream) (write-string "novalue" stream))
     (return-from print-object))
   (print-unreadable-object (object stream :identity t)
@@ -1848,6 +2068,7 @@ variable: an unreadable object representing the error is printed instead.")
              (#.sb-vm:value-cell-widetag
               (write-string "value cell " stream)
               (output-object (value-cell-ref object) stream))
+             #+nil
              (#.sb-vm:filler-widetag
               (write-string "pad " stream)
               (write (1+ (get-header-data object)) :stream stream)
