@@ -110,6 +110,7 @@
   (def value-cell-ref)
   (def %caller-frame ())
   (def %caller-pc ())
+  (def %code-debug-info)
   (def sb-vm::%code-fixups)
   #+x86-64 (def pointerp)
   (def sb-bignum:%bignum-length)
@@ -119,20 +120,19 @@
   (def %make-instance/mixed)
   (def %instance-length) ; Given an instance, return its length.
   (def %instance-layout)
-  (def %instance-wrapper)
   (def %set-instance-layout (instance new-value))
   ; (def %instance-ref (instance index)) ; defined in 'target-defstruct'
   (def %instance-set (instance index new-value))
   ;; funcallable instances
-  (def %make-funcallable-instance)
+  ;(def %make-funcallable-instance)
   (def %fun-layout)
-  (def %fun-wrapper)
   (def %set-fun-layout (fin new-value))
   (def %funcallable-instance-fun)
   (def (setf %funcallable-instance-fun) (fin new-value))
   (def %funcallable-instance-info (fin i))
-  #+compact-instance-header (progn (def wrapper-of)
+  #+compact-instance-header (progn (def layout-of)
                                    (def %instanceoid-layout))
+  #+64-bit (def layout-depthoid)
 
   ;; lists
   (def %rplaca (x val))
@@ -159,8 +159,8 @@
   (def current-fp ())
   (def stack-ref (s n))
   (def fun-code-header)
+  (def symbol-package-id)
   (def symbol-hash)
-  #+sb-thread (def symbol-tls-index)
   (def symbol-%info) ; primitive reader always needs a stub
   ;; but the "wrapped" reader might not need a stub.
   ;; If it's already a proper function, then it doesn't.
@@ -201,8 +201,8 @@
 ;;; I should add a vop for uint32 access to raw slots.
 (defun sb-c::%structure-is-a (object-layout test-layout)
   (or (eq object-layout test-layout)
-      (let ((depthoid (wrapper-depthoid test-layout))
-            (inherits (wrapper-inherits object-layout)))
+      (let ((depthoid (layout-depthoid test-layout))
+            (inherits (layout-inherits object-layout)))
         (and (> (length inherits) depthoid)
              (eq (svref inherits depthoid) test-layout)))))
 
@@ -210,8 +210,8 @@
   (and (%instancep object)
        (let ((object-layout (%instance-layout object)))
         (or (eq object-layout test-layout)
-            (let ((depthoid (wrapper-depthoid test-layout))
-                  (inherits (wrapper-inherits object-layout)))
+            (let ((depthoid (layout-depthoid test-layout))
+                  (inherits (layout-inherits object-layout)))
               (and (> (length inherits) depthoid)
                    (eq (svref inherits depthoid) test-layout)))))))
 
