@@ -117,6 +117,10 @@
 (defknown sign-extend ((signed-byte 64) t) fixnum
     (foldable flushable movable))
 
+(defoptimizer (sign-extend derive-type) ((x size))
+  (when (sb-c:constant-lvar-p size)
+    (specifier-type `(signed-byte ,(sb-c:lvar-value size)))))
+
 (define-vop (sign-extend)
   (:translate sign-extend)
   (:policy :fast-safe)
