@@ -9,22 +9,28 @@
 (defun 32-bit-register-p (dstate)
   (not (logbitp 31 (current-instruction dstate))))
 
-(defun print-lsl-alias-name (value stream dstate)
+(defun print-ubfm-alias-name (value stream dstate)
   (declare (ignore dstate))
   (destructuring-bind (immr imms) value
-    (princ (if (and (/= imms 63)
-                    (= (1+ imms) immr))
-               'lsl
-               'ubfm)
+    (princ (cond ((and (/= imms 63)
+                       (= (1+ imms) immr))
+                  'lsl)
+                 ((< imms immr)
+                  'ubfiz)
+                 (t
+                  'ubfm))
            stream)))
 
-(defun print-lsl-alias (value stream dstate)
+(defun print-ubfm-alias (value stream dstate)
   (declare (ignore dstate))
   (destructuring-bind (immr imms) value
-    (if (and (/= imms 63)
-             (= (1+ imms) immr))
-        (format stream "#~d" (- 63 imms))
-        (format stream "#~d, #~d" immr imms))))
+    (cond ((and (/= imms 63)
+                (= (1+ imms) immr))
+           (format stream "#~d" (- 63 imms)))
+          ((< imms immr)
+           (format stream "#~d, #~d" (- 64 immr) (1+ imms)))
+          (t
+           (format stream "#~d, #~d" immr imms)))))
 
 (defun print-mem-bar-kind (value stream dstate)
   (declare (ignore dstate))
@@ -101,7 +107,7 @@
 (defun print-test-branch-immediate (value stream dstate)
   (declare (ignore dstate))
   (format stream "#~D"
-          (dpb (car value) (byte 1 5) (car value))))
+          (dpb (car value) (byte 1 5) (cadr value))))
 
 (defun decode-scaled-immediate (value)
   (destructuring-bind (size opc value simd) value
@@ -231,13 +237,114 @@
                 "S")
             value)))
 
+(defun decode-vector-size (q size)
+  (case q
+    (0
+     (case size
+       (#b00 "8B")
+       (#b01 "4H")
+       (#b10 "2S")))
+    (1
+     (case size
+       (#b00 "16B")
+       (#b01 "8H")
+       (#b10 "4S")
+       (#b11 "2D")))))
+
 (defun print-simd-reg (value stream dstate)
   (declare (ignore dstate))
-  (destructuring-bind (size offset) value
+  (multiple-value-bind (q size offset)
+      (if (= (length value) 3)
+          (destructuring-bind (q size offset) value
+            (values q size offset))
+          (destructuring-bind (q offset) value
+            (values q 0 offset)))
     (format stream "V~d.~a" offset
-            (if (zerop size)
-                "8B"
-                "16B"))))
+            (decode-vector-size q size))))
+
+(defun print-simd-immh-reg (value stream dstate)
+  (declare (ignore dstate))
+  (if (= (length value) 2)
+      (destructuring-bind (immh offset) value
+        (format stream "V~d.~a" offset
+                (cond ((logbitp 0 immh)
+                       "8H")
+                      ((logbitp 1 immh)
+                       "4S")
+                      ((logbitp 2 immh)
+                       "2D"))))
+      (destructuring-bind (q immh offset) value
+        (format stream "V~d.~a" offset
+                (cond ((logbitp 0 immh)
+                       (if (zerop q)
+                           "8B"
+                           "16B"))
+                      ((logbitp 1 immh)
+                       (if (zerop q)
+                           "4H"
+                           "8H"))
+                      ((logbitp 2 immh)
+                       (if (zerop q)
+                           "2S"
+                           "4S")))))))
+
+(defun print-simd-reg-cmode (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (q cmode offset) value
+    (format stream "V~d.~a" offset
+            (cond ((eq cmode #b1110)
+                   (if (zerop q)
+                       "8B"
+                       "16B"))
+                  ((zerop (logand cmode #b1001))
+                   (if (zerop q)
+                       "2S"
+                       "4S"))))))
+
+(defun print-simd-modified-imm (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (abc cmode defgh) value
+    (let ((shift
+            (cond ((eq cmode #b1110)
+                   0)
+                  ((zerop (logand cmode #b1001))
+                   (ash cmode 2)))))
+      (princ (dpb abc (byte 3 5) defgh) stream)
+      (when (plusp shift)
+        (format stream ", LSL #~d" shift)))))
+
+(defun decode-fp-immediate (imm type)
+  (let ((sign (ldb (byte 1 7) imm))
+        (exp (ldb (byte 3 4) imm))
+        (frac (ldb (byte 4 0) imm)))
+    (case type
+      (double-float
+       (sb-kernel::double-from-bits
+        sign
+        (logior (ash (logandc1 (ldb (byte 1 2) exp) 1) 10)
+                (ash (if (zerop (ldb (byte 1 2) exp))
+                         0
+                         (ldb (byte 8 0) -1))
+                     2)
+                (ldb (byte 2 0) exp))
+        (ash frac 48)))
+      (single-float
+       (sb-kernel::single-from-bits
+        sign
+        (logior (ash (logandc1 (ldb (byte 1 2) exp) 1) 7)
+                (ash (if (zerop (ldb (byte 1 2) exp))
+                         0
+                         (ldb (byte 5 0) -1))
+                     2)
+                (ldb (byte 2 0) exp))
+        (ash frac 19))))))
+
+(defun print-fp-imm (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (type imm) value
+    (format stream "#~a" (decode-fp-immediate imm (if (= type 0)
+                                                      'single-float
+                                                      'double-float)))))
 
 (defun print-vbhs (value stream dstate)
   (declare (ignore dstate))
@@ -301,6 +408,9 @@
   (declare (ignore dstate))
   (princ (svref +condition-name-vec+ value) stream))
 
+(defun print-negated-cond (value stream dstate)
+  (print-cond (logxor 1 value) stream dstate))
+
 (defun use-label (value dstate)
   (let* ((value (if (consp value)
                     (logior (ldb (byte 2 0) (car value))
@@ -317,36 +427,63 @@
   (case register
     (#.sb-vm::null-offset
      (let ((offset (+ sb-vm:nil-value offset)))
-       (maybe-note-static-symbol (logior offset other-pointer-lowtag)
-                                              dstate)))
+       (or (maybe-note-static-symbol (logior offset other-pointer-lowtag)
+                                     dstate)
+           (maybe-note-assembler-routine offset nil dstate))))
     #+sb-thread
     (#.sb-vm::thread-offset
      (let* ((thread-slots
-             (load-time-value
-              (primitive-object-slots (primitive-object 'sb-vm::thread))
-              t))
-            (slot (find (ash offset (- word-shift)) thread-slots
-                        :key #'slot-offset)))
-       (when slot
-         (note (lambda (stream)
-                 (format stream "thread.~(~A~)" (slot-name slot)))
-               dstate))))))
+              (load-time-value
+               (primitive-object-slots (primitive-object 'sb-vm::thread))
+               t))
+            (slot (find (ash offset (- word-shift)) thread-slots :key #'slot-offset)))
+       (if slot
+           (note (lambda (stream)
+                   (format stream "~(~A~)" (slot-name slot)))
+                 dstate)
+           (flet ((guess-symbol (predicate)
+                    (binding* ((code-header (seg-code (dstate-segment dstate)) :exit-if-null)
+                               (header-n-words (code-header-words code-header)))
+                      (loop for word-num from code-constants-offset below header-n-words
+                            for obj = (code-header-ref code-header word-num)
+                            when (and (symbolp obj) (funcall predicate obj))
+                            do (return obj)))))
+             (let ((symbol (or (guess-symbol
+                                (lambda (s) (= (symbol-tls-index s) offset)))
+                               ;; static symbols aren't in the code header
+                               (find offset +static-symbols+
+                                     :key #'symbol-tls-index))))
+               (when symbol
+                 (note (lambda (stream) (format stream "tls: ~S" symbol))
+                       dstate)))))))))
 
-(defun find-value-from-previos-inst (register dstate)
+(defun find-value-from-previous-inst (register dstate)
   ;; Needs to be MOVZ REGISTER, imm, LSL #0
   ;; Should cover most offsets in sane code
   (let ((inst (current-instruction dstate -4)))
-    (when (and (= (ldb (byte 9 23) inst) #b110100101) ;; MOVZ
-               (= (ldb (byte 5 0) inst) register)
-               (= (ldb (byte 2 21) inst) 0)) ;; LSL #0
-      (ldb (byte 16 5) inst))))
+    (cond ((and (= (ldb (byte 9 23) inst) #b110100101) ;; MOVZ
+                (= (ldb (byte 5 0) inst) register)
+                (= (ldb (byte 2 21) inst) 0)) ;; LSL #0
+           (ldb (byte 16 5) inst))
+          ((and (= (ldb (byte 6 26) inst) #b010110) ;; LDR literal
+                (= (ldb (byte 5 0) inst) register))
+           (let ((value (sb-disassem::code-constant-value
+                         (sb-disassem::segment-offs-to-code-offs
+                          (+ (dstate-cur-offs dstate)
+                             -4
+                             (* (sb-c::mask-signed-field 19 (ldb (byte 19 5) inst))
+                                4))
+                          (dstate-segment dstate))
+                         dstate)))
+             (when (fixnump value)
+               (fixnumize value)))))))
 
 (defun annotate-ldr-str-reg (value stream dstate)
   (declare (ignore stream))
   (let* ((inst (current-instruction dstate))
          (float (ldb-test (byte 1 26) inst)))
     (unless float
-      (let ((value (find-value-from-previos-inst value dstate)))
+      (let ((value (find-value-from-previous-inst value dstate)))
         (when value
           (annotate-ldr-str (ldb (byte 5 5) inst) value dstate))))))
 
@@ -361,39 +498,100 @@
                             value)
                         dstate))))
 
+(defun annotate-ldr-str-pair (value stream dstate)
+  (declare (ignore stream) (ignorable dstate))
+  (destructuring-bind (reg offset) value
+    (declare (ignorable offset))
+    (case reg
+      #+sb-thread
+      (#.sb-vm::thread-offset
+       (let* ((thread-slots
+                (load-time-value
+                 (primitive-object-slots (primitive-object 'sb-vm::thread))
+                 t))
+              (slot1 (find offset thread-slots :key #'slot-offset))
+              (slot2 (find (1+ offset) thread-slots :key #'slot-offset)))
+         (when slot1
+           (note (lambda (stream)
+                   (if (memq (slot-name slot1) '(sb-vm::mixed-tlab
+                                                 sb-vm::cons-tlab
+                                                 sb-vm::unboxed-tlab))
+                       (format stream "~(~a~).{free-pointer, end-addr}" (slot-name slot1))
+                       (format stream "~(~A, ~A~)" (slot-name slot1) (slot-name slot2))))
+                 dstate)))))))
+
 (defun annotate-ldr-literal (value stream dstate)
   (declare (ignore stream))
   (let* ((value (* 4 value))
          (seg (dstate-segment dstate))
-         (code (seg-code seg)))
+         (code (seg-code seg))
+         (inst (current-instruction dstate))
+         (v (ldb (byte 1 26) inst))
+         (addr (+ (dstate-cur-addr dstate) value)))
     (when code
-      (or (note-code-constant (sb-disassem::segment-offs-to-code-offs
-                               (+ (dstate-cur-offs dstate) value) seg)
-                              dstate)
-          (let ((addr (+ (dstate-cur-addr dstate) value)))
-            (and (sb-disassem::points-to-code-constant-p addr code)
-                 (maybe-note-assembler-routine (sap-ref-word (int-sap addr) 0)
-                                               nil dstate)))))))
+      (if (plusp v)
+          (when (sb-disassem::points-to-code-constant-p addr code)
+            (case (ldb (byte 2 30) inst)
+              (#b00
+               (note (lambda (stream)
+                       (format stream "~a" (sap-ref-single (int-sap addr) 0)))
+                     dstate))
+              (#b01
+               (note
+                (lambda (stream)
+                  (format stream "~a" (sap-ref-double (int-sap addr) 0)))
+                dstate))
+              (#b10
+               (note
+                (lambda (stream)
+                  (format stream "~x ~x"(sap-ref-double (int-sap addr) 0)
+                          (sap-ref-double (int-sap addr) 8)))
+                dstate))))
+          (or (note-code-constant (sb-disassem::segment-offs-to-code-offs
+                                   (+ (dstate-cur-offs dstate) value) seg)
+                                  dstate)
+              (and (sb-disassem::points-to-code-constant-p addr code)
+                   (maybe-note-assembler-routine (sap-ref-word (int-sap addr) 0)
+                                                 nil dstate)))))))
 
 ;;;; special magic to support decoding internal-error and related traps
+;;; See EMIT-ERROR-BREAK for the scheme
 (defun snarf-error-junk (sap offset trap-number &optional length-only)
-  (declare (ignore trap-number))
   (let* ((inst (sap-ref-32 sap (- offset 4)))
-         (error-number (ldb (byte 8 13) inst))
+         (error-number (cond
+                         ((>= trap-number sb-vm:error-trap)
+                          (prog1
+                              (- trap-number sb-vm:error-trap)
+                            (setf trap-number sb-vm:error-trap)))
+                         (t
+                          (prog1 (sap-ref-8 sap offset)
+                            (incf offset)))))
+         (first-arg (ldb (byte 8 13) inst))
+         (first-offset (ldb (byte 5 0) first-arg))
+         (first-sc (ldb (byte 2 5) first-arg))
          (length (sb-kernel::error-length error-number))
          (index offset))
     (declare (type sb-sys:system-area-pointer sap)
              (type (unsigned-byte 8) length))
+    (unless (or (= first-arg sb-vm::zr-offset)
+                (zerop length))
+      (decf length))
     (cond (length-only
            (loop repeat length do (sb-c:sap-read-var-integerf sap index))
            (values 0 (- index offset) nil nil))
           (t
            (collect ((sc+offsets)
                      (lengths))
+             (unless (= first-offset sb-vm::zr-offset)
+               (sc+offsets (make-sc+offset (case first-sc
+                                             (1 sb-vm:unsigned-reg-sc-number)
+                                             (2 sb-vm:signed-reg-sc-number)
+                                             (t sb-vm:descriptor-reg-sc-number)) first-offset))
+               (lengths 0))
              (loop repeat length do
-                  (let ((old-index index))
-                    (sc+offsets (sb-c:sap-read-var-integerf sap index))
-                    (lengths (- index old-index))))
+                   (let ((old-index index))
+                     (sc+offsets (sb-c:sap-read-var-integerf sap index))
+                     (lengths (- index old-index))))
              (values error-number
                      (- index offset)
                      (sc+offsets)
@@ -403,7 +601,9 @@
   (declare (ignore inst chunk))
   (let ((code (ldb (byte 8 5) (current-instruction dstate))))
     (flet ((nt (x) (if stream (note x dstate))))
-      (case code
+      (case (if (> code error-trap)
+                error-trap
+                code)
         (#.halt-trap
          (nt "Halt trap"))
         (#.pending-interrupt-trap
