@@ -237,19 +237,33 @@
          ;; deemed unreachable?
          (and
           (almost-immediately-used-p lvar cast)
-          (if (and (lvar-fun-is (combination-fun dest)
-                                '(hairy-data-vector-set/check-bounds
-                                  hairy-data-vector-ref/check-bounds
-                                  hairy-data-vector-ref
-                                  hairy-data-vector-set))
-                   (eql (car (combination-args dest))
-                        lvar))
-              ;; These functions work on all arrays, but the error
-              ;; message is about vectors, which is used more frequently.
-              (csubtypep (specifier-type 'vector)
-                         (single-value-type (cast-type-to-check cast)))
-              (values-subtypep (lvar-externally-checkable-type lvar)
-                               (cast-type-to-check cast)))))))
+
+          (cond ((and (lvar-fun-is (combination-fun dest)
+                                   '(hairy-data-vector-set/check-bounds
+                                     hairy-data-vector-ref/check-bounds
+                                     hairy-data-vector-ref
+                                     hairy-data-vector-set))
+                      (eql (car (combination-args dest))
+                           lvar))
+                 ;; These functions work on all arrays, but the error
+                 ;; message is about vectors, which is used more frequently.
+                 (csubtypep (specifier-type 'vector)
+                            (single-value-type (cast-type-to-check cast))))
+                #+(or arm64 x86-64)
+                ((lvar-fun-is (combination-fun dest)
+                              '(values-list)))
+                ;; Not great
+                ((lvar-fun-is (combination-fun dest)
+                              '(%%primitive))
+                 (destructuring-bind (vop &rest args) (combination-args dest)
+                   (and (constant-lvar-p vop)
+                        (memq (vop-info-name (lvar-value vop)) '(sb-vm::overflow+t
+                                                                 sb-vm::overflow-t
+                                                                 sb-vm::overflow*t))
+                        (eq lvar (car args)))))
+                (t
+                 (values-subtypep (lvar-externally-checkable-type lvar)
+                                  (cast-type-to-check cast))))))))
 
 ;; Type specifiers handled by the general-purpose MAKE-TYPE-CHECK-FORM are often
 ;; trivial enough to have an internal error number assigned to them that can be
