@@ -67,6 +67,50 @@
 (defun callee-return-pc-tn (2env)
   (ir2-environment-return-pc-pass 2env))
 
+;;;; Fixups
+
+;;; a fixup of some kind
+(defstruct (fixup
+            (:constructor make-fixup (name flavor &optional offset))
+            (:copier nil))
+  ;; the name and flavor of the fixup. The assembler makes no
+  ;; assumptions about the contents of these fields; their semantics
+  ;; are imposed by the dumper.
+  (name nil :read-only t)
+  ;; FIXME: "-flavor" and "-kind" are completedly devoid of meaning.
+  ;; They former should probably be "fixup-referent-type" or "fixup-source"
+  ;; to indicate that it denotes a namespace for NAME, and latter should be
+  ;; "fixup-how" as it conveys a manner in which to modify encoded bytes.
+  (flavor nil :read-only t)
+  ;; OFFSET is an optional offset from whatever external label this
+  ;; fixup refers to. Or in the case of the :CODE-OBJECT flavor of
+  ;; fixups on the :X86 architecture, NAME is always NIL, so this
+  ;; fixup doesn't refer to an external label, and OFFSET is an offset
+  ;; from the beginning of the current code block.
+  ;; A LABEL can also be used for ppc or ppc64 in which case the value
+  ;; of the fixup will be the displacement to the label from CODE-TN.
+  (offset 0 :type (or sb-vm:signed-word label)
+            :read-only t))
+
+;;; A FIXUP-NOTE tells you where the assembly patch is to be performed
+(defstruct (fixup-note
+            (:constructor make-fixup-note (kind fixup position))
+            (:copier nil))
+  ;; KIND is architecture-dependent (see the various 'vm' files)
+  (kind nil :type symbol)
+  (fixup (missing-arg) :type fixup)
+  (position 0 :type fixnum))
+(declaim (freeze-type fixup fixup-note))
+
+;;; Record a FIXUP of KIND occurring at the current position in SEGMENT
+(defun note-fixup (segment kind fixup)
+  (emit-back-patch
+   segment 0
+   (lambda (segment posn)
+     (push (make-fixup-note kind fixup
+                            (- posn (segment-header-skew segment)))
+           (sb-assem::segment-fixup-notes segment)))))
+
 ;;;; noise to emit an instruction trace
 
 (defun trace-instruction (section vop inst args state
@@ -257,8 +301,7 @@
                       (unless (and (constant-p (tn-leaf x))
                                    (eql (tn-value x) 0)
                                    (register-p y))
-                        (push-in tn-ref-next args (tn-reads constant))
-                        (setf (tn-ref-tn args) constant)
+                        (change-tn-ref-tn args constant)
                         (setf (tn-ref-load-tn args) nil)))
                      ((register-p y)
                       (setf (svref loaded-constants (tn-offset y))
@@ -291,6 +334,7 @@
          (filler-pattern 0)
          (asmstream (make-asmstream))
          (*asmstream* asmstream))
+    (declare (special sb-vm::*adjustable-vectors*))
 
     (emit (asmstream-elsewhere-section asmstream)
           (asmstream-elsewhere-label asmstream))
