@@ -456,8 +456,8 @@
                     ((not (similar-check-table x file))
                      (dump-list x file t)
                      (similar-save-object x file))))
-             (wrapper
-              (dump-wrapper x file)
+             (layout
+              (dump-layout x file)
               (eq-save-object x file))
              #+sb-xc-host
              (ctype
@@ -586,7 +586,6 @@
 ;;; We peek at the object type so that we only pay the circular
 ;;; detection overhead on types of objects that might be circular.
 (defun dump-object (x file)
-  #+(and metaspace sb-xc-host) (when (cl:typep x 'sb-vm:layout) (error "can't dump sb-vm:layout"))
   (if (compound-object-p x)
       (let ((*circularities-detected* ())
             (circ (fasl-output-circularity-table file)))
@@ -1030,7 +1029,7 @@
 
 (defconstant-eqx +fixup-flavors+
   #(:assembly-routine :assembly-routine*
-    :gc-barrier :symbol-tls-index
+    :card-table-index-mask :symbol-tls-index
     :alien-code-linkage-index :alien-data-linkage-index
     :foreign :foreign-dataref
     :code-object
@@ -1084,7 +1083,7 @@
     (let* ((fixup (fixup-note-fixup note))
            (name (fixup-name fixup))
            (flavor (fixup-flavor fixup))
-           (named (not (member flavor '(:code-object :gc-barrier))))
+           (named (not (member flavor '(:code-object :card-table-index-mask))))
            (data
             (or #-sb-xc-host ; ASM routine indices aren't known to the cross-compiler
                 (when (member flavor '(:assembly-routine :assembly-routine*))
@@ -1097,16 +1096,12 @@
                               flavor data))
            (operand
             (ecase flavor
-              ((:code-object :gc-barrier) (the null name))
+              ((:code-object :card-table-index-mask) (the null name))
               (:layout
                (if (symbolp name)
                    name
-                   (wrapper-classoid-name
-                    (cond #+metaspace
-                          ((sb-kernel::layout-p name) (layout-friend name))
-                          (t name)))))
-              (:layout-id
-               (the wrapper name))
+                   (layout-classoid-name name)))
+              (:layout-id (the layout name))
               ((:assembly-routine :assembly-routine*
                :symbol-tls-index
                ;; Only #+immobile-space can use the following two flavors.
@@ -1241,6 +1236,9 @@
     ;; So these go on the stack last, i.e. nearest the top.
     ;; Reversing sorts the entry points in ascending address order
     ;; except possibly when there are multiple entry points to one routine
+    (unless (= (length (remove-duplicates (mapcar 'car routines)))
+               (length routines))
+      (error "Duplicated asm routine name"))
     (dolist (routine (reverse routines))
       (dump-object (car routine) file)
       (dump-integer (+ (label-position (cadr routine)) (caddr routine))
@@ -1393,12 +1391,12 @@
            struct))
   (note-potential-circularity struct file)
   (do* ((length (%instance-length struct))
-        (wrapper (%instance-wrapper struct))
-        (bitmap (wrapper-bitmap wrapper))
+        (layout (%instance-layout struct))
+        (bitmap (layout-bitmap layout))
         (circ (fasl-output-circularity-table file))
         (index sb-vm:instance-data-start (1+ index)))
       ((>= index length)
-       (dump-non-immediate-object wrapper file)
+       (dump-non-immediate-object layout file)
        (dump-fop 'fop-struct file length))
     (let* ((obj (if (logbitp index bitmap)
                     (%instance-ref struct index)
@@ -1415,14 +1413,14 @@
                              (t obj))
                        file))))
 
-(defun dump-wrapper (obj file &aux (flags (wrapper-flags obj)))
-  (when (wrapper-invalid obj)
+(defun dump-layout (obj file)
+  (when (layout-invalid obj)
     (compiler-error "attempt to dump reference to obsolete class: ~S"
-                    (wrapper-classoid obj)))
+                    (layout-classoid obj)))
   ;; STANDARD-OBJECT could in theory be dumpable, but nothing else,
   ;; because all its subclasses can evolve to have new layouts.
-  (aver (not (logtest flags +pcl-object-layout-flag+)))
-  (let ((name (wrapper-classoid-name obj)))
+  (aver (not (logtest (layout-flags obj) +pcl-object-layout-flag+)))
+  (let ((name (layout-classoid-name obj)))
     ;; Q: Shouldn't we aver that NAME is the proper name for its classoid?
     (unless name
       (compiler-error "dumping anonymous layout: ~S" obj))
@@ -1432,14 +1430,14 @@
     #-sb-xc-host
     (let ((fop (known-layout-fop name)))
       (when fop
-        (return-from dump-wrapper (dump-byte fop file))))
+        (return-from dump-layout (dump-byte fop file))))
     (dump-object name file))
-  (sub-dump-object (wrapper-bitmap obj) file)
-  (sub-dump-object (wrapper-inherits obj) file)
+  (sub-dump-object (layout-bitmap obj) file)
+  (sub-dump-object (layout-inherits obj) file)
   (dump-fop 'fop-layout file
-            (1+ (wrapper-depthoid obj)) ; non-stack args can't be negative
-            (logand flags sb-kernel::layout-flags-mask)
-            (wrapper-length obj)))
+            (1+ (layout-depthoid obj)) ; non-stack args can't be negative
+            (logand (layout-flags obj) sb-kernel::layout-flags-mask)
+            (layout-length obj)))
 
 ;;;; dumping instances which just save their slots
 
@@ -1474,3 +1472,21 @@
   (dump-object namestring file)
   (dump-object cc file)
   (dump-fop 'fop-record-code-coverage file))
+
+(defun dump-emitted-full-calls (hash-table fasl)
+  (let ((list (%hash-table-alist hash-table)))
+    #+sb-xc-host ; enforce host-insensitive reproducible ordering
+    (labels ((symbol< (a b)
+               (cond ((string< a b) t)
+                     ((string= a b)
+                      ;; this does find a few pairs of lookalikes
+                      (string< (cl:package-name (sb-xc:symbol-package a))
+                               (cl:package-name (sb-xc:symbol-package b))))))
+             (fname< (a b)
+               (cond ((and (atom a) (atom b)) (symbol< a b))
+                     ((atom a) t) ; symbol < list
+                     ((atom b) nil) ; opposite
+                     ((symbol< (cadr a) (cadr b))))))
+      (setq list (sort list #'fname< :key #'car)))
+    (dump-object list fasl)
+    (dump-fop 'fop-note-full-calls fasl)))
