@@ -98,12 +98,12 @@
 (defknown find-classoid (name-for-class &optional t)
   (or classoid null) ())
 (defknown classoid-of (t) classoid (flushable))
-(defknown wrapper-of (t) wrapper (flushable))
-(defknown wrapper-depthoid (wrapper) layout-depthoid (flushable))
-#+64-bit (defknown layout-depthoid (sb-vm:layout) layout-depthoid (flushable always-translatable))
-#+(or x86 x86-64) (defknown (layout-depthoid-ge) (sb-vm:layout integer) boolean (flushable))
+(defknown layout-of (t) layout (flushable))
+#+64-bit (defknown layout-depthoid (layout) layout-depthoid (flushable always-translatable))
+#+(or x86 x86-64) (defknown (layout-depthoid-ge) (layout integer) boolean (flushable))
 (defknown %structure-is-a (instance t) boolean (foldable flushable))
 (defknown structure-typep (t t) boolean (foldable flushable))
+(defknown classoid-cell-typep (t t) boolean (foldable flushable no-verify-arg-count))
 (defknown copy-structure (structure-object) structure-object
   (flushable)
   :derive-type #'result-type-first-arg)
@@ -122,7 +122,7 @@
 ;;; either a macro or a special operator, and if the symbol is fbound,
 ;;; a function object is returned".  Our objects of
 ;;; implementation-dependent nature happen to be functions.
-(defknown (symbol-function) (symbol) function ())
+(defknown (symbol-function) (symbol) function (unsafely-flushable))
 
 (defknown boundp (symbol) boolean (flushable))
 (defknown fboundp ((or symbol cons)) (or null function) (unsafely-flushable))
@@ -132,8 +132,8 @@
 (defknown set (symbol t) t ()
   :derive-type #'result-type-last-arg)
 (defknown fdefinition ((or symbol cons)) function ())
-(defknown %set-fdefinition ((or symbol cons) function) function ()
-  :derive-type #'result-type-last-arg)
+(defknown ((setf fdefinition)) (function (or symbol cons)) function ()
+  :derive-type #'result-type-first-arg)
 (defknown makunbound (symbol) symbol ()
   :derive-type #'result-type-first-arg)
 (defknown fmakunbound ((or symbol cons)) (or symbol cons)
@@ -193,6 +193,7 @@
 (defknown copy-symbol (symbol &optional t) symbol (flushable))
 (defknown gensym (&optional (or string unsigned-byte)) symbol ())
 (defknown symbol-package (symbol) (or package null) (flushable))
+(defknown %symbol-package (t) (or package null) (flushable)) ;; doesn't check the type.
 (defknown keywordp (t) boolean (flushable)) ; semi-foldable, see src/compiler/typetran
 
 ;;;; from the "Packages" chapter:
@@ -273,10 +274,14 @@
   (integer integer) rational
   (no-verify-arg-count unsafely-flushable))
 
-(defknown (two-arg-< two-arg-= two-arg->)
+(defknown (two-arg-< two-arg-= two-arg-> two-arg-<= two-arg->=)
     (number number) boolean
     (no-verify-arg-count))
 
+(defknown (range< range<= range<<= range<=<)
+    (fixnum real fixnum) boolean
+    (foldable flushable movable no-verify-arg-count))
+
 (defknown (two-arg-gcd two-arg-lcm two-arg-and two-arg-ior two-arg-xor two-arg-eqv)
   (integer integer) integer
   (no-verify-arg-count))
@@ -334,7 +339,10 @@
 (defknown (rationalize) (real) rational
   (movable foldable flushable recursive))
 
-(defknown (numerator denominator) (rational) integer
+(defknown numerator (rational) integer
+  (movable foldable flushable))
+
+(defknown denominator (rational) (integer 1)
   (movable foldable flushable))
 
 (defknown (floor ceiling round)
@@ -348,8 +356,11 @@
 (defknown unary-truncate (real) (values integer real)
   (movable foldable flushable no-verify-arg-count))
 
+;;; KLUDGE: Don't fold these, the compiler may call it on floats that
+;;; do not truncate into a bignum, and the functions do not check
+;;; their input values and produce corrupted memory.
 (defknown unary-truncate-single-float-to-bignum (single-float) (values bignum (eql $0f0))
-   (foldable movable flushable fixed-args))
+    (#+(or) foldable movable flushable fixed-args))
 (defknown unary-truncate-double-float-to-bignum (double-float)
     (values #+64-bit bignum #-64-bit integer
             (and
@@ -357,12 +368,19 @@
                     (not (or riscv ppc64))) ;; they can't survive cold-init
              (eql $0d0)
              double-float))
-   (foldable movable flushable fixed-args))
+   (#+(or) foldable movable flushable fixed-args))
 
 (defknown %unary-truncate-single-float-to-bignum (single-float) bignum
-   (foldable movable flushable fixed-args))
+   (#+(or) foldable movable flushable fixed-args))
 (defknown %unary-truncate-double-float-to-bignum (double-float) bignum
-   (foldable movable flushable fixed-args))
+   (#+(or) foldable movable flushable fixed-args))
+
+(defknown (subtract-bignum add-bignums) (bignum bignum) integer
+    (movable flushable no-verify-arg-count))
+(defknown (add-bignum-fixnum subtract-bignum-fixnum) (bignum fixnum) integer
+    (movable flushable no-verify-arg-count))
+(defknown subtract-fixnum-bignum (fixnum bignum) integer
+    (movable flushable no-verify-arg-count))
 
 (defknown sxhash-bignum-double-float (double-float) hash-code
   (foldable movable flushable fixed-args))
@@ -439,11 +457,12 @@
 (defknown lognot (integer) integer (movable foldable flushable))
 (defknown logtest (integer integer) boolean (movable foldable flushable commutative))
 (defknown logbitp (unsigned-byte integer) boolean (movable foldable flushable))
-(defknown ash (integer integer) integer
+(defknown (ash ash-inverted) (integer integer) integer
   (movable foldable flushable))
 (defknown %ash/right ((or word sb-vm:signed-word) (mod #.sb-vm:n-word-bits))
   (or word sb-vm:signed-word)
   (movable foldable flushable always-translatable))
+
 (defknown (logcount integer-length) (integer) bit-index
   (movable foldable flushable))
 ;;; FIXME: According to the ANSI spec, it's legal to use any
@@ -888,6 +907,8 @@
   (call important-result)
   :derive-type (creation-result-type-specifier-nth-arg 0))
 
+;; N.B., sb-simple-streams clobbers this; if this changes, make sure
+;; sb-simple-streams follows along, where necessary.
 (defknown read-sequence ((modifying sequence) stream
                          &key
                          (:start index)
@@ -911,7 +932,7 @@
 
 ;; Correct argument type restrictions for these functions are
 ;; complicated, so we just declare them to accept LISTs and suppress
-;; flushing is safe code.
+;; flushing in safe code.
 (defknown (caar cadr cdar cddr
                 caaar caadr cadar caddr cdaar cdadr cddar cdddr
                 caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr
@@ -1365,7 +1386,7 @@
   (movable foldable flushable))
 (defknown open-stream-p (stream) boolean (flushable))
 (defknown close (stream &key (:abort t)) (eql t) ())
-(defknown file-string-length (ansi-stream (or string character))
+(defknown file-string-length (stream (or string character))
   (or unsigned-byte null)
   (flushable))
 
@@ -1459,8 +1480,12 @@
   ()
   :derive-type (read-elt-type-deriver t 'character nil))
 
+;; N.B., sb-simple-streams clobbers this; if this changes, make sure
+;; sb-simple-streams follows along, where necessary.
 (defknown listen (&optional stream-designator) boolean (flushable))
 
+;; N.B., sb-simple-streams clobbers this; if this changes, make sure
+;; sb-simple-streams follows along, where necessary.
 (defknown clear-input (&optional stream-designator) null ())
 
 (defknown read-from-string
@@ -1653,6 +1678,8 @@
 
 (defknown user-homedir-pathname (&optional t) pathname (flushable))
 
+;; N.B., sb-simple-streams clobbers this; if this changes, make sure
+;; sb-simple-streams follows along, where necessary.
 (defknown open
   (pathname-designator &key
                        (:class symbol)
@@ -1850,7 +1877,7 @@
 ;;; special optimizer anyway.
 (defknown %typep (t (or type-specifier ctype)) boolean
   (movable flushable no-verify-arg-count))
-(defknown %instance-typep (t (or type-specifier ctype wrapper)) boolean
+(defknown %instance-typep (t (or type-specifier ctype layout)) boolean
   (movable flushable always-translatable))
 ;;; We should never emit a call to %typep-wrapper
 (defknown %typep-wrapper (t t (or type-specifier ctype)) t
@@ -1888,7 +1915,7 @@
 (defknown %continue-unwind () nil)
 (defknown %throw (t &rest t) nil) ; This is MV-called.
 (defknown %nlx-entry (t) *)
-(defknown %%primitive (t t &rest t) *)
+(defknown %%primitive (t &rest t) *)
 (defknown %pop-values (t) t)
 (defknown %nip-values (t t &rest t) (values))
 (defknown %dummy-dx-alloc (t t) t)
@@ -1902,6 +1929,7 @@
 
 (defknown %odd-key-args-error () nil)
 (defknown %unknown-key-arg-error (t t) nil)
+(defknown nil-fun-returned-error (t) nil)
 (defknown (%ldb %mask-field) (bit-index bit-index integer) unsigned-byte
   (movable foldable flushable no-verify-arg-count))
 (defknown (%dpb %deposit-field) (integer bit-index bit-index integer) integer
@@ -1921,7 +1949,7 @@
 (defknown data-vector-set (array index t) (values) (dx-safe always-translatable))
 (defknown data-vector-set-with-offset (array fixnum fixnum t) (values)
   (dx-safe always-translatable))
-(defknown hairy-data-vector-ref (array index) t (foldable no-verify-arg-count))
+(defknown hairy-data-vector-ref (array index) t (foldable flushable no-verify-arg-count))
 (defknown hairy-data-vector-set (array index t) t (no-verify-arg-count))
 (defknown hairy-data-vector-ref/check-bounds (array index) t (foldable no-verify-arg-count))
 (defknown hairy-data-vector-set/check-bounds (array index t) t (no-verify-arg-count))
@@ -2055,6 +2083,8 @@
 (defknown sb-vm::touch-object (t) (values)
   (always-translatable))
 
+(defknown sb-vm::touch-object-identity (t) t
+  (always-translatable))
 (defknown foreign-symbol-dataref-sap (simple-string)
   system-area-pointer
   (movable flushable))
@@ -2172,7 +2202,7 @@
   (movable foldable flushable))
 
 ;; FIXME: should T be be (OR INSTANCE FUNCALLABLE-INSTANCE) etc?
-(defknown slot-value (t symbol) t (any))
+(defknown (slot-value slot-makunbound) (t symbol) t (any))
 (defknown (slot-boundp slot-exists-p) (t symbol) boolean)
 (defknown sb-pcl::set-slot-value (t symbol t) t (any))
 
@@ -2210,22 +2240,7 @@
   (defknown round-single (single-float #1#) single-float
       (foldable flushable movable always-translatable)))
 
-(defknown fixnum*
-  (fixnum fixnum t)
-  fixnum
-  (movable foldable always-translatable))
-
-(defknown (signed* signed+ signed-)
-  (sb-vm:signed-word sb-vm:signed-word t)
-  sb-vm:signed-word
-  (movable foldable commutative always-translatable))
-
-(defknown (unsigned* unsigned+ unsigned-)
-  (word word t)
-  word
-  (movable foldable commutative always-translatable))
-
-(defknown (unsigned+signed unsigned-signed)
-  (word sb-vm:signed-word t)
-  word
-  (movable foldable commutative always-translatable))
+(defknown (overflow* overflow+ overflow-)
+  (integer integer t)
+  integer
+  (movable always-translatable))
