@@ -95,28 +95,6 @@
      (- other-pointer-lowtag)
      (* fdefn-raw-addr-slot n-word-bytes)))
 
-;;; Various error-code generating helpers
-(defvar *adjustable-vectors*)
-
-(defmacro with-adjustable-vector ((var) &rest body)
-  `(let ((,var (or (pop *adjustable-vectors*)
-                   (make-array 16
-                               :element-type '(unsigned-byte 8)
-                               :fill-pointer 0
-                               :adjustable t))))
-     ;; Don't declare the length - if it gets adjusted and pushed back
-     ;; onto the freelist, it's anyone's guess whether it was expanded.
-     ;; This code was wrong for >12 years, so nobody must have needed
-     ;; more than 16 elements. Maybe we should make it nonadjustable?
-     (declare (type (vector (unsigned-byte 8)) ,var))
-     (setf (fill-pointer ,var) 0)
-     ;; No UNWIND-PROTECT here - semantics are unaffected by nonlocal exit,
-     ;; and this macro is about speeding up the compiler, not slowing it down.
-     ;; GC will clean up any debris, and since the vector does not point
-     ;; to anything, even an accidental promotion to a higher generation
-     ;; will not cause transitive garbage retention.
-     (prog1 (progn ,@body)
-       (push ,var *adjustable-vectors*))))
 
 ;;;; interfaces to IR2 conversion
 
@@ -296,6 +274,12 @@
 (defun instance-tn-ref-p (tn-ref)
   (csubtypep (tn-ref-type tn-ref) (specifier-type 'instance)))
 
+;;; Note that this is a allowed to fail by returning NIL.
+;;; So it's really testing "CERTAINLY-STACK-CONSED-P", which is
+;;; T if and only if if knows, and NIL if it doesn't know,
+;;; or OBJECT is heap-consed.
+;;; In general this is a crummy way to deduce the object's creator,
+;;; because MOVE-OPERAND has a nasty way of interfering.
 (defun stack-consed-p (object)
   (let ((write (sb-c::tn-writes object))) ; list of write refs
     (when (or (not write)    ; grrrr, the only write is from a LOAD tn
@@ -462,3 +446,22 @@
                     smallest-f d n))
            (values (ceiling (expt 2 fraction-bits) d) fraction-bits)))))
 
+(defun system-tlab-p (type node)
+  #-system-tlabs (declare (ignore type node))
+  #+system-tlabs
+  (or sb-c::*force-system-tlab*
+      (let ((typename (cond ((sb-kernel::layout-p type)
+                             (classoid-name (layout-classoid type)))
+                            ((sb-kernel::defstruct-description-p type)
+                             (dd-name type)))))
+        (when (and typename (sb-xc:subtypep typename 'ctype))
+          (error "~S instance constructor called in a non-system file"
+                 typename)))
+      (and node
+           (named-let search-env ((env (sb-c::node-lexenv node)))
+             (dolist (data (sb-c::lexenv-user-data env)
+                           (and (sb-c::lexenv-parent env)
+                                (search-env (sb-c::lexenv-parent env))))
+               (when (and (eq (first data) :declare)
+                          (eq (second data) 'sb-c::tlab))
+                 (return (eq (third data) :system))))))))
