@@ -20,9 +20,7 @@
 
 (define-source-transform compiled-function-p (x)
   (once-only ((x x))
-    `(and (functionp ,x)
-          #+(or sb-fasteval sb-eval)
-          (not (typep ,x 'interpreted-function)))))
+    `(and (functionp ,x) (not (funcallable-instance-p ,x)))))
 
 (define-source-transform char-int (x)
   `(char-code ,x))
@@ -87,6 +85,18 @@
                          sb-vm:bignum-digits-offset
                          index offset))
 
+;;; When copying a structure, try to make the best decision possible
+;;; as to placement. This matters in a few circumstances:
+;;; - when the "system" mixed TLAB is different from the "user" mixed TLAB.
+;;; - when we distinguish boxed from mixed allocation to generation 0.
+;;; GIVE-UP-IR1-TRANSFORM might put the allocation in the wrong TLAB,
+;;; as the general fallback doesn't make the same distinctions.
+;;; Unfortunately the DEFSTRUCT-defined copiers were not inlining even when
+;;; explicitly requested, because COPY-SOMESTRUCT always transforms into
+;;; COPY-STRUCTURE, so we've lost any declared inline-ness of COPY-SOMESTRUCT.
+;;; Therefore we just try to infer it based on whether this transform is
+;;; "acting as" COPY-SOMESTRUCT for a particular struct whose copier
+;;; was requested to be inline.
 (deftransform copy-structure ((instance) * * :result result :node node)
   (let* ((classoid (lvar-type instance))
          (name (and (structure-classoid-p classoid) (classoid-name classoid)))
@@ -98,7 +108,8 @@
          (class-eq (and name
                         (eq (classoid-state classoid) :sealed)
                         (not (classoid-subclasses classoid))))
-         (dd (and class-eq (wrapper-info layout)))
+         (dd (and class-eq (layout-info layout)))
+         (dd-copier (and dd (sb-kernel::dd-copier-name dd)))
          (max-inlined-words 5))
     (unless (and result ; could be unused result (but entire call wasn't flushed?)
                  layout
@@ -111,6 +122,8 @@
                  ;; Definitely do this if copying to stack
                  ;; (Allocation has to be inlined, otherwise there's no way to DX it)
                  (or (lvar-dynamic-extent result)
+                     (and dd-copier
+                          (eq (sb-int:info :function :inlinep dd-copier) 'inline))
                      ;; Or if it's a small fixed number of words
                      ;; and speed at least as important as size.
                      (and class-eq
@@ -152,11 +165,17 @@
              (not (varying-length-struct-p classoid))
              ;; TODO: if sealed with subclasses which add no slots, use the fixed length
              (not (classoid-subclasses classoid)))
-        (dd-length (wrapper-dd (sb-kernel::compiler-layout-or-lose (classoid-name classoid))))
+        (dd-length (layout-dd (sb-kernel::compiler-layout-or-lose (classoid-name classoid))))
         (give-up-ir1-transform))))
 
-(define-source-transform %instance-wrapper (x) `(layout-friend (%instance-layout ,x)))
-(define-source-transform %fun-wrapper (x) `(layout-friend (%fun-layout ,x)))
+;;; This doesn't help a whole lot, but it does fire during compilation of 'info-vector'
+;;; which uses variable-length instances of PACKED-INFO, having no slot transforms.
+(define-source-transform (setf %instance-ref) (newval instance index)
+  `(let ((.newval. ,newval)
+         (.instance. ,instance)
+         (.index. ,index))
+     (%instance-set .instance. .index. .newval.)
+     .newval.))
 
 ;;; *** These transforms should be the only code, aside from the C runtime
 ;;;     with knowledge of the layout index.
@@ -165,9 +184,9 @@
 #-compact-instance-header
 (progn
   (define-source-transform %instance-layout (x)
-    `(truly-the sb-vm:layout (%instance-ref ,x 0)))
+    `(truly-the layout (%instance-ref ,x 0)))
   (define-source-transform %set-instance-layout (instance layout)
-    `(%instance-set ,instance 0 (the sb-vm:layout ,layout)))
+    `(%instance-set ,instance 0 (the layout ,layout)))
   (define-source-transform function-with-layout-p (x)
     `(funcallable-instance-p ,x)))
 
@@ -269,12 +288,12 @@
         ;; so explicitly return the NEW-VALUE
         `(typecase string
            ((simple-array character (*))
-            (let ((c (the* (character :context :aref) new-value)))
+            (let ((c (the* (character :context 'aref-context) new-value)))
               (data-vector-set string index c)
               c))
            #+sb-unicode
            ((simple-array base-char (*))
-            (let ((c (the* (base-char :context :aref :silent-conflict t) new-value)))
+            (let ((c (the* (base-char :context 'aref-context :silent-conflict t) new-value)))
               (data-vector-set string index c)
               c))))))
 
@@ -751,15 +770,14 @@
   (logior sb-vm:character-widetag
           (ash (char-code (lvar-value obj)) sb-vm:n-widetag-bits)))
 
+;;; FIXME: The following should really be done by defining
+;;; UNBOUND-MARKER as a primitive object.
 ;; So that the PCL code walker doesn't observe any use of %PRIMITIVE,
 ;; MAKE-UNBOUND-MARKER is an ordinary function, not a macro.
 #-sb-xc-host
 (defun make-unbound-marker () ; for interpreters
   (sb-sys:%primitive make-unbound-marker))
-;; Get the main compiler to transform MAKE-UNBOUND-MARKER
-;; without the fopcompiler seeing it - the fopcompiler does
-;; expand compiler-macros, but not source-transforms -
-;; because %PRIMITIVE is not generally fopcompilable.
+;; Get the main compiler to transform MAKE-UNBOUND-MARKER.
 (sb-c:define-source-transform make-unbound-marker ()
   `(sb-sys:%primitive make-unbound-marker))
 
