@@ -317,12 +317,7 @@
                    `(macro . (the ,type ,expansion))))
                 (:constant
                  (let ((value (symbol-value name)))
-                   ;; Objects that are freely coalescible become
-                   ;; anonymous since we aren't interested in
-                   ;; accessing them through their name.
-                   (if (sb-xc:typep value '(or number character symbol))
-                       (find-constant value)
-                       (make-constant value (ctype-of value) name))))
+                   (make-constant value (ctype-of value) name)))
                 (t
                  (make-global-var :kind kind
                                   :%source-name name
@@ -342,7 +337,7 @@
       (cl:typep obj 'debug-name-marker)
       ;; STANDARD-OBJECT layouts use MAKE-LOAD-FORM, but all other layouts
       ;; have the same status as symbols - composite objects but leaflike.
-      (and (typep obj 'wrapper) (not (layout-for-pcl-obj-p obj)))
+      (and (typep obj 'layout) (not (layout-for-pcl-obj-p obj)))
       ;; PACKAGEs are also leaflike.
       (cl:typep obj 'package)
       ;; The cross-compiler wants to dump CTYPE instances as leaves,
@@ -393,7 +388,7 @@
            ;; user-defined MAKE-LOAD-FORM methods?
            (when (emit-make-load-form value)
              #+sb-xc-host
-             (aver (eql (wrapper-bitmap (%instance-wrapper value))
+             (aver (eql (layout-bitmap (%instance-layout value))
                         sb-kernel:+layout-all-tagged+))
              (do-instance-tagged-slot (i value)
                (grovel (%instance-ref value i)))))
@@ -1127,8 +1122,7 @@
 
 (declaim (start-block process-decls make-new-inlinep
                       find-in-bindings
-                      process-muffle-decls
-                      %processing-decls))
+                      process-muffle-decls))
 
 ;;; Given a list of LAMBDA-VARs and a variable name, return the
 ;;; LAMBDA-VAR for that name, or NIL if it isn't found. We return the
@@ -1480,7 +1474,7 @@ the stack without triggering overflow protection.")
 (defun process-extent-decl (names vars fvars kind)
   (let ((extent
           (ecase kind
-            ((dynamic-extent dynamic-extent-no-note)
+            ((dynamic-extent)
              (when *stack-allocate-dynamic-extent*
                kind))
             ((truly-dynamic-extent)
@@ -1591,7 +1585,7 @@ the stack without triggering overflow protection.")
          :default res
          :handled-conditions (process-unmuffle-conditions-decl
                               spec (lexenv-handled-conditions res))))
-       ((dynamic-extent truly-dynamic-extent dynamic-extent-no-note)
+       ((dynamic-extent truly-dynamic-extent)
         (process-extent-decl (cdr spec) vars fvars (first spec))
         res)
        ((disable-package-locks enable-package-locks)
@@ -1736,34 +1730,6 @@ the stack without triggering overflow protection.")
             (when local-optimize
               (process-optimize-decl local-optimize (lexenv-policy lexenv))))))
 
-(defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
-  (multiple-value-bind (*lexenv* result-type post-binding-lexenv)
-      (process-decls decls vars fvars :binding-form-p binding-form-p)
-    (cond ((eq result-type *wild-type*)
-           (funcall fun ctran lvar post-binding-lexenv))
-          (t
-           (let ((value-ctran (make-ctran))
-                 (value-lvar (make-lvar)))
-             (multiple-value-prog1
-                 (funcall fun value-ctran value-lvar post-binding-lexenv)
-               (let ((cast (make-cast value-lvar result-type
-                                      (lexenv-policy *lexenv*))))
-                 (link-node-to-previous-ctran cast value-ctran)
-                 (setf (lvar-dest value-lvar) cast)
-                 (use-continuation cast ctran lvar))))))))
-
-(defmacro processing-decls ((decls vars fvars ctran lvar
-                                   &optional post-binding-lexenv)
-                            &body forms)
-  (declare (symbol ctran lvar))
-  (let ((post-binding-lexenv-p (not (null post-binding-lexenv)))
-        (post-binding-lexenv (or post-binding-lexenv (gensym "LEXENV"))))
-    `(%processing-decls ,decls ,vars ,fvars ,ctran ,lvar
-                        ,post-binding-lexenv-p
-                        (lambda (,ctran ,lvar ,post-binding-lexenv)
-                          (declare (ignorable ,post-binding-lexenv))
-                          ,@forms))))
-
 ;;; Return the SPECVAR for NAME to use when we see a local SPECIAL
 ;;; declaration. If there is a global variable of that name, then
 ;;; check that it isn't a constant and return it. Otherwise, create an
