@@ -67,7 +67,7 @@
     (apply #'compiler-mumble foo)))
 
 
-(deftype object () '(or fasl-output core-object null))
+(deftype object () '(or fasl-output #-sb-xc-host core-object null))
 
 (defvar *compile-object* nil)
 (declaim (type object *compile-object*))
@@ -424,16 +424,9 @@ necessary, since type inference may take arbitrarily long to converge.")
                         (not (component-reanalyze component))
                         (eq (component-reoptimize component) :maybe))
                (maybe-mumble "*")
-               (cond ((retry-delayed-ir1-transforms :optimize)
-                      (maybe-mumble "+")
-                      (setq count 0))
-                     (t
-                      (event ir1-optimize-maxed-out)
-                      (ir1-optimize-last-effort component)
-                      (return)))))
-            ((retry-delayed-ir1-transforms :optimize)
-             (setf count 0)
-             (maybe-mumble "+"))
+               (event ir1-optimize-maxed-out)
+               (ir1-optimize-last-effort component)
+               (return)))
             (t
              (return)))
       (when (setq fastp (>= count *max-optimize-iterations*))
@@ -466,7 +459,7 @@ necessary, since type inference may take arbitrarily long to converge.")
 
 (defun ir1-optimize-phase-1 (component)
   (let ((loop-count 0)
-        (*constraint-propagate* *constraint-propagate*))
+        (constraint-propagate *constraint-propagate*))
     (tagbody
      again
        (loop
@@ -484,7 +477,7 @@ necessary, since type inference may take arbitrarily long to converge.")
                (return)))
         (eliminate-dead-code component)
         (dfo-as-needed component)
-        (when *constraint-propagate*
+        (when constraint-propagate
           (maybe-mumble "Constraint ")
           (constraint-propagate component)
           (when (retry-delayed-ir1-transforms :constraint)
@@ -507,7 +500,7 @@ necessary, since type inference may take arbitrarily long to converge.")
        ;; part, so avoid it.
        (when (retry-delayed-ir1-transforms :ir1-phases)
          (setf loop-count 0
-               *constraint-propagate* nil)
+               constraint-propagate nil)
          (go again)))))
 
 ;;; Do all the IR1 phases for a non-top-level component.
@@ -555,6 +548,7 @@ necessary, since type inference may take arbitrarily long to converge.")
 (progn
   (defun component-mem-space (component)
     (or (component-%mem-space component)
+        #-sb-xc-host
         (setf (component-%mem-space component)
               (if (fasl-output-p *compile-object*)
                   (and (eq *compile-file-to-memory-space* :immobile)
@@ -567,10 +561,10 @@ necessary, since type inference may take arbitrarily long to converge.")
   (defun code-immobile-p (thing)
     #+sb-xc-host (declare (ignore thing)) #+sb-xc-host t
     #-sb-xc-host
-    (let ((component (typecase thing
+    (let ((component (etypecase thing
                        (vop  (node-component (vop-node thing)))
                        (node (node-component thing))
-                       (t    thing))))
+                       (component thing))))
       (eq (component-mem-space component) :immobile))))
 
 (defun %compile-component (component)
@@ -588,11 +582,6 @@ necessary, since type inference may take arbitrarily long to converge.")
   (when (or (ir2-component-values-receivers (component-info component))
             (component-dx-lvars component))
     (maybe-mumble "Stack ")
-    ;; STACK only uses dominance information for DX LVAR back
-    ;; propagation (see BACK-PROPAGATE-ONE-DX-LVAR).
-    (when (component-dx-lvars component)
-      (clear-dominators component)
-      (find-dominators component))
     (stack-analyze component)
     ;; Assign BLOCK-NUMBER for any cleanup blocks introduced by
     ;; stack analysis. There shouldn't be any unreachable code after
@@ -772,6 +761,8 @@ necessary, since type inference may take arbitrarily long to converge.")
     (environment-analyze component)
     (dfo-as-needed component)
 
+    (delete-if-no-entries component)
+
     (if (eq (block-next (component-head component))
             (component-tail component))
         (report-code-deletion)
@@ -829,6 +820,21 @@ necessary, since type inference may take arbitrarily long to converge.")
     ;; Its refs will be scanned redundantly, which is harmless.
     (blast (eql-constants ns)))
   (values))
+
+;;; Clear the global hash tables held in IR1-NAMESPACE.
+(defun clear-ir1-namespace ()
+  (when (boundp '*ir1-namespace*)
+    (let ((ir1-namespace *ir1-namespace*))
+      (clrhash (free-funs ir1-namespace))
+      (clrhash (free-vars ir1-namespace))
+      ;; FIXME: It would make sense to clear these tables as well, but
+      ;; ARM64 relies on the constant for NIL to stay around in order to
+      ;; assign a wired TN to it, so we let people share it. Investigate a
+      ;; proper fix.
+      #+(or)
+      (progn
+        (clrhash (eql-constants ir1-namespace))
+        (clrhash (similar-constants ir1-namespace))))))
 
 ;;;; trace output
 
@@ -846,7 +852,7 @@ necessary, since type inference may take arbitrarily long to converge.")
     (format t "~4TL~D: ~S~:[~; [closure]~]~%"
             (label-id (entry-info-offset entry))
             (entry-info-name entry)
-            (entry-info-closure-p entry)))
+            (entry-info-closure-tn entry)))
   (terpri)
   (pre-pack-tn-stats component *standard-output*)
   (terpri)
@@ -1060,7 +1066,12 @@ necessary, since type inference may take arbitrarily long to converge.")
                       :disabled-package-locks *disabled-package-locks*))
            (tll (ir1-toplevel form path nil)))
       (if (eq (block-compile *compilation*) t)
-          (push tll (toplevel-lambdas *compilation*))
+          (let ((compilation *compilation*))
+            (push tll (toplevel-lambdas compilation))
+            (when (package-environment-changed compilation)
+              (finish-block-compilation)
+              (setf (block-compile compilation) t)
+              (setf (package-environment-changed compilation) nil)))
           (compile-toplevel (list tll) nil))
       nil)))
 
@@ -1504,6 +1515,7 @@ necessary, since type inference may take arbitrarily long to converge.")
   (let ((object *compile-object*))
     (etypecase object
       (fasl-output (fasl-dump-toplevel-lambda-call tll object))
+      #-sb-xc-host
       (core-object (core-call-toplevel-lambda      tll object))
       (null))))
 
@@ -1570,16 +1582,13 @@ necessary, since type inference may take arbitrarily long to converge.")
   (locall-analyze-clambdas-until-done lambdas)
 
   (maybe-mumble "IDFO ")
-  (multiple-value-bind (components top-components hairy-top)
+  (multiple-value-bind (components top-components)
       (find-initial-dfo lambdas)
     (when *check-consistency*
       (maybe-mumble "[Check]~%")
       (check-ir1-consistency (append components top-components)))
 
     (let ((top-level-closure nil))
-      (dolist (component (append hairy-top top-components))
-        (when (pre-environment-analyze-top-level component)
-          (setq top-level-closure t)))
       (dolist (component components)
         (compile-component component)
         (when (replace-toplevel-xeps component)
@@ -1593,7 +1602,9 @@ necessary, since type inference may take arbitrarily long to converge.")
           (compile-load-time-value-lambda lambdas)
           (compile-toplevel-lambdas lambdas top-level-closure)))
 
-    (mapc #'clear-ir1-info components))
+    (dolist (component components)
+      (clear-ir1-info component))
+    (clear-ir1-namespace))
   (values))
 
 ;;; Actually compile any stuff that has been queued up for block
@@ -1695,7 +1706,7 @@ necessary, since type inference may take arbitrarily long to converge.")
         (sb-impl::*eval-tlf-index* nil)
         (sb-impl::*eval-source-context* nil))
     (handler-case
-        (handler-bind (((satisfies handle-condition-p) #'handle-condition-handler))
+        (handler-bind (((satisfies handle-condition-p) 'handle-condition-handler))
           (with-compilation-values
             (with-compilation-unit ()
               (fasl-dump-partial-source-info info *compile-object*)
@@ -1717,6 +1728,9 @@ necessary, since type inference may take arbitrarily long to converge.")
                       #-sb-xc-host
                       (core-object (fix-core-source-info info object))
                       (null)))))
+              ;; FIXME: dump/restore "linkage" information, produce deferred warnings
+              ;; (sb-fasl::dump-emitted-full-calls (emitted-full-calls *compilation*)
+              ;;                                  *compile-object*)
               (let ((code-coverage-records
                       (code-coverage-records (coverage-metadata *compilation*))))
                   (unless (zerop (hash-table-count code-coverage-records))
@@ -1795,20 +1809,15 @@ necessary, since type inference may take arbitrarily long to converge.")
 (defglobal *compile-elapsed-time* 0) ; nanoseconds
 (defglobal *compile-file-elapsed-time* 0) ; nanoseconds
 (defun get-thread-virtual-time ()
-  #+(and linux (not sb-xc-host))
-  (multiple-value-bind (sec nsec)
-      (sb-unix::clock-gettime sb-unix:clock-thread-cputime-id)
-    (cons sec nsec))
-  #-(and linux (not sb-xc-host))
-  '(0 . 0))
+  #+(and linux (not sb-xc-host)) (sb-unix:clock-gettime sb-unix:clock-thread-cputime-id)
+  #-(and linux (not sb-xc-host)) (values 0 0))
 
-(defun accumulate-compiler-time (symbol start)
-  (declare (ignorable symbol start))
+(defun accumulate-compiler-time (symbol start-sec start-nsec)
+  (declare (ignorable symbol start-sec start-nsec))
   #+(and linux (not sb-xc-host))
-  (destructuring-bind (sec-after . nsec-after) (get-thread-virtual-time)
-    (destructuring-bind (sec-before . nsec-before) start
-      (let* ((sec-diff (- sec-after sec-before))
-             (nsec-diff (- nsec-after nsec-before))
+  (multiple-value-bind (stop-sec stop-nsec) (get-thread-virtual-time)
+      (let* ((sec-diff (- stop-sec start-sec))
+             (nsec-diff (- stop-nsec start-nsec))
              (total-nsec-diff (+ (* sec-diff (* 1000 1000 1000))
                                  nsec-diff))
              (old (symbol-global-value symbol)))
@@ -1821,7 +1830,7 @@ necessary, since type inference may take arbitrarily long to converge.")
                                   (cas (symbol-value symbol) old new)
                                   #+x86-64
                                   (%cas-symbol-global-value symbol old new)))
-                (return))))))))
+                (return)))))))
 
 ;;; Open some files and call SUB-COMPILE-FILE. If something unwinds
 ;;; out of the compile, then abort the writing of the output file, so
@@ -1891,14 +1900,15 @@ returning its filename.
   :EMIT-CFASL
      (Experimental). If true, outputs the toplevel compile-time effects
      of this file into a separate .cfasl file."
-  (let* ((output-file-pathname nil)
+  (binding*
+        ((output-file-pathname nil)
          (fasl-output nil)
          (cfasl-pathname nil)
          (cfasl-output nil)
          (abort-p t)
          (warnings-p nil)
          (failure-p t) ; T in case error keeps this from being set later
-         (clock-start (get-thread-virtual-time))
+         ((start-sec start-nsec) (get-thread-virtual-time))
          (input-pathname (verify-source-file input-file))
          (source-info
           (make-file-source-info input-pathname external-format
@@ -1958,7 +1968,7 @@ returning its filename.
       (when (and trace-file (not (streamp trace-file)))
         (close *compiler-trace-output*)))
 
-    (accumulate-compiler-time '*compile-file-elapsed-time* clock-start)
+    (accumulate-compiler-time '*compile-file-elapsed-time* start-sec start-nsec)
 
     ;; CLHS says that the first value is NIL if the "file could not
     ;; be created". We interpret this to mean "a valid fasl could not
