@@ -80,9 +80,12 @@
   ;; be able to affect it from :WITH-COMPILATION-UNIT.) NIL here also
   ;; works as a convenient null-lexenv identifier.
   (%policy nil :type (or null policy))
-  ;; A list associating extra user info to symbols.  The entries
-  ;; are of the form (:declare name . value),
+  ;; A list associating extra user info to symbols.
+  ;; sb-cltl2 entries are of the form (:declare name . value),
   ;; (:variable name key . value), or (:function name key . value)
+  ;; The NO-COMPILER-MACRO declartion is also stored here (because it
+  ;; can't be attached to a function, as compiler macros can be
+  ;; defined on macros).
   (user-data nil :type list)
   (parent nil)
   ;; Cache of all visible variables, including the ones coming from
@@ -191,8 +194,8 @@
   ;; the optimizer for this node type doesn't care, it can elect not
   ;; to clear this flag.
   (reoptimize t :type boolean)
-  ;; if the LVAR value is DYNAMIC-EXTENT, CLEANUP protecting it.
-  (dynamic-extent nil :type (or null cleanup))
+  ;; if the LVAR value is DYNAMIC-EXTENT, some information.
+  (dynamic-extent nil :type (or null dx-info))
   ;; something or other that the back end annotates this lvar with
   (info nil)
   ;; Nodes to reoptimize together with the lvar
@@ -201,6 +204,25 @@
   (dependent-annotations nil))
 (!set-load-form-method lvar (:xc :target) :ignore-it)
 
+;;; A DX-INFO structure is used to accumulate information about a
+;;; dynamic extent declaration.
+(defstruct (dx-info (:copier nil))
+  ;; The kind of dynamic extent this is.
+  (kind (missing-arg) :type (member enclose dynamic-extent truly-dynamic-extent))
+  ;; The value recognized to be declared dynamic extent.
+  (value (missing-arg) :type lvar)
+  ;; The stack-allocatable values in the transitive closure of the
+  ;; relation determined by the "otherwise inaccessible part"
+  ;; criterion. This is filled in by environment analysis.
+  (subparts nil :type list)
+  ;; The CLEANUP associated with this dynamic extent.
+  (cleanup (missing-arg) :type cleanup))
+
+(defprinter (dx-info :identity t)
+  kind
+  value
+  subparts)
+
 ;;; These are used for annotating a LVAR with information that can't
 ;;; be expressed using types or if the CAST semantics are undesirable
 ;;; (type derivation, runtime errors).
@@ -345,7 +367,7 @@
 
 (defun %with-ir1-environment-from-node (node fun)
   (declare (type node node) (type function fun))
-  #-sb-xc-host (declare (dynamic-extent fun)) ; "unable"
+  (declare (dynamic-extent fun))
   (let ((*current-component* (node-component node))
         (*lexenv* (node-lexenv node))
         (*current-path* (node-source-path node)))
@@ -692,11 +714,8 @@
   ;; structures whose NLX-INFO-CLEANUP is this cleanup. This is filled
   ;; in by environment analysis.
   ;;
-  ;; For :DYNAMIC-EXTENT: a list of all DX LVARs, preserved by this
-  ;; cleanup. This is filled when the cleanup is created (now by
-  ;; locall call analysis) and is rechecked by environment
-  ;; analysis. (For closures this is a list of the LVAR of the enclose
-  ;; after environment analysis.)
+  ;; For :DYNAMIC-EXTENT: a list of all DX-INFOs, preserved by this
+  ;; cleanup.
   (nlx-info nil :type list))
 (defprinter (cleanup :identity t)
   kind
@@ -758,15 +777,14 @@
 ;;; ENVIRONMENT-NLX-INFO.
 (defstruct (nlx-info
             (:copier nil)
-            (:constructor make-nlx-info (cleanup block)))
+            (:constructor make-nlx-info (cleanup block exit)))
   ;; the cleanup associated with this exit. In a catch or
   ;; unwind-protect, this is the :CATCH or :UNWIND-PROTECT cleanup,
   ;; and not the cleanup for the escape block. The CLEANUP-KIND of
   ;; this thus provides a good indication of what kind of exit is
   ;; being done.
   (cleanup (missing-arg) :type cleanup)
-  ;; the ``continuation'' exited to (the block, succeeding the EXIT
-  ;; nodes). If this exit is from an escape function (CATCH or
+  ;; the continuation exited to. If this exit is from an escape function (CATCH or
   ;; UNWIND-PROTECT), then environment analysis deletes the
   ;; escape function and instead has the %NLX-ENTRY use this
   ;; continuation.
@@ -776,6 +794,7 @@
   ;; ENTRY must also be used to disambiguate, since exits to different
   ;; places may deliver their result to the same continuation.
   (block (missing-arg) :type cblock)
+  (exit (missing-arg) :type exit)
   ;; the entry stub inserted by environment analysis. This is a block
   ;; containing a call to the %NLX-ENTRY funny function that has the
   ;; original exit destination as its successor. Null only
@@ -788,6 +807,7 @@
   (info nil))
 (defprinter (nlx-info :identity t)
   block
+  exit
   target
   info)
 (!set-load-form-method nlx-info (:xc :target) :ignore-it)
@@ -851,16 +871,11 @@
   ;; This may be non-nil when REFS and SETS are null, since code can be deleted.
   (ever-used nil :type (member nil set t))
   ;; is it declared dynamic-extent, or truly-dynamic-extent?
-  (extent nil :type (member nil truly-dynamic-extent dynamic-extent indefinite-extent))
+  (dynamic-extent nil :type (member nil truly-dynamic-extent dynamic-extent))
   ;; some kind of info used by the back end
   (info nil))
 (!set-load-form-method leaf (:xc :target) :ignore-it)
 
-(defun leaf-dynamic-extent (leaf)
-  (let ((extent (leaf-extent leaf)))
-    (unless (member extent '(nil indefinite-extent))
-      extent)))
-
 ;;; LEAF name operations
 (defun leaf-has-source-name-p (leaf)
   (not (eq (leaf-%source-name leaf)
@@ -946,15 +961,13 @@
   ;; was semi-inline, or because it was defined in this block). If
   ;; this function is not an entry point, then this may be deleted or
   ;; LET-converted. NULL if we haven't converted the expansion yet.
-  ;; Note: We need separate functionals for each policy in which
-  ;; the function is used.
-  (functionals nil :type list))
+  (functional nil :type (or functional null)))
 (defprinter (defined-fun :identity t
              :pretty-ir-printer (pretty-print-global-var structure stream))
   %source-name
   inlinep
   same-block-p
-  (functionals :test functionals))
+  (functional :test functional))
 
 ;;;; function stuff
 
@@ -1140,9 +1153,6 @@
       ;; it looks as though it's never interesting to get debug names
       ;; from them, so it's moot. -- WHN)
       (leaf-source-name leaf)))
-(defun leaf-%debug-name (leaf)
-  (when (functional-p leaf)
-    (functional-%debug-name leaf)))
 
 ;;; Is FUNCTIONAL LET-converted? (where we're indifferent to whether
 ;;; it returns one value or multiple values)
@@ -1180,7 +1190,6 @@
 (defstruct (clambda (:include functional)
                     (:conc-name lambda-)
                     (:predicate lambda-p)
-                    (:constructor make-lambda)
                     (:copier nil))
   ;; list of LAMBDA-VAR descriptors for arguments
   (vars nil :type list)
@@ -1215,15 +1224,8 @@
   ;; depends on in such a way that DFO shouldn't put them in separate
   ;; components.
   (calls-or-closes (make-sset) :type (or null sset))
-  ;; the TAIL-SET that this LAMBDA is in. This is null during creation.
-  ;;
-  ;; In CMU CL, and old SBCL, this was also NILed out when LET
-  ;; conversion happened. That caused some problems, so as of
-  ;; sbcl-0.pre7.37.flaky5.2 when I was trying to get the compiler to
-  ;; emit :EXTERNAL functions directly, and so now the value
-  ;; is no longer NILed out in LET conversion, but instead copied
-  ;; (so that any further optimizations on the rest of the tail
-  ;; set won't modify the value) if necessary.
+  ;; the TAIL-SET that this LAMBDA is in. This is null during creation
+  ;; and in let lambdas.
   (tail-set nil :type (or tail-set null))
   ;; the structure which represents the phsical environment that this
   ;; function's variables are allocated in. This is filled in by
@@ -1235,11 +1237,7 @@
   ;; we will still have caller's lexenv to figure out which cleanup is
   ;; in effect.
   (call-lexenv nil :type (or lexenv null))
-  (allow-instrumenting *allow-instrumenting* :type boolean)
-  ;; True if this is a system introduced lambda: it may contain user code, but
-  ;; the lambda itself is not, and the bindings introduced by it are considered
-  ;; transparent by the nested DX analysis.
-  (system-lambda-p nil :type boolean))
+  (allow-instrumenting *allow-instrumenting* :type boolean))
 (defprinter (clambda :conc-name lambda- :identity t
              :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
@@ -1472,9 +1470,7 @@
                 (:copier nil))
   ;; The leaf referenced.
   (leaf nil :type leaf)
-  ;; CONSTANT nodes are always anonymous, since we wish to coalesce named and
-  ;; unnamed constants that are equivalent, we need to keep track of the
-  ;; reference name for XREF.
+  ;; KLUDGE: This is supposed to help with keyword debug messages somehow.
   (%source-name (missing-arg) :type symbol :read-only t)
   ;; Constraints that cannot be expressed as NODE-DERIVED-TYPE
   (constraints nil))
@@ -1629,10 +1625,10 @@
   ;; assertion is satisfied:
   ;;
   ;; NIL
-  ;;    No type check is necessary (VALUE type is a subtype of the TYPE-TO-CHECK.)
+  ;;    No type check is necessary (proven type of VALUE is a subtype of the TYPE-TO-CHECK.)
   ;;
   ;; :EXTERNAL
-  ;;    Type check will be performed by NODE-DEST.
+  ;;    A type check will be performed by NODE-DEST.
   ;;
   ;; T
   ;;    A type check is needed.
@@ -1721,8 +1717,11 @@
 ;;; would be emitted, if necessary.
 (defstruct (enclose (:include valued-node) ; this node uses a dummy lvar for dx analysis
                     (:copier nil))
-  ;; the list of functionals that this ENCLOSE node allocates.
-  (funs nil :type list))
+  ;; The list of functionals that this ENCLOSE node allocates.
+  (funs nil :type list)
+  ;; The cleanup for this enclose if any of its functionals are
+  ;; declared dynamic extent.
+  (cleanup nil :type (or null cleanup)))
 (defprinter (enclose :identity t)
   funs)
 
@@ -1781,8 +1780,21 @@
                         (lexenv thing)
                         (node (node-lexenv thing))
                         (functional (functional-lexenv thing)))))))
+
+;;; The basic interval type. It can handle open and closed intervals.
+;;; A bound is open if it is a list containing a number, just like
+;;; Lisp says. NIL means unbounded.
+(defstruct (interval (:constructor %make-interval (low high))
+                     (:copier nil))
+  low high)
+
+(defstruct (conditional-flags
+            (:constructor make-conditional-flags (flags))
+            (:copier nil))
+  flags)
 
 ;;;; Freeze some structure types to speed type testing.
 
 (declaim (freeze-type node lexenv ctran lvar cblock component cleanup
-                      environment tail-set nlx-info leaf))
+                      environment tail-set nlx-info leaf interval
+                      conditional-flags))
