@@ -17,7 +17,8 @@
 ;;; A list of UNDEFINED-WARNING structures representing references to unknown
 ;;; stuff which came up in a compilation unit.
 (defvar *undefined-warnings*)
-(declaim (list *undefined-warnings*))
+(defvar *argument-mismatch-warnings*)
+(declaim (list *undefined-warnings* *argument-mismatch-warnings*))
 
 ;;; Delete any undefined warnings for NAME and KIND. This is for the
 ;;; benefit of the compiler, but it's sometimes called from stuff like
@@ -68,6 +69,35 @@
      (compiler-error "~@<Illegal function name: ~S.~@:>" name)))
   name)
 
+;;; Check that NAME is a valid class name, returning the name if OK,
+;;; and signalling an error if not.
+(declaim (inline sb-pcl::check-class-name))
+(defun sb-pcl::check-class-name (name &optional (allow-nil t))
+  ;; Apparently, FIND-CLASS and (SETF FIND-CLASS) accept any symbol,
+  ;; but DEFCLASS only accepts non-NIL symbols.
+  (if (or (not (legal-class-name-p name))
+          (and (null name) (not allow-nil)))
+      (error 'sb-kernel::illegal-class-name-error :name name)
+      name))
+
+;;; Check that NAME is a valid designator for the defining macro
+;;; MACRO. This is used mostly to give a consistent message for all
+;;; defining forms, except for DEFCLASS, which uses CHECK-CLASS-NAME.
+(defun check-designator (name macro &optional (predicate #'symbolp)
+                                              (what "symbol")
+                                              (arg-reference "NAME"))
+  ;; If we decide that the correct behavior is to actually macroexpand
+  ;; and then fail later, well, I suppose we could express all macros
+  ;; such that they perform their LEGAL-FUN-NAME-P/SYMBOLP check as
+  ;; part of the ordinary code, as in: (DEFPARAMETER "foo" 3) ->
+  ;; (%defparameter (the symbol '"foo") ...)  which seems at least
+  ;; slightly preferable to failing in the internal function that
+  ;; would store the globaldb info.
+  (unless (funcall predicate name)
+    (error (format nil "The ~A argument to ~A, ~~S, is not a ~A."
+                   arg-reference macro what)
+           name)))
+
 ;;; This is called to do something about SETF functions that overlap
 ;;; with SETF macros. Perhaps we should interact with the user to see
 ;;; whether the macro should be blown away, but for now just give a
@@ -104,7 +134,7 @@
              (assert-symbol-home-package-unlocked name "proclaiming ~S as a function")))
 
       (let ((kind (info :function :kind name)))
-        ;; scrubbing old data I: possible collision with a macro
+        ;; scrubbing old data: possible collision with a macro
         ;; There's a silly little problem with fun names that are not ANSI-legal names,
         ;; e.g. (CAS mumble). We can't ask the host whether that is FBOUNDP,
         ;; because it would rightly complain. So, just assume that it is not FBOUNDP.
@@ -123,17 +153,6 @@
           (unless (pcl-methodfn-name-p name)
             (setf (info :function :kind name) :function))))))
 
-  ;; scrubbing old data II: dangling forward references
-  ;;
-  ;; (This could happen if someone executes PROCLAIM FTYPE at
-  ;; macroexpansion time, which is bad style, or at compile time, e.g.
-  ;; in EVAL-WHEN (:COMPILE) inside something like DEFSTRUCT, in which
-  ;; case it's reasonable style. Either way, NAME is no longer a free
-  ;; function.)
-  (when (boundp '*ir1-namespace*)       ; when compiling
-    (unless (block-compile *compilation*)
-      (remhash name (free-funs *ir1-namespace*))))
-
   (values))
 
 ;;; Make NAME no longer be a function name: clear everything back to
@@ -283,22 +302,14 @@
   (when (and (eq kind 'always-bound) (eq info-value :always-bound)
              (not (boundp name))
              ;; Allow it to be unbound at compile-time.
-             (not *compile-time-eval*)
-             ;; Check if we are still bootstrapping.
-             (not (boundp '*queued-proclaims*)))
+             (not *compile-time-eval*))
     (error "Cannot proclaim an unbound symbol as ~A: ~S" kind name))
 
   (multiple-value-bind (allowed test)
       (ecase kind
         (special
-         ;; KLUDGE: There is probably a better place to do this.
-         (when (boundp '*ir1-namespace*)
-           (remhash name (free-vars *ir1-namespace*)))
          (values '(:special :unknown) #'eq))
         (global
-         ;; KLUDGE: Ditto.
-         (when (boundp '*ir1-namespace*)
-           (remhash name (free-vars *ir1-namespace*)))
          (values '(:global :unknown) #'eq))
         (always-bound (values '(:constant) #'neq)))
     (let ((old (info :variable :kind name)))
@@ -375,8 +386,8 @@
 (defun seal-class (classoid)
   (declare (type classoid classoid))
   (setf (classoid-state classoid) :sealed)
-  (sb-kernel::do-subclassoids ((subclassoid wrapper) classoid)
-    (declare (ignore wrapper))
+  (sb-kernel::do-subclassoids ((subclassoid layout) classoid)
+    (declare (ignore layout))
     (setf (classoid-state subclassoid) :sealed)))
 
 (defun process-freeze-type-declaration (type-specifier)
@@ -384,23 +395,6 @@
     (when (typep class 'classoid)
       (seal-class class))))
 
-(defun process-inline-declaration (name kind)
-  (declare (type (and inlinep (not null)) kind))
-  ;; since implicitly it is a function, also scrubs (FREE-FUNS *IR1-NAMESPACE*)
-  (proclaim-as-fun-name name)
-  (warn-if-inline-failed/proclaim name kind)
-  (setf (info :function :inlinep name) kind))
-
-(defun process-block-compile-declaration (entries kind)
-  (ecase kind
-    (start-block
-     (finish-block-compilation)
-     (let ((compilation *compilation*))
-       (setf (block-compile compilation) t)
-       (setf (entry-points compilation) entries)))
-    (end-block
-     (finish-block-compilation))))
-
 (defun check-deprecation-declaration (state since form)
   (unless (typep state 'deprecation-state)
     (error 'simple-type-error
@@ -475,6 +469,19 @@
 (defun %proclaim (raw-form location)
   (destructuring-bind (&whole form &optional kind &rest args)
       (canonized-decl-spec raw-form)
+    ;; It seems strange to test whether we are currently in
+    ;; compile-time mode this way, but the reason we don't just call
+    ;; %COMPILER-PROCLAIM in a :COMPILE-TOPLEVEL-only situation in the
+    ;; macro-expansion of DECLAIM is that unlike the DEFmumble macros,
+    ;; DECLAIM and PROCLAIM both exist, and it is unclear whether the
+    ;; intent of the ANSI specification is that
+    ;;   (EVAL-WHEN (:COMPILE-TOPLEVEL ...)
+    ;;     (LET ()
+    ;;       (PROCLAIM ...)))
+    ;; should have the exact same compile time effects as (DECLAIM ...).
+    ;; We make the assumption that yes, they should have the same semantics.
+    (when (boundp '*compilation*)
+      (%compiler-proclaim kind args))
     (labels ((store-location (name &key (key kind))
                (if location
                    (setf (getf (info :source-location :declaration name) key)
@@ -507,17 +514,14 @@
                                     (type #'proclaim-type)
                                     (ftype #'proclaim-ftype))
                             ctype type :declared)))
-             (push raw-form *queued-proclaims*)))
-        #-sb-fluid
+             #-sb-xc-host
+             (push raw-form *queued-proclaims*)
+             #+sb-xc-host
+             (error "Type system not yet initialized.")))
         (freeze-type
          (map-args #'process-freeze-type-declaration))
-        ((start-block end-block)
-         (when (and *compile-time-eval* (boundp '*compilation*))
-           (if (eq *block-compile-argument* :specified)
-               (process-block-compile-declaration args kind)
-               (compiler-notify "ignoring ~S declaration since ~
-                                :BLOCK-COMPILE is not :SPECIFIED"
-                                kind))))
+        ;; This only has compile-time effects.
+        ((start-block end-block))
         (optimize
          (multiple-value-bind (new-policy specified-qualities)
              (process-optimize-decl form *policy*)
@@ -541,8 +545,11 @@
         ((disable-package-locks enable-package-locks)
          (setq *disabled-package-locks*
                (process-package-lock-decl form *disabled-package-locks*)))
-        ((#-sb-fluid inline notinline #-sb-fluid maybe-inline)
-         (map-args #'process-inline-declaration kind))
+        ((inline notinline maybe-inline)
+         (dolist (name args)
+           (warn-if-inline-failed/proclaim name kind)
+           (setf (info :function :inlinep name)
+                 (the (and inlinep (not null)) kind))))
         (deprecated
          (destructuring-bind (state since &rest things) args
            (multiple-value-bind (state software version)
