@@ -70,7 +70,7 @@
       (inst bne result old DONT-STORE-TLS)
       (storew new lip)
       DONT-STORE-TLS
-      (inst xori temp result no-tls-value-marker-widetag)
+      (inst xori temp result no-tls-value-marker)
       (inst bne temp zero-tn CHECK-UNBOUND))
 
     (inst addi lip symbol (- (* symbol-value-slot n-word-bytes)
@@ -97,7 +97,7 @@
     (load-tls-index tls-slot symbol)
     (inst add lip thread-base-tn tls-slot)
     (loadw temp lip)
-    (inst xori temp temp no-tls-value-marker-widetag)
+    (inst xori temp temp no-tls-value-marker)
     (inst bne temp zero-tn TLS-VALUE)
     (storew value symbol symbol-value-slot other-pointer-lowtag)
     (inst j DONE)
@@ -129,7 +129,7 @@
     (load-tls-index value object)
     (inst add lip thread-base-tn value)
     (loadw value lip)
-    (inst xori temp value no-tls-value-marker-widetag)
+    (inst xori temp value no-tls-value-marker)
     (inst bne temp zero-tn CHECK-UNBOUND)
     (loadw value object symbol-value-slot other-pointer-lowtag)
     CHECK-UNBOUND
@@ -162,7 +162,7 @@
     (load-tls-index value object)
     (inst add lip thread-base-tn value)
     (loadw value lip)
-    (inst xori temp value no-tls-value-marker-widetag)
+    (inst xori temp value no-tls-value-marker)
     (inst bne temp zero-tn CHECK-UNBOUND)
     (loadw value object symbol-value-slot other-pointer-lowtag)
     CHECK-UNBOUND
@@ -218,7 +218,7 @@
   (:args (symbol :scs (descriptor-reg)))
   (:results (result :scs (unsigned-reg)))
   (:result-types positive-fixnum)
-  (:translate sb-impl::symbol-package-id)
+  (:translate symbol-package-id)
   (:policy :fast-safe)
   (:generator 1 ; ASSUMPTION: symbol-package-bits = 16
    (inst lhu result symbol (+ (ash symbol-name-slot word-shift)
@@ -238,8 +238,6 @@
       (inst srli result result sb-impl::package-id-bits))))
 
 ;;;; Fdefinition (fdefn) objects.
-(define-vop (fdefn-fun cell-ref)
-  (:variant fdefn-fun-slot other-pointer-lowtag))
 
 (define-vop (safe-fdefn-fun)
   (:translate safe-fdefn-fun)
@@ -414,7 +412,8 @@
 (define-vop (closure-init)
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg)))
-  (:info offset)
+  (:info offset dx)
+  (:ignore dx)
   (:generator 4
     (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
@@ -425,8 +424,6 @@
     (storew cfp-tn object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 ;;;; Value Cell hackery.
-(define-vop (value-cell-ref cell-ref)
-  (:variant value-cell-value-slot other-pointer-lowtag))
 
 (define-vop (value-cell-set cell-set)
   (:variant value-cell-value-slot other-pointer-lowtag))
