@@ -547,8 +547,7 @@
   (:generator 6
     (check-ocfp-and-return-pc old-fp return-pc)
     ;; Zot all of the stack except for the old-fp and return-pc.
-    (inst mov rsp-tn rbp-tn)
-    (inst pop rbp-tn)
+    (inst leave)
     (inst ret)))
 
 ;;;; full call
@@ -589,7 +588,7 @@
 ;;; In tail call with fixed arguments, the passing locations are
 ;;; passed as a more arg, but there is no new-FP, since the arguments
 ;;; have been set up in the current frame.
-(macrolet ((define-full-call (vop-name named return variable)
+(macrolet ((define-full-call (vop-name named return variable &optional args)
             (aver (not (and variable (eq return :tail))))
             #+immobile-code (when named (setq named :direct))
             `(define-vop (,vop-name ,@(when (eq return :unknown)
@@ -610,7 +609,9 @@
                    '((old-fp)
                      (return-pc)))
 
-               ,@(unless variable '((args :more t :scs (descriptor-reg)))))
+               ,@(unless variable
+                   `((args :more t ,@(unless (eq args :fixed)
+                                       '(:scs (descriptor-reg control-stack)))))))
 
                ,@(when (eq return :fixed)
                    '((:results (values :more t))))
@@ -618,7 +619,9 @@
                (:save-p ,(if (eq return :tail) :compute-only t))
 
                ,@(unless (or (eq return :tail) variable)
-               '((:move-args :full-call)))
+                   `((:move-args ,(if (eq args :fixed)
+                                      :fixed
+                                      :full-call))))
 
                (:vop-var vop)
                (:node-var node)
@@ -795,34 +798,14 @@
                DONE
                (note-this-location vop :call-site)
                ,(cond ((eq named :direct)
-                       #+immobile-code
-                       `(let* ((fixup (make-fixup
-                                       fun
-                                       (if (static-fdefn-offset fun)
-                                           :static-call
-                                           :named-call)))
-                               (target
-                                 (if (and (sb-c::code-immobile-p node)
-                                          (not step-instrumenting))
-                                     fixup
-                                     (progn
-                                       ;; RAX-TN was not declared as a temp var,
-                                       ;; however it's sole purpose at this point is
-                                       ;; for function call, so even if it was used
-                                       ;; to compute a stack argument, it's free now.
-                                       ;; If the call hits the undefined fun trap,
-                                       ;; RAX will get loaded regardless.
-                                       (inst mov rax-tn fixup)
-                                       rax-tn))))
-                          (inst ,(if (eq return :tail) 'jmp 'call) target))
-                       #-immobile-code
-                       `(inst ,(if (eq return :tail) 'jmp 'call)
-                              (ea (+ nil-value (static-fun-offset fun)))))
+                       #+immobile-code `(emit-direct-call fun ',(if (eq return :tail) 'jmp 'call)
+                                                          node step-instrumenting)
+                       #-immobile-code `(inst ,(if (eq return :tail) 'jmp 'call)
+                                              (ea (+ nil-value (static-fun-offset fun)))))
                       #-immobile-code
                       (named
                        `(inst ,(if (eq return :tail) 'jmp 'call)
-                              (ea (- (* fdefn-raw-addr-slot n-word-bytes)
-                                     other-pointer-lowtag) rax)))
+                              (object-slot-ea rax fdefn-raw-addr-slot other-pointer-lowtag)))
                       ((eq return :tail)
                        `(tail-call-unnamed rax fun-type vop))
                       (t
@@ -850,7 +833,35 @@
   (define-full-call static-tail-call-named :direct :tail nil)
 
   (define-full-call call-variable nil :fixed t)
-  (define-full-call multiple-call-variable nil :unknown t))
+  (define-full-call multiple-call-variable nil :unknown t)
+  (define-full-call fixed-call-named t :fixed nil :fixed)
+  (define-full-call fixed-tail-call-named t :tail nil :fixed))
+
+;;; Call NAME "directly" meaning in a single JMP or CALL instruction,
+;;; if possible (without loading RAX)
+(defun emit-direct-call (name instruction node step-instrumenting)
+      ;; a :STATIC-CALL fixup is the address of the entry point of
+      ;; the function itself, and a :FDEFN-CALL fixup is the address
+      ;; of the JMP instruction embedded in the header for the named FDEFN.
+  (when (static-fdefn-offset name)
+    (let ((fixup (make-fixup name :static-call)))
+      (return-from emit-direct-call
+        (inst* instruction (if (sb-c::code-immobile-p node) fixup (ea fixup))))))
+  (let* ((fixup (make-fixup name :fdefn-call))
+         (target
+              (if (and (sb-c::code-immobile-p node)
+                       (not step-instrumenting))
+                  fixup
+                  (progn
+                    ;; RAX-TN was not declared as a temp var,
+                    ;; however it's sole purpose at this point is
+                    ;; for function call, so even if it was used
+                    ;; to compute a stack argument, it's free now.
+                    ;; If the call hits the undefined fun trap,
+                    ;; RAX will get loaded regardless.
+                    (inst mov rax-tn fixup)
+                    rax-tn))))
+    (inst* instruction target)))
 
 ;;; Invoke the function-designator FUN.
 (defun tail-call-unnamed (fun type vop)
@@ -932,12 +943,10 @@
   (:ignore value)
   (:generator 6
     (check-ocfp-and-return-pc old-fp return-pc)
-    ;; Drop stack above old-fp
-    (inst mov rsp-tn rbp-tn)
+    ;; Drop stack above old-fp and restore old frame pointer
+    (inst leave)
     ;; Clear the multiple-value return flag
     (inst clc)
-    ;; Restore the old frame pointer
-    (inst pop rbp-tn)
     ;; And return.
     (inst ret)))
 
@@ -991,8 +1000,7 @@
     ;; stack and we've changed the stack pointer. So we have to
     ;; tell it to index off of RBX instead of RBP.
     (cond ((<= nvals register-arg-count)
-           (inst mov rsp-tn rbp-tn)
-           (inst pop rbp-tn)
+           (inst leave)
            (inst ret))
           (t
            ;; Some values are on the stack after RETURN-PC and OLD-FP,
@@ -1039,11 +1047,10 @@
         ;; Return with one value.
         (loadw a0 vals -1)
         ;; Clear the stack until ocfp.
-        (inst mov rsp-tn rbp-tn)
+        (inst leave)
         ;; clear the multiple-value return flag
         (inst clc)
         ;; Out of here.
-        (inst pop rbp-tn)
         (inst ret)
         ;; Nope, not the single case. Jump to the assembly routine.
         (emit-label not-single)))
@@ -1072,19 +1079,19 @@
   (:generator 20
     ;; Avoid the copy if there are no more args.
     (cond ((zerop fixed)
-           (inst test rcx-tn rcx-tn)
+           (inst test :dword rcx-tn rcx-tn)
            (inst jmp :z JUST-ALLOC-FRAME))
           ((and (eql min-verified fixed)
                 (> fixed 1))
            ;; verify-arg-count will do a CMP
            (inst jmp :e JUST-ALLOC-FRAME))
           (t
-           (inst cmp rcx-tn (fixnumize fixed))
+           (inst cmp :dword rcx-tn (fixnumize fixed))
            (inst jmp :be JUST-ALLOC-FRAME)))
 
     ;; Create a negated copy of the number of arguments to allow us to
     ;; use EA calculations in order to do scaled subtraction.
-    (inst mov temp rcx-tn)
+    (inst mov :dword temp rcx-tn)
     (inst neg temp)
 
     ;; Allocate the space on the stack.
@@ -1279,38 +1286,37 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 20
-    (let ((loop (gen-label))
-          (done (gen-label))
-          (leave-pa (gen-label))
-          (stack-allocate-p (node-stack-allocate-p node)))
-      ;; Compute the number of bytes to allocate
-      (let ((shift (- (1+ word-shift) n-fixnum-tag-bits)))
-        (if (location= count rcx)
-            (inst shl :dword rcx shift)
-            (inst lea :dword rcx (ea nil count (ash 1 shift)))))
-      ;; Setup for the CDR of the last cons (or the entire result) being NIL.
-      (inst mov result nil-value)
-      (inst jrcxz DONE)
-      (unless stack-allocate-p
-        (instrument-alloc +cons-primtype+ rcx node (list value dst) thread-tn))
-      (pseudo-atomic (:elide-if stack-allocate-p :thread-tn thread-tn)
+    ;; Compute the number of bytes to allocate
+    (let ((shift (- (1+ word-shift) n-fixnum-tag-bits)))
+      (if (location= count rcx)
+          (inst shl :dword rcx shift)
+          (inst lea :dword rcx (ea nil count (ash 1 shift)))))
+    ;; Setup for the CDR of the last cons (or the entire result) being NIL.
+    (inst mov result nil-value)
+    (inst jrcxz DONE)
+    (unless (node-stack-allocate-p node)
+      (instrument-alloc +cons-primtype+ rcx node (list value dst) thread-tn))
+    (pseudo-atomic (:elide-if (node-stack-allocate-p node) :thread-tn thread-tn)
        ;; Produce an untagged pointer into DST
-       (if stack-allocate-p
+       (if (node-stack-allocate-p node)
            (stack-allocation rcx 0 dst)
            (allocation +cons-primtype+ rcx 0 dst node value thread-tn
-                       :overflow (lambda ()
-                                   (inst push rcx)
-                                   (inst push context)
-                                   (invoke-asm-routine 'call 'listify-&rest node)
-                                   (inst pop result)
-                                   (inst jmp leave-pa))))
+                       :overflow
+                       (lambda ()
+                         (inst push rcx)
+                         (inst push context)
+                         (invoke-asm-routine
+                          'call (if (system-tlab-p 0 node) 'sys-listify-&rest 'listify-&rest)
+                          node)
+                         (inst pop result)
+                         (inst jmp leave-pa))))
        ;; Recalculate DST as a tagged pointer to the last cons
        (inst lea dst (ea (- list-pointer-lowtag (* cons-size n-word-bytes)) dst rcx))
        (inst shr :dword rcx (1+ word-shift)) ; convert bytes to number of cells
        ;; The rightmost arguments are at lower addresses.
        ;; Start by indexing the last argument
        (inst neg rcx) ; :QWORD because it's a signed number
-       (emit-label LOOP)
+       LOOP
        ;; Grab one value and store into this cons. Use RCX as an index into the
        ;; vector of values in CONTEXT, but add 8 because CONTEXT points exactly at
        ;; the 0th value, which means that the index is 1 word too low.
@@ -1325,8 +1331,8 @@
        (inst sub dst (* cons-size n-word-bytes)) ; get the preceding cons
        (inst inc rcx) ; :QWORD because it's a signed number
        (inst jmp :nz loop)
-       (emit-label leave-pa))
-      (emit-label done))))
+       LEAVE-PA)
+    DONE))
 
 ;;; Return the location and size of the &MORE arg glob created by
 ;;; COPY-MORE-ARG. SUPPLIED is the total number of arguments supplied
@@ -1362,6 +1368,7 @@
   (:policy :fast-safe)
   (:args (nargs :scs (any-reg)))
   (:arg-types positive-fixnum (:constant t) (:constant t))
+  (:temporary (:sc unsigned-reg :offset rbx-offset) temp)
   (:info min max)
   (:vop-var vop)
   (:save-p :compute-only)
@@ -1369,24 +1376,24 @@
     ;; NOTE: copy-more-arg expects this to issue a CMP for min > 1
     (let ((err-lab
             (generate-error-code vop 'invalid-arg-count-error nargs)))
-      (flet ((check-min ()
-               (cond ((= min 1)
-                      (inst test nargs nargs)
-                      (inst jmp :e err-lab))
-                     ((plusp min)
-                      (inst cmp nargs (fixnumize min))
-                      (inst jmp :b err-lab)))))
-        (cond ((not min)
-               (if (zerop max)
-                   (inst test nargs nargs)
-                   (inst cmp nargs (fixnumize max)))
-               (inst jmp :ne err-lab))
-              (max
-               (check-min)
-               (inst cmp nargs (fixnumize max))
-               (inst jmp :a err-lab))
-              (t
-               (check-min)))))))
+      (cond ((not min)
+             (if (zerop max)
+                 (inst test :dword nargs nargs)
+                 (inst cmp :dword nargs (fixnumize max)))
+             (inst jmp :ne err-lab))
+            (max
+             (if (zerop min)
+                 (setf temp nargs)
+                 (inst lea :dword temp (ea (fixnumize (- min)) nargs)))
+             (inst cmp :dword temp (fixnumize (- max min)))
+             (inst jmp :a err-lab))
+            (t
+             (cond ((= min 1)
+                    (inst test :dword nargs nargs)
+                    (inst jmp :e err-lab))
+                   ((plusp min)
+                    (inst cmp :dword nargs (fixnumize min))
+                    (inst jmp :b err-lab))))))))
 
 ;; Signal an error about an untagged number.
 ;; These are pretty much boilerplate and could be generic except:
