@@ -33,7 +33,7 @@
 ;;;    be younger than the newly constructed thing.
 ;;; 2. hash-table k/v pair should mark once only.
 ;;;    (the vector elements are certainly on the same card)
-(defun emit-gc-store-barrier (object cell-address scratch-reg &optional value-tn-ref value-tn)
+(defun emit-gengc-barrier (object cell-address scratch-reg &optional value-tn-ref value-tn)
   (when (sc-is object constant immediate)
     (aver (symbolp (tn-value object))))
   (when (require-gc-store-barrier-p object value-tn-ref value-tn)
@@ -53,7 +53,7 @@
     ;; Otherwise, it has to remember who "wins". 0 makes it indifferent.
     (inst mov :byte (ea gc-card-table-reg-tn scratch-reg) 0)))
 
-(defun gen-cell-set (ea value val-temp)
+(defun emit-store (ea value val-temp)
   (sc-case value
    (immediate
       (let ((bits (encode-value-if-immediate value)))
@@ -73,17 +73,8 @@
    (t
       (inst mov :qword ea value))))
 
-;;; CELL-REF and CELL-SET are used to define VOPs like CAR, where the
-;;; offset to be read or written is a property of the VOP used.
-(define-vop (cell-ref)
-  (:args (object :scs (descriptor-reg)))
-  (:results (value :scs (descriptor-reg any-reg)))
-  (:variant-vars offset lowtag)
-  (:policy :fast-safe)
-  (:generator 4
-    (loadw value object offset lowtag)))
-;; This vop's sole purpose is to be an ancestor for other vops, to assign
-;; default operands, policy, and generator.
+;; This vop's sole purpose is to provide the implementation of value-cell-set.
+;; It could be removed, for x86-64 anyway.
 (define-vop (cell-set)
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg immediate)))
@@ -92,9 +83,9 @@
   (:temporary (:sc unsigned-reg) val-temp)
   (:vop-var vop)
   (:generator 4
-    (emit-gc-store-barrier object nil val-temp (vop-nth-arg 1 vop) value)
+    (emit-gengc-barrier object nil val-temp (vop-nth-arg 1 vop) value)
     (let ((ea (object-slot-ea object offset lowtag)))
-      (gen-cell-set ea value val-temp))))
+      (emit-store ea value val-temp))))
 
 ;;; X86 special
 (define-vop (cell-xadd)
@@ -204,14 +195,13 @@
 (defmacro compute-splat-bits (value)
   ;; :SAFE-DEFAULT means any unspecific value that is safely a default.
   ;; Heap allocation uses 0 since that costs nothing.
-  ;; Stack allocation pick no-tls-value-marker even though it differs from the heap.
   ;; If the user wanted a specific value, it could have been explicitly given.
   `(if (typep ,value 'sb-vm:word)
        ,value
        (case ,value
          (:unbound (unbound-marker-bits))
          ((nil) (bug "Should not see SPLAT NIL"))
-         (t #+ubsan no-tls-value-marker-widetag
+         (t #+ubsan unwritten-vector-element-marker
             #-ubsan 0))))
 
 ;;; This logic was formerly in ALLOCATE-VECTOR-ON-STACK.
