@@ -85,7 +85,7 @@
 ;;; It will never be used with a constant displacement.
 (define-symbol-macro card-table-reg 12)
 (define-symbol-macro gc-card-table-reg-tn r12-tn)
-(define-symbol-macro card-index-mask (make-fixup nil :gc-barrier))
+(define-symbol-macro card-index-mask (make-fixup nil :card-table-index-mask))
 
 (macrolet ((defreg (name offset size)
              (declare (ignore size))
@@ -174,7 +174,8 @@
   #-win32
   (defregset    *c-call-register-arg-offsets* rdi rsi rdx rcx r8 r9)
   #+win32
-  (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9))
+  (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9)
+  (defregset *descriptor-args* rdx rdi rsi rbx rcx r8 r9 r10 r14 r15))
 
 ;;;; SB definitions
 
@@ -186,9 +187,7 @@
 ;;; Start from 2, for the old RBP (aka OCFP) and return address
 (define-storage-base stack :unbounded :size 2 :size-increment 1)
 (define-storage-base constant :non-packed)
-(define-storage-base immediate-constant :non-packed)
-(define-storage-base noise :unbounded :size 2)
-)
+(define-storage-base immediate-constant :non-packed))
 
 ;;;; SC definitions
 
@@ -244,12 +243,6 @@
   #+sb-simd-pack-256
   (single-avx2-stack stack :element-size 4)
 
-  ;;
-  ;; magic SCs
-  ;;
-
-  (ignore-me noise)
-
   ;;
   ;; things that can go in the integer registers
   ;;
@@ -471,28 +464,19 @@
          character)
      immediate-sc-number)
     (symbol ; Symbols in static and immobile space are immediate
-     (when (or ;; With #+immobile-symbols, all symbols are in immobile-space.
-               ;; And the cross-compiler always uses immobile-space if enabled.
-               #+(or immobile-symbols (and immobile-space sb-xc-host)) t
-
-               ;; Otherwise, if #-immobile-symbols, and the symbol was present
-               ;; in the initial core image as indicated by the symbol header, then
-               ;; it's in immobile-space. There is a way in which the bit can be wrong,
-               ;; but it's highly unlikely - the symbol would have to be uninterned from
-               ;; the loading SBCL, reallocated in dynamic space and re-interned into its
-               ;; initial package. All without breaking anything. Hence, unlikely.
-               ;; Also note that if compiling to memory, the symbol's current address
-               ;; is used to determine whether it's immediate.
-               #+(and (not sb-xc-host) immobile-space (not immobile-symbols))
-               (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
-                   (locally (declare (notinline sb-c::producing-fasl-file))
-                     (and (not (sb-c::producing-fasl-file))
-                          (immobile-space-obj-p value))))
-
-               (static-symbol-p value))
+     (when (or (static-symbol-p value)
+               ;; The cross-compiler always uses immobile-space if it exists.
+               #+(and immobile-space sb-xc-host) t
+               ;; With #+immobile-symbols, all interned symbols are in immobile-space.
+               #+immobile-symbols (sb-xc:symbol-package value)
+               #-sb-xc-host
+               (if (immobile-space-obj-p value)
+                   (or (= (generation-of value) +pseudo-static-generation+)
+                       ;; If compiling to memory, the symbol's address alone suffices.
+                       (locally (declare (notinline sb-c::producing-fasl-file))
+                         (not (sb-c::producing-fasl-file))))))
        immediate-sc-number))
-    #+metaspace (sb-vm:layout (bug "Can't reference layout as a constant"))
-    #+(and immobile-space (not metaspace)) (wrapper immediate-sc-number)
+    #+compact-instance-header (layout immediate-sc-number)
     (single-float
        (if (eql value $0f0) fp-single-zero-sc-number fp-single-immediate-sc-number))
     (double-float
@@ -505,18 +489,23 @@
        (if (eql value (complex $0d0 $0d0))
             fp-complex-double-zero-sc-number
             fp-complex-double-immediate-sc-number))
+    ;; This case has to follow the numeric cases because proxy floating-point numbers
+    ;; are host structs. Or we could implement and use something like SB-XC:TYPECASE
+    (structure-object
+     (when (eq value sb-lockless:+tail+)
+       immediate-sc-number))
     #+(and sb-simd-pack (not sb-xc-host))
-    ((simd-pack double-float) double-sse-immediate-sc-number)
-    #+(and sb-simd-pack (not sb-xc-host))
-    ((simd-pack single-float) single-sse-immediate-sc-number)
-    #+(and sb-simd-pack (not sb-xc-host))
-    (simd-pack int-sse-immediate-sc-number)
+    (simd-pack
+     (typecase value
+       ((simd-pack double-float) double-sse-immediate-sc-number)
+       ((simd-pack single-float) single-sse-immediate-sc-number)
+       (t int-sse-immediate-sc-number)))
     #+(and sb-simd-pack-256 (not sb-xc-host))
-    ((simd-pack-256 double-float) double-avx2-immediate-sc-number)
-    #+(and sb-simd-pack-256 (not sb-xc-host))
-    ((simd-pack-256 single-float) single-avx2-immediate-sc-number)
-    #+(and sb-simd-pack-256 (not sb-xc-host))
-    (simd-pack-256 int-avx2-immediate-sc-number)))
+    (simd-pack-256
+     (typecase value
+       ((simd-pack-256 double-float) double-avx2-immediate-sc-number)
+       ((simd-pack-256 single-float) single-avx2-immediate-sc-number)
+       (t int-avx2-immediate-sc-number)))))
 
 (defun boxed-immediate-sc-p (sc)
   (eql sc immediate-sc-number))
@@ -529,13 +518,22 @@
           (symbol   (if (static-symbol-p val)
                         (+ nil-value (static-symbol-offset val))
                         (make-fixup val :immobile-symbol)))
-          #+(and immobile-space (not metaspace))
-          (wrapper
+          #+immobile-space
+          (layout
            (make-fixup val :layout))
           (character (if tag
                          (logior (ash (char-code val) n-widetag-bits)
                                  character-widetag)
-                         (char-code val)))))
+                         (char-code val)))
+          (single-float
+           (let ((bits (single-float-bits val)))
+             (if tag
+                 (dpb bits (byte 32 32) single-float-widetag)
+                 bits)))
+          (structure-object
+           (if (eq val sb-lockless:+tail+)
+               (progn (aver tag) sb-vm::lockfree-list-tail-value)
+               (bug "immediate structure-object ~S" val)))))
       tn))
 
 ;;;; miscellaneous function call parameters
@@ -621,3 +619,19 @@
 
 (defmacro unbound-marker-bits ()
   (logior (+ sb-vm:static-space-start #x100) unbound-marker-widetag))
+
+;;; See WRITE-FUNINSTANCE-PROLOGUE in x86-64-vm.
+;;; There are 4 bytes available in the imm32 operand of a dummy MOV instruction.
+;;; (It's a valid instruction on the theory that illegal opcodes might cause
+;;; the decode stage in the CPU to behave suboptimally)
+(defmacro compact-fsc-instance-hash (fin)
+  `(sap-ref-32 (int-sap (get-lisp-obj-address ,fin))
+               (+ (ash 3 word-shift) 4 (- fun-pointer-lowtag))))
+
+(eval-when (:compile-toplevel)
+  (let ((locs (sb-c::sc-locations (gethash 'any-reg sb-c:*backend-sc-names*))))
+    ;; These locations are not saved by WITH-REGISTERS-PRESERVED
+    ;; because Lisp can't treat them as general purpose.
+    ;; By design they are also (i.e. must be) nonvolatile aross C call.
+    (aver (not (logbitp 12 locs)))
+    #-gs-seg (aver (not (logbitp 13 locs)))))
