@@ -16,7 +16,7 @@
 #include "sbcl.h"
 #include "alloc.h"
 #include "thread.h"
-#include "getallocptr.h"
+#include "pseudo-atomic.h"
 #include "genesis/code.h"
 
 lispobj* atomic_bump_static_space_free_ptr(int nbytes)
@@ -51,12 +51,6 @@ pthread_mutex_t alloc_profiler_lock = PTHREAD_MUTEX_INITIALIZER;
 #include <stdio.h>
 #include "genesis/vector.h"
 
-// Counters 0 and 1 are reserve for variable-size allocations
-// (hit count and total size) that overflow the maximum counter index.
-// Counter 2 is reserved for fixed-size allocations.
-// Constant-size allocations consume 1 entry (hit count)
-// Variable-size consume 2 (hit count and total size).
-unsigned int alloc_profile_n_counters = 3;
 unsigned int max_alloc_point_counters;
 
 void allocation_profiler_start()
@@ -115,33 +109,4 @@ void allocation_profiler_stop()
     }
     ret = mutex_release(&alloc_profiler_lock);
     gc_assert(ret);
-#if 0
-    if (warning_issued) {
-        fprintf(stderr, "allocation profile needed %d counters\n",
-                alloc_profile_n_counters);
-        warning_issued = 0;
-    }
-#endif
 }
-
-#ifdef LISP_FEATURE_METASPACE
-#include "gc-private.h"
-lispobj valid_metaspace_ptr_p(void* addr)
-{
-    struct slab_header* slab = (void*)ALIGN_DOWN((lispobj)addr, METASPACE_SLAB_SIZE);
-    fprintf(stderr, "slab base %p chunk_size %d capacity %d\n", slab, slab->chunksize, slab->capacity);
-    if (!slab->capacity) return 0;
-    lispobj slab_end = (lispobj)slab + METASPACE_SLAB_SIZE;
-    int index = (slab_end - (lispobj)addr) / slab->chunksize;
-    //    for(int i=0; i<slab->capacity; ++i) fprint(stderr, "goober @ %p\n", slab_end - (1+i)*chunksize);
-    //    fprintf(stderr, "index=%d\n", index);
-    if (index < slab->capacity) {
-        lispobj* obj_base = (lispobj*)(slab_end - (index+1)*slab->chunksize);
-        if (widetag_of(obj_base) == INSTANCE_WIDETAG) {
-            fprintf(stderr, "word @ %p is good\n", obj_base);
-            return (lispobj)obj_base;
-        }
-    }
-    return 0;
-}
-#endif
