@@ -81,6 +81,9 @@ static void dragonfly_init();
 #ifdef LISP_FEATURE_X86
 #include <machine/cpu.h>
 #endif
+#ifdef LISP_FEATURE_SB_FUTEX
+#include <sys/futex.h>
+#endif
 
 static void openbsd_init();
 #endif
@@ -117,18 +120,24 @@ os_context_sigmask_addr(os_context_t *context)
 }
 
 os_vm_address_t
-os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id)
+os_alloc_gc_space(int space_id, int attributes, os_vm_address_t addr, os_vm_size_t len)
 {
     int __attribute((unused))
       executable = (space_id == READ_ONLY_CORE_SPACE_ID) ||
-                   (space_id == LINKAGE_TABLE_CORE_SPACE_ID) ||
+                   (space_id == ALIEN_LINKAGE_TABLE_CORE_SPACE_ID) ||
                    (space_id == STATIC_CODE_CORE_SPACE_ID),
       jit = (space_id == STATIC_CODE_CORE_SPACE_ID) || (space_id == DYNAMIC_CORE_SPACE_ID)
-            ? 1 : (space_id == LINKAGE_TABLE_CORE_SPACE_ID) ? 2 : 0;
+            ? 1 : (space_id == ALIEN_LINKAGE_TABLE_CORE_SPACE_ID) ? 2 : 0;
 
     int protection;
     int flags = 0;
 
+#if defined(LISP_FEATURE_OPENBSD) && defined(MAP_STACK)
+        /* OpenBSD requires MAP_STACK for pages used as stack.
+         * Note that FreeBSD has a MAP_STACK with different behavior. */
+    if (space_id == THREAD_STRUCT_CORE_SPACE_ID) flags = MAP_STACK;
+#endif
+
     // FIXME: This probaby needs to use MAP_TRYFIXED
     if (attributes & IS_GUARD_PAGE)
         protection = OS_VM_PROT_NONE;
@@ -154,7 +163,6 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id
     attributes &= ~IS_GUARD_PAGE;
 
 #ifndef LISP_FEATURE_DARWIN // Do not use MAP_FIXED, because the OS is sane.
-
     /* The *BSD family of OSes seem to ignore 'addr' when it is outside
      * of some range which I could not figure out.  Sometimes it seems like the
      * condition is that any address below 4GB can't be requested without MAP_FIXED,
@@ -182,16 +190,7 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id
        Except for MAP_FIXED mappings, the system will never replace existing mappings. */
 
     // ALLOCATE_LOW seems never to get what we want
-    if (!(attributes & MOVABLE) || (attributes & ALLOCATE_LOW)) {
-        flags = MAP_FIXED;
-    }
-    if (attributes & IS_THREAD_STRUCT) {
-#if defined(LISP_FEATURE_OPENBSD) && defined(MAP_STACK)
-        /* OpenBSD requires MAP_STACK for pages used as stack.
-         * Note that FreeBSD has a MAP_STACK with different behavior. */
-        flags = MAP_STACK;
-#endif
-    }
+    if (!(attributes & MOVABLE) || (attributes & ALLOCATE_LOW)) flags = MAP_FIXED;
 #endif
 
 #ifdef MAP_EXCL // not defined in OpenBSD, NetBSD, DragonFlyBSD
@@ -247,13 +246,6 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id
 
     return addr;
 }
-
-void
-os_invalidate(os_vm_address_t addr, os_vm_size_t len)
-{
-    if (munmap(addr, len) == -1)
-        perror("munmap");
-}
 
 /*
  * any OS-dependent special low-level handling for signals
@@ -287,27 +279,15 @@ memory_fault_handler(int signal, siginfo_t *siginfo, os_context_t *context)
             lisp_memory_fault_error(context, fault_addr);
 }
 
-#if defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER)
-void
-mach_error_memory_fault_handler(int signal, siginfo_t *siginfo,
-                                os_context_t *context) {
-    lose("Unhandled memory fault. Exiting.");
-}
-#endif
-
 void
 os_install_interrupt_handlers(void)
 {
     if (INSTALL_SIG_MEMORY_FAULT_HANDLER) {
-#if defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER)
-    ll_install_handler(SIG_MEMORY_FAULT, mach_error_memory_fault_handler);
-#else
     ll_install_handler(SIG_MEMORY_FAULT,
 #if defined(LISP_FEATURE_FREEBSD) && !defined(__GLIBC__)
                                                  (__siginfohandler_t *)
 #endif
                                                  memory_fault_handler);
-#endif
 
 #ifdef LISP_FEATURE_DARWIN
     /* Unmapped pages get this and not SIGBUS. */
@@ -430,17 +410,17 @@ static void freebsd_init()
 
 #ifdef LISP_FEATURE_SB_FUTEX
 int
-futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
+futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
 {
     struct timespec timeout;
     int ret;
 
     if (sec < 0)
-        ret = _umtx_op((void *)lock_word, UMTX_OP_WAIT, oldval, 0, 0);
+        ret = _umtx_op(lock_word, UMTX_OP_WAIT_UINT_PRIVATE, oldval, 0, 0);
     else {
         timeout.tv_sec = sec;
         timeout.tv_nsec = usec * 1000;
-        ret = _umtx_op((void *)lock_word, UMTX_OP_WAIT, oldval, (void*)sizeof timeout, &timeout);
+        ret = _umtx_op(lock_word, UMTX_OP_WAIT_UINT_PRIVATE, oldval, (void*)sizeof timeout, &timeout);
     }
     if (ret == 0) return 0;
     // technically we would not need to check any of the error codes if the lisp side
@@ -453,7 +433,7 @@ futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
 int
 futex_wake(int *lock_word, int n)
 {
-    return _umtx_op((void *)lock_word, UMTX_OP_WAKE, n, 0, 0);
+    return _umtx_op(lock_word, UMTX_OP_WAKE_PRIVATE, n, 0, 0);
 }
 #endif
 #endif /* __FreeBSD__ */
@@ -602,7 +582,7 @@ The system may fail to start.\n",
      */
     getrlimit (RLIMIT_DATA, &rl);
     if (dynamic_space_size + READ_ONLY_SPACE_SIZE + STATIC_SPACE_SIZE +
-        LINKAGE_TABLE_SPACE_SIZE + wantfree > rl.rlim_cur)
+        ALIEN_LINKAGE_TABLE_SPACE_SIZE + wantfree > rl.rlim_cur)
         fprintf (stderr,
                  "RUNTIME WARNING: data size resource limit may be too low,\n"
                  "  try decreasing the dynamic space size with --dynamic-space-size\n"
@@ -624,4 +604,25 @@ os_dlsym(void *handle, const char *symbol)
     return ret;
 }
 
+#ifdef LISP_FEATURE_SB_FUTEX
+int
+futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
+{
+    struct timespec timeout = {.tv_sec = sec, .tv_nsec = usec * 1000};
+    int ret = futex(lock_word, FUTEX_WAIT_PRIVATE, oldval, sec < 0 ? NULL : &timeout, NULL);
+
+    if (ret == 0) return 0;
+    if (errno == ETIMEDOUT) return 1;
+    if (errno == EINTR) return 2;
+    return -1;
+}
+
+int
+futex_wake(int *lock_word, int n)
+{
+    futex(lock_word, FUTEX_WAKE_PRIVATE, n, NULL, NULL);
+    return 0;
+}
+#endif
+
 #endif
