@@ -41,6 +41,7 @@
 #include "gc-internal.h"
 #include "gc-private.h"
 #include "code.h"
+#include "graphvisit.h"
 
 #include <errno.h>
 
@@ -377,6 +378,26 @@ static inline void fix_fun_header_layout(lispobj __attribute__((unused)) *fun,
 #endif
 }
 
+/* Refer to the figures above CALCULATE-DD-BITMAP in src/code/defstruct
+ * for the meaning of these constants.
+ * I think this fixes what may have been a latent bug in space relocation.
+ * Due to inconsistent use of the bitmap, it could have processed funinstance
+ * LAYOUT slots twice.  Layout is supposed to have a 0 bit, meaning do NOT scan
+ * in a loop over slots, but instead externally to the loop. By marking it as 1,
+ * it could be processed both outside and inside the loop. In the extremely rare
+ * case of partial overlap of the desired and actual heap addresses, a pointer
+ * somewhere in the middle can not be discerned as having been fixed up or not
+ * fixed up based on its value. You have to know based on code flow.
+ * Therefore layouts slots always have to be treated as raw in the bitmap,
+ * but I did not see where it is enforced in lisp for funinstances. */
+#ifndef LISP_FEATURE_EXECUTABLE_FUNINSTANCES
+# define FUNINSTANCE_BITMAP -4
+#elif defined LISP_FEATURE_COMPACT_INSTANCE_HEADER
+# define FUNINSTANCE_BITMAP -8
+#else
+# define FUNINSTANCE_BITMAP -24
+#endif
+
 static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 {
     lispobj *where = (lispobj*)start;
@@ -398,14 +419,6 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
         widetag = header_widetag(word);
         nwords = sizetab[widetag](where);
         switch (widetag) {
-        case FUNCALLABLE_INSTANCE_WIDETAG:
-            // Special note on the word at where[1] in funcallable instances:
-            // - If no immobile code, then the word points to read-only space,
-            ///  hence needs no adjustment.
-            // - Otherwise, the word might point to a relocated range,
-            //   either the instance itself, or a trampoline in immobile space.
-            adjust_word_at(where+1, adj);
-            /* FALLTHROUGH */
         case INSTANCE_WIDETAG:
             layout = layout_of(where);
             adjusted_layout = adjust_word(adj, layout);
@@ -413,11 +426,32 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             // so it would not be fixed up otherwise.
             if (adjusted_layout != layout) layout_of(where) = adjusted_layout;
             struct bitmap bitmap = get_layout_bitmap(LAYOUT(adjusted_layout));
+            {
             lispobj* slots = where+1;
-            for (i=0; i<(nwords-1); ++i)
+            for (i=0; i<(nwords-1); ++i) // -1 from nwords because 'slots' is +1 from 'where'
                 if (bitmap_logbitp(i, bitmap)) adjust_pointers(slots+i, 1, adj);
+            }
             continue;
-          case SYMBOL_WIDETAG:
+        case FUNCALLABLE_INSTANCE_WIDETAG:
+            /* If non-executable funinstance, then word index 1 points at read-only space,
+             * hence needs no adjustment. Otherwise, the word points within the funinstance.
+             * Either way, adjust_word_at will do the right thing */
+            adjust_word_at(where+1, adj);
+            /* We have to adjust the layout, but its bitmap can be disregarded.
+             * There is no variation in bitmap by particular subtype of funinstance */
+            layout = funinstance_layout(where);
+            adjusted_layout = adjust_word(adj, layout);
+            if (adjusted_layout != layout) funinstance_layout(where) = adjusted_layout;
+            const int mask = FUNINSTANCE_BITMAP;
+            gc_assert((mask & (1<<0)) == 0); // bitmap must reflect it as untagged
+            {
+            lispobj* slots = where+1;
+            // slots[0] is where[1] which we just adjusted, so skip it.
+            for (i=1; i<(nwords-1); ++i) // -1 from nwords because 'slots' is +1 from 'where'
+                if ((1<<i) & mask) adjust_pointers(slots+i, 1, adj);
+            }
+            continue;
+        case SYMBOL_WIDETAG:
             { // Copied from scav_symbol() in gc-common
             struct symbol* s = (void*)where;
             adjust_pointers(&s->value, 3, adj); // value, fdefn, info
@@ -586,12 +620,7 @@ static void relocate_heap(struct heap_adjust* adj)
 /// Compute the bounds of the lisp assembly routine code object
 void calc_asm_routine_bounds()
 {
-#ifdef LISP_FEATURE_METASPACE
-    if (widetag_of((lispobj*)READ_ONLY_SPACE_START) == CODE_HEADER_WIDETAG)
-        asm_routines_start = READ_ONLY_SPACE_START;
-    else
-        asm_routines_start = READ_ONLY_SPACE_START + (256+2)*N_WORD_BYTES;
-#elif defined LISP_FEATURE_IMMOBILE_CODE
+#if defined LISP_FEATURE_IMMOBILE_CODE
     asm_routines_start = TEXT_SPACE_START;
 #else
     if ((uword_t)read_only_space_free_pointer > READ_ONLY_SPACE_START &&
@@ -1066,37 +1095,25 @@ void asm_routine_poke(const char* routine, int offset, char byte)
         address[offset] = byte;
 }
 
-// Caution: use at your own risk
-#if defined DEBUG_CORE_LOADING && DEBUG_CORE_LOADING
-#include "hopscotch.h"
-#include "genesis/cons.h"
-#include "genesis/layout.h"
-#include "genesis/gc-tables.h"
-#include "code.h"
-#include "gc-private.h"
+static void trace_sym(lispobj, struct symbol*, struct grvisit_context*);
 
-struct visitor {
-    // one item per value of widetag>>2
-    // element 0 is for conses, element 64 is for totals.
-    struct {
-        int count;
-        int words;
-    } headers[65], sv_subtypes[3];
-    struct hopscotch_table *reached;
-};
+#define RECURSE(x) if(is_lisp_pointer(x))graph_visit(ptr,x,context)
 
-static void trace_sym(lispobj, struct symbol*, struct hopscotch_table*);
-#define RECURSE(x) if(is_lisp_pointer(x))graph_visit(ptr,x,seen)
-static void graph_visit(lispobj __attribute__((unused)) referer,
-                        lispobj ptr,
-                        struct hopscotch_table* seen)
+/* Despite this being a nice concise expression of a pointer tracing algorithm,
+ * it turns out to be almost unusable in any sufficiently complicated object graph
+ * due to stack overflow. The pristine core alone hits a recursion depth of >8000. */
+static void graph_visit(lispobj referer, lispobj ptr, struct grvisit_context* context)
 {
+#define ILLEGAL_WORD 0xFFFFFFFFDEADBEEF
+    if (ptr == ILLEGAL_WORD) lose("object %"OBJ_FMTX" contains a garbage word", referer);
     if (lowtag_of(ptr) == FUN_POINTER_LOWTAG
         && widetag_of(FUNCTION(ptr)) == SIMPLE_FUN_WIDETAG)
         ptr = fun_code_tagged(FUNCTION(ptr));
-    if (hopscotch_get(seen, ptr, 0))
-        return;
-    hopscotch_insert(seen, ptr, 1);
+    if (hopscotch_get(context->seen, ptr, 0)) return;
+    if (++context->depth > context->maxdepth) context->maxdepth = context->depth;
+    // TODO: add rejection function for off-heap objects as part of supplied context
+    hopscotch_insert(context->seen, ptr, 1);
+    if (context->action) context->action(ptr, context->data);
     lispobj layout, *obj;
     sword_t nwords, i;
     if (lowtag_of(ptr) == LIST_POINTER_LOWTAG) {
@@ -1113,7 +1130,7 @@ static void graph_visit(lispobj __attribute__((unused)) referer,
         case INSTANCE_WIDETAG:
         case FUNCALLABLE_INSTANCE_WIDETAG:
             layout = layout_of(obj);
-            graph_visit(ptr, layout, seen);
+            graph_visit(ptr, layout, context);
             nwords = headerobj_size(obj);
             struct bitmap bitmap = get_layout_bitmap(LAYOUT(layout));
             for (i=0; i<(nwords-1); ++i)
@@ -1129,7 +1146,7 @@ static void graph_visit(lispobj __attribute__((unused)) referer,
         // then we scan word indices 1 and 2 off the object base address.
         case CLOSURE_WIDETAG:
             // We must scan the closure's trampoline word.
-            graph_visit(ptr, fun_taggedptr_from_self(obj[1]), seen);
+            graph_visit(ptr, fun_taggedptr_from_self(obj[1]), context);
             // Closures can utilize one payload word beyond what the header
             // indicates. This is quite sucky and I don't know why I did that.
             // However, it is correctly accounted for by SHORT_BOXED_NWORDS
@@ -1138,7 +1155,7 @@ static void graph_visit(lispobj __attribute__((unused)) referer,
             for(i=2; i<=nwords; ++i) RECURSE(obj[i]);
             break;
         case SYMBOL_WIDETAG:
-            trace_sym(ptr, SYMBOL(ptr), seen);
+            trace_sym(ptr, SYMBOL(ptr), context);
             break;
         case WEAK_POINTER_WIDETAG:
             nwords = TINY_BOXED_NWORDS(*obj);
@@ -1155,8 +1172,10 @@ static void graph_visit(lispobj __attribute__((unused)) referer,
                 for(i=1; i<size; ++i) RECURSE(obj[i]);
             }
       }
+      --context->depth;
 }
-static void trace_sym(lispobj ptr, struct symbol* sym, struct hopscotch_table* seen)
+
+static void trace_sym(lispobj ptr, struct symbol* sym, struct grvisit_context* context)
 {
     RECURSE(decode_symbol_name(sym->name));
     RECURSE(sym->value);
@@ -1164,14 +1183,57 @@ static void trace_sym(lispobj ptr, struct symbol* sym, struct hopscotch_table* s
     RECURSE(sym->fdefn);
 }
 
+/* Caller must provide an uninitialized hopscotch table.
+ * This function will initialize it and perform a graph visit.
+ * Caller may subsequently inspect the table and/or visit other objects as
+ * dictated by thread stacks, etc. Caller may - but need not - provide
+ * an 'action' to invoke on each object */
+struct grvisit_context*
+visit_heap_from_static_roots(struct hopscotch_table* reached,
+                             void (*action)(lispobj, void*),
+                             void* data)
+{
+    hopscotch_create(reached, HOPSCOTCH_HASH_FUN_DEFAULT,
+                     0, // no values
+                     1<<18, /* initial size */
+                     0);
+
+    struct grvisit_context* context = malloc(sizeof (struct grvisit_context));
+    context->seen = reached;
+    context->action = action;
+    context->data = data;
+    context->depth = context->maxdepth = 0;
+    trace_sym(NIL, SYMBOL(NIL), context);
+    lispobj* where = (lispobj*)STATIC_SPACE_OBJECTS_START;
+    lispobj* end = static_space_free_pointer;
+    while (where<end) {
+        graph_visit(0, compute_lispobj(where), context);
+        where += object_size(where);
+    }
+    return context;
+}
+
+// Caution: use at your own risk
+#if defined DEBUG_CORE_LOADING && DEBUG_CORE_LOADING
+struct visitor {
+    // one item per value of widetag>>2
+    // element 0 is for conses, element 64 is for totals.
+    struct {
+        int count;
+        int words;
+    } headers[65], sv_subtypes[3];
+    struct hopscotch_table *reached;
+};
+
 static void tally(lispobj ptr, struct visitor* v)
 {
+    sword_t words;
     if (lowtag_of(ptr) == LIST_POINTER_LOWTAG)
-        ++v->headers[0].count;
+        ++v->headers[0].count, words = 2;
     else {
         lispobj* obj = native_pointer(ptr);
         lispobj header = *obj;
-        sword_t words = object_size2(obj, header);
+        words = object_size2(obj, header);
         int widetag = header_widetag(header);
         int header_index = widetag>>2;
         ++v->headers[header_index].count;
@@ -1224,25 +1286,16 @@ static void sanity_check_loaded_core(lispobj initial_function)
     struct hopscotch_table reached;
     memset(v, 0, sizeof v);
     // Pass 1: Count objects reachable from known roots.
-    hopscotch_create(&reached, HOPSCOTCH_HASH_FUN_DEFAULT,
-                     0, // no values
-                     1<<18, /* initial size */
-                     0);
-    {
-      trace_sym(NIL, SYMBOL(NIL), &reached);
-      lispobj* where = (lispobj*)STATIC_SPACE_OBJECTS_START;
-      lispobj* end = static_space_free_pointer;
-      while (where<end) {
-        graph_visit(0, compute_lispobj(where), &reached);
-        where += object_size(where);
-      }
-    }
-    graph_visit(0, initial_function, &reached); // not otherwise reachable
+    struct grvisit_context* c
+        = visit_heap_from_static_roots(&reached, 0, 0);
+    graph_visit(0, initial_function, c); // initfun is not otherwise reachable
     // having computed the reaching graph, tally up the dynamic space objects
     int key_index;
     lispobj ptr;
     for_each_hopscotch_key(key_index, ptr, reached)
         if (count_this_pointer_p(ptr)) tally(ptr, &v[0]);
+    printf("graphvisit.maxdepth=%d\n", c->maxdepth);
+    free(c);
     // Pass 2: Count all heap objects
     v[1].reached = &reached;
     walk_generation(visit, -1, (uword_t)&v[1]);
