@@ -15,14 +15,15 @@
 #include "gencgc-private.h"
 #include "genesis/gc-tables.h"
 #include "genesis/closure.h"
-#include "genesis/cons.h"
 #include "genesis/instance.h"
 #include "genesis/vector.h"
 #include "genesis/layout.h"
 #include "genesis/hash-table.h"
+#include "genesis/list-node.h"
 #include "code.h"
 #include "immobile-space.h"
 #include "queue.h"
+#include "os.h"
 
 #include <stdio.h>
 #ifndef LISP_FEATURE_WIN32
@@ -32,47 +33,8 @@
 #include <sys/resource.h> // for getrusage()
 #endif
 
-
-/* Most headered objects use MARK_BIT to record liveness.
- * Bignums always use the leftmost bit regardless of word size.
- * Fdefns use 0x4000 which overlaps the 'written' bit in the generation byte,
- * but 'written' is not used except for code objects, so this is fine.
- *
- * Bit 31 of the header is the mark bit for all remaining object types.
- * This avoids clash with the layout pointer of instances and functions,
- * the TLS index of symbols, and various other bits.
- * The mark bit occupies the same byte as the generation number
- * in immobile space, but doesn't conflict with that usage.
- */
-#define MARK_BIT ((uword_t)1 << 31)
-#define FDEFN_MARK_BIT 0x4000
-#ifdef LISP_FEATURE_64_BIT
-#define BIGNUM_MARK_BIT ((uword_t)1 << 63)
-#else
-#define BIGNUM_MARK_BIT MARK_BIT
-#endif
-
 extern lispobj lisp_init_function, gc_object_watcher;
 
-static inline uword_t boxedobj_widetag_to_markbit(int widetag) {
-    return (widetag == FDEFN_WIDETAG || widetag == WEAK_POINTER_WIDETAG)
-         ? FDEFN_MARK_BIT : MARK_BIT;
-}
-static inline uword_t general_widetag_to_markbit(int widetag) {
-    return (widetag == FDEFN_WIDETAG || widetag == WEAK_POINTER_WIDETAG)
-        ? FDEFN_MARK_BIT : ((widetag == BIGNUM_WIDETAG) ? BIGNUM_MARK_BIT : MARK_BIT);
-}
-
-#define interesting_pointer_p(x) \
-  (find_page_index((void*)x) >= 0 || immobile_space_p(x))
-
-#ifdef DEBUG
-#  define dprintf(arg) printf arg
-FILE * logfile;
-#else
-#  define dprintf(arg)
-#endif
-
 struct unbounded_queue {
   struct Qblock* head_block;
   struct Qblock* tail_block;
@@ -88,55 +50,37 @@ static page_index_t free_page;
  * If it collides with 'next_free_page', then you lose.
  * TODO: It would be reasonably simple to have this request more memory from
  * the OS instead of failing on overflow */
-static void* get_free_page() {
+static void* get_free_page(boolean prezero) {
     --free_page;
     if (free_page < next_free_page)
         lose("Needed more space to GC");
     page_table[free_page].type = PAGE_TYPE_UNBOXED;
     char* mem = page_address(free_page);
-    zeroize_pages_if_needed(free_page, free_page, 0);
-    return mem;
-}
-
-/* The suballocator doles out blocks of bits for marking conses live.
- * Example: If pages are 32768 bytes, and Lisp words are 8 bytes,
- * then one GC page can hold 2K cons cells.
- * One byte marks 8 conses (1 bit per cons), 256 bytes mark 2048 conses.
- * 128 blocks of 256 bytes fit on a 32K GC page. */
-static char *suballocator_free_ptr, *suballocator_end_ptr;
-
-#ifndef LISP_FEATURE_USE_CONS_REGION
-static inline uword_t compute_page_key(lispobj cons) {
-    return ALIGN_DOWN(cons, GENCGC_PAGE_BYTES);
-}
-static void* allocate_cons_mark_bits() {
-    int nbytes = GENCGC_PAGE_BYTES / (2 * N_WORD_BYTES) / 8;
-    if (suballocator_free_ptr + nbytes > suballocator_end_ptr) {
-        suballocator_free_ptr = get_free_page();
-        suballocator_end_ptr = suballocator_free_ptr + GENCGC_PAGE_BYTES;
-    }
-    void* mem = suballocator_free_ptr;
-    suballocator_free_ptr += nbytes;
-    return mem;
-}
+#ifdef LISP_FEATURE_DARWIN_JIT
+    prezero = 1; // Might need to alter MMU-based protection
+#elif defined LISP_FEATURE_WIN32
+    os_commit_memory(page_address(free_page), GENCGC_PAGE_BYTES);
 #endif
+    if (prezero) zeroize_pages_if_needed(free_page, free_page, 0);
+    return mem;
+}
 
 static void gc_enqueue(lispobj object)
 {
     gc_dcheck(is_lisp_pointer(object));
+    gc_dcheck(widetag_of(native_pointer(object)) != SIMPLE_FUN_WIDETAG);
     struct Qblock* block = scav_queue.tail_block;
     if (block->count == QBLOCK_CAPACITY) {
         struct Qblock* next;
         next = scav_queue.recycler;
         if (next) {
             scav_queue.recycler = next->next;
-            next->next = 0;
-            dprintf(("Popped recycle list\n"));
         } else {
-            next = (struct Qblock*)get_free_page();
-            dprintf(("Alloc'd new block\n"));
+            next = (struct Qblock*)get_free_page(0);
         }
         block = block->next = next;
+        block->next = 0;
+        block->tail = block->count = 0;
         scav_queue.tail_block = block;
     }
     block->elements[block->tail] = object;
@@ -156,115 +100,131 @@ static lispobj gc_dequeue()
             scav_queue.head_block = next;
             block->next = scav_queue.recycler;
             scav_queue.recycler = block;
-            dprintf(("Qblock emptied - returned to recycle list\n"));
-        } else {
-            dprintf(("Qblock emptied - NOT returned to recycle list\n"));
         }
     }
     return object;
 }
 
-/* The 'mark_bits' hashtable maps a page address to a block of mark bits
- * for headerless objects (conses) */
-struct hopscotch_table mark_bits;
-
-static inline int compute_dword_number(lispobj cons) {
-    return (cons & (GENCGC_PAGE_BYTES - 1)) >> (1+WORD_SHIFT);
+static inline sword_t dword_index(uword_t ptr, uword_t base) {
+    return (ptr - base) >> (1+WORD_SHIFT);
 }
 
-static inline int cons_markedp(lispobj pointer) {
-    unsigned int index = compute_dword_number(pointer);
-#ifdef LISP_FEATURE_USE_CONS_REGION /* all conses must be on cons pages */
-    unsigned char* bits = (unsigned char*)ALIGN_DOWN(pointer, GENCGC_PAGE_BYTES)
-                          + CONS_PAGE_USABLE_BYTES;
-#else
-    unsigned char* bits = (unsigned char*)
-        hopscotch_get(&mark_bits, compute_page_key(pointer), 0);
-    if (!bits) return 0;
+/* The "canonical" pointer to an object is usually just the object itself.
+ * This is true even for SIMPLE-FUN- we don't need to regard only the code base
+ * as canonical. The exception is that LRAs can't be marked because they can't
+ * be discovered and marked when marking their containing code */
+static inline lispobj canonical_ptr(lispobj pointer)
+{
+#ifdef RETURN_PC_WIDETAG
+  /* NO_TLS_VALUE is all 1s, and so it might look like it has OTHER_POINTER_LOWTAG
+   * depending on the architecture (the word size, etc), but there is no memory
+   * at 0xff...ff so definitely don't call widetag_of - that won't fly! */
+    if (lowtag_of(pointer)==OTHER_POINTER_LOWTAG
+        && pointer != NO_TLS_VALUE_MARKER
+        && widetag_of(native_pointer(pointer)) == RETURN_PC_WIDETAG)
+        return fun_code_tagged(native_pointer(pointer));
 #endif
-    return (bits[index / 8] >> (index % 8)) & 1;
+    return pointer;
 }
 
+sword_t fixedobj_index_bit_bias, text_index_bit_bias;
+uword_t *fullcgcmarks;
+static size_t markbits_size;
+static inline sword_t ptr_to_bit_index(lispobj pointer) {
+    if (pointer == NIL) return -1;
+    page_index_t p = find_page_index((void*)pointer);
+    if (p >= 0) return dword_index(pointer, DYNAMIC_SPACE_START);
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    p = find_fixedobj_page_index((void*)pointer);
+    if (p >= 0) return dword_index(pointer, FIXEDOBJ_SPACE_START) + fixedobj_index_bit_bias;
+    p = find_text_page_index((void*)pointer);
+    if (p >= 0) return dword_index(pointer, TEXT_SPACE_START) + text_index_bit_bias;
+#endif
+    return -1;
+}
+#define interesting_pointer_p(x) ptr_to_bit_index(x)>=0
+
 /* Return true if OBJ has already survived the current GC. */
 static inline int pointer_survived_gc_yet(lispobj pointer)
 {
-    if (!interesting_pointer_p(pointer))
-        return 1;
-    if (listp(pointer))
-        return cons_markedp(pointer);
-    lispobj header = *native_pointer(pointer);
-    int widetag = header_widetag(header);
-    switch (widetag) {
-    case BIGNUM_WIDETAG: return (header & BIGNUM_MARK_BIT) != 0;
-    case WEAK_POINTER_WIDETAG:
-    case FDEFN_WIDETAG: return (header & FDEFN_MARK_BIT) != 0;
-    }
-    if (embedded_obj_p(widetag))
-        header = *fun_code_header(native_pointer(pointer));
-    return (header & MARK_BIT) != 0;
+    sword_t mark_index = ptr_to_bit_index(canonical_ptr(pointer));
+    if (mark_index < 0) return 1; // "uninteresting" objects always survive GC
+    return (fullcgcmarks[mark_index / N_WORD_BITS] >> (mark_index % N_WORD_BITS)) & 1;
 }
 
-void __mark_obj(lispobj pointer)
-{
-    gc_dcheck(is_lisp_pointer(pointer));
-    if (!interesting_pointer_p(pointer))
-        return;
-    if (!listp(pointer)) {
-        lispobj* base = native_pointer(pointer);
-        lispobj header = *base;
-        int widetag = header_widetag(header);
-        if (widetag == BIGNUM_WIDETAG) {
-            if (points_to_asm_code_p(pointer)) return;
-            *base |= BIGNUM_MARK_BIT;
-            return; // don't enqueue - no pointers
-        } else {
-            if (embedded_obj_p(widetag)) {
-                base = fun_code_header(base);
-                pointer = make_lispobj(base, OTHER_POINTER_LOWTAG);
-                header = *base;
+int fullcgc_lispobj_livep(lispobj pointer) {
+    return pointer_survived_gc_yet(pointer);
+}
+
+void dump_marked_objects() {
+    fprintf(stderr, "Marked objects:\n");
+    page_index_t first = 0;
+    int n = 0;
+    while (first < next_free_page) {
+        page_index_t last = contiguous_block_final_page(first);
+        lispobj* where = (lispobj*)page_address(first);
+        lispobj* limit = (lispobj*)page_address(last) + page_words_used(last);
+        while (where < limit) {
+            lispobj obj = compute_lispobj(where);
+            if (pointer_survived_gc_yet(obj)) {
+                ++n;
+                fprintf(stderr, " %"OBJ_FMTX"\n", obj);
             }
-            uword_t markbit = boxedobj_widetag_to_markbit(widetag);
-            if (header & markbit) return; // already marked
-            *base |= markbit;
+            where += object_size(where);
         }
-#ifdef LISP_FEATURE_UBSAN
-        if (specialized_vector_widetag_p(widetag) && is_lisp_pointer(base[1]))
-            gc_mark_obj(base[1]);
-        else if (widetag == SIMPLE_VECTOR_WIDETAG && fixnump(base[1])) {
-            char *origin_pc = (char*)(base[1]>>4);
-            lispobj* code = component_ptr_from_pc(origin_pc);
-            if (code) gc_mark_obj(make_lispobj(code, OTHER_POINTER_LOWTAG));
-            /* else lose("can't find code containing %p (vector=%p)", origin_pc, base); */
-        }
-#endif
-        if (leaf_obj_widetag_p(widetag)) return;
-    } else {
-        unsigned int index = compute_dword_number(pointer);
-#ifdef LISP_FEATURE_USE_CONS_REGION /* all conses must be on cons pages */
-        unsigned char* bits = (unsigned char*)ALIGN_DOWN(pointer, GENCGC_PAGE_BYTES)
-                              + CONS_PAGE_USABLE_BYTES;
-#else
-        /* Only _some_ conses are on cons pages depending if they got moved there.
-         * We could use a hybrid of storing mark bits on the page if possible,
-         * or the hash-table if not. However, I'd rather not complicate things.
-         * All architectures should be made to allocate directly on cons pages! */
-        uword_t key = compute_page_key(pointer);
-        unsigned char* bits = (unsigned char*)hopscotch_get(&mark_bits, key, 0);
-        if (!bits) {
-            bits = allocate_cons_mark_bits();
-            hopscotch_insert(&mark_bits, key, (sword_t)bits);
-        }
-#endif
-        if (bits[index / 8] & (1 << (index % 8))) return;
-        // Mark the cons
-        bits[index / 8] |= 1 << (index % 8);
+        first = 1 + last;
     }
-    gc_enqueue(pointer);
+    fprintf(stderr, "Total: %d\n", n);
+}
+
+/* If stray pointer detection is being performed, then all weak references
+ * (weak pointers, weak hash tables) are treated as strong.
+ * This informs the consumer whether it is possible to reach an object
+ * that satisfies the test given the current heap state */
+lispobj stray_pointer_source_obj;
+int (*stray_pointer_detector_fn)(lispobj); // return value is unused
+static void __mark_obj(lispobj pointer)
+{
+    lispobj* base;
+
+    pointer = canonical_ptr(pointer);
+    sword_t mark_index = ptr_to_bit_index(pointer);
+    if (mark_index < 0) {
+        if (stray_pointer_detector_fn) stray_pointer_detector_fn(pointer);
+        return; // uninteresting pointer
+    }
+    uword_t wordindex = mark_index / N_WORD_BITS;
+    uword_t bit = (uword_t)1 << (mark_index % N_WORD_BITS);
+    if (fullcgcmarks[wordindex] & bit) return; // already marked
+    if (lowtag_of(pointer) == FUN_POINTER_LOWTAG
+        && embedded_obj_p(widetag_of(FUNCTION(pointer)))) {
+        lispobj* code = fun_code_header(FUNCTION(pointer));
+        mark_index -= ((char*)FUNCTION(pointer) - (char*)code) >> (1+WORD_SHIFT);
+        pointer = make_lispobj(code, OTHER_POINTER_LOWTAG);
+        base = code;
+        wordindex = mark_index / N_WORD_BITS;
+        bit = (uword_t)1 << (mark_index % N_WORD_BITS);
+        if (fullcgcmarks[wordindex] & bit) return; // already marked
+    } else
+        base = native_pointer(pointer);
+    fullcgcmarks[wordindex] |= bit;
+    // FIXME: restore the code for #ifdef LISP_FEATURE_UBSAN
+    if (widetag_of(base) == CODE_HEADER_WIDETAG) {
+        struct code* code = (void*)base;
+        /* mark all simple-funs which speeds up pointer_survived_gc_yet.
+         * Just add the offset in dwords from base to each fun to compute
+         * the mark bit index (rather than calling ptr_to_bit_index) */
+        for_each_simple_fun(i, fun, code, 0, {
+            unsigned int offset = ((char*)fun - (char*)base) >> (1+WORD_SHIFT);
+            uword_t funmark = mark_index + offset;
+            fullcgcmarks[funmark / N_WORD_BITS] |= (uword_t)1 << (funmark % N_WORD_BITS);
+        })
+    }
+    if (listp(pointer) || !leaf_obj_widetag_p(widetag_of(base))) gc_enqueue(pointer);
 }
 
 inline void gc_mark_obj(lispobj thing) {
-    if (is_lisp_pointer(thing))
-        __mark_obj(thing);
+    if (is_lisp_pointer(thing)) __mark_obj(thing);
 }
 
 static inline void mark_pair(lispobj* where)
@@ -275,8 +235,7 @@ static inline void mark_pair(lispobj* where)
 
 void gc_mark_range(lispobj* where, long count) {
     long i;
-    for(i=0; i<count; ++i)
-        gc_mark_obj(where[i]);
+    for(i=0; i<count; ++i) gc_mark_obj(where[i]);
 }
 
 #define HT_ENTRY_LIVENESS_FUN_ARRAY_NAME alivep_funs
@@ -287,14 +246,10 @@ static void trace_using_layout(lispobj layout, lispobj* where, int nslots)
     // Apart from the allowance for untagged pointers in lockfree list nodes,
     // this contains almost none of the special cases that gencgc does.
     if (!layout) return;
-#ifdef LISP_FEATURE_METASPACE
-    gc_mark_obj(LAYOUT(layout)->friend);
-#else
     gc_mark_obj(layout);
-#endif
     if (lockfree_list_node_layout_p(LAYOUT(layout))) { // allow untagged 'next'
-        struct instance* node = (struct instance*)where;
-        lispobj next = node->slots[INSTANCE_DATA_START];
+        struct list_node* node = (void*)where;
+        lispobj next = node->_node_next;
         // ignore if 0
         if (fixnump(next) && next) __mark_obj(next|INSTANCE_POINTER_LOWTAG);
     }
@@ -302,8 +257,7 @@ static void trace_using_layout(lispobj layout, lispobj* where, int nslots)
     int i;
     lispobj* slots = where+1;
     for (i=0; i<nslots; ++i)
-        if (bitmap_logbitp(i, bitmap) && is_lisp_pointer(slots[i]))
-            __mark_obj(slots[i]);
+        if (bitmap_logbitp(i, bitmap)) gc_mark_obj(slots[i]);
 }
 
 static void trace_object(lispobj* where)
@@ -317,7 +271,6 @@ static void trace_object(lispobj* where)
     sword_t scan_from = 1;
     sword_t scan_to = sizetab[widetag](where);
     sword_t i;
-    struct weak_pointer *weakptr;
     switch (widetag) {
     case SIMPLE_VECTOR_WIDETAG:
 #ifdef LISP_FEATURE_UBSAN
@@ -327,7 +280,7 @@ static void trace_object(lispobj* where)
         // so the table will not need rehash as a result of gc.
         // Ergo, those may be treated just like ordinary simple vectors.
         // However, weakness remains as a special case.
-        if (vector_flagp(header, VectorWeak)) {
+        if (!stray_pointer_detector_fn && vector_flagp(header, VectorWeak)) {
             if (!vector_flagp(header, VectorHashing)) {
                 add_to_weak_vector_list(where, header);
                 return;
@@ -381,52 +334,62 @@ static void trace_object(lispobj* where)
         return;
 #endif
         break;
-#ifdef LISP_FEATURE_COMPACT_SYMBOL
     case SYMBOL_WIDETAG:
         {
         struct symbol* s = (void*)where;
         gc_mark_obj(decode_symbol_name(s->name));
-        gc_mark_obj(s->value);
-        gc_mark_obj(s->info);
-        gc_mark_obj(s->fdefn);
-        // process the unnamed slot of augmented symbols
-        if ((s->header & 0xFF00) == (SYMBOL_SIZE<<8)) gc_mark_obj(*(1+&s->fdefn));
+        gc_mark_range(&s->value, 3);
         }
         return;
-#endif
     case FDEFN_WIDETAG:
-        gc_mark_obj(fdefn_callee_lispobj((struct fdefn*)where));
+        gc_mark_obj(decode_fdefn_rawfun((struct fdefn*)where));
         scan_to = 3;
         break;
     case WEAK_POINTER_WIDETAG:
-        weakptr = (struct weak_pointer*)where;
-        if (is_lisp_pointer(weakptr->value) && interesting_pointer_p(weakptr->value))
+        {
+        struct weak_pointer *weakptr = (void*)where;
+        if (stray_pointer_detector_fn) gc_mark_obj(weakptr->value);
+        else if (is_lisp_pointer(weakptr->value) && interesting_pointer_p(weakptr->value))
             add_to_weak_pointer_chain(weakptr);
+        }
         return;
     default:
         if (leaf_obj_widetag_p(widetag)) return;
     }
-    for(i=scan_from; i<scan_to; ++i)
-        gc_mark_obj(where[i]);
+    for (i=scan_from; i<scan_to; ++i) gc_mark_obj(where[i]);
 }
 
 void prepare_for_full_mark_phase()
 {
-#ifndef LISP_FEATURE_USE_CONS_REGION
-    hopscotch_create(&mark_bits, HOPSCOTCH_HASH_FUN_DEFAULT,
-                     N_WORD_BYTES, /* table values are machine words */
-                     65536, /* initial size */
-                     0);
-#endif
-
     free_page = page_table_pages;
-    suballocator_free_ptr = suballocator_end_ptr = 0;
-    struct Qblock* block = (struct Qblock*)get_free_page();
-    dprintf(("Queue block holds %d objects\n", (int)QBLOCK_CAPACITY));
+    struct Qblock* block = (struct Qblock*)get_free_page(0);
     scav_queue.head_block = block;
     scav_queue.tail_block = block;
     scav_queue.recycler   = 0;
-    gc_assert(!scav_queue.head_block->count);
+    block->next = 0;
+    block->tail = block->count = 0;
+    /* Consume as many bits as cover the entire dynamic space regardless
+     * of its current usage.  Same for the other spaces.
+     * This previously tried to be clever about using only as many bits for
+     * dynamic space as correspond to the current high water mark, which was
+     * an ill-conceived idea, because cull_weak_hash_tables() can consume
+     * dynamic space when processing finalizers. So it marks an object live,
+     * but that object's mark bit could be past the reserved range of dynamic
+     * space mark bits, thus accidentally marking some _other_ thing live.
+     * And heaven forbid that other object isn't supposed to be live,
+     * you're in for a heap of trouble (pun intended) */
+    sword_t nbits_dynamic = dynamic_space_size / (2*N_WORD_BYTES);
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    sword_t nbits_fixedobj = FIXEDOBJ_SPACE_SIZE / (2*N_WORD_BYTES);
+    sword_t nbits_text = text_space_size / (2*N_WORD_BYTES);
+    fixedobj_index_bit_bias = nbits_dynamic;
+    text_index_bit_bias = fixedobj_index_bit_bias + nbits_fixedobj;
+    sword_t nbytes = (nbits_dynamic + nbits_fixedobj + nbits_text) / 8;
+#else
+    sword_t nbytes = nbits_dynamic / 8;
+#endif
+    markbits_size = ALIGN_UP(nbytes, GETPAGESIZE);
+    fullcgcmarks = (void*)os_allocate(markbits_size);
 }
 
 void execute_full_mark_phase()
@@ -443,15 +406,6 @@ void execute_full_mark_phase()
         gc_enqueue(obj);
         where += listp(obj) ? 2 : headerobj_size(where);
     }
-#ifdef LISP_FEATURE_METASPACE
-    where = (lispobj*)METASPACE_START;
-    end = (lispobj*)READ_ONLY_SPACE_END;
-    while (where < end) {
-        lispobj obj = compute_lispobj(where);
-        gc_enqueue(obj);
-        where += listp(obj) ? 2 : headerobj_size(where);
-    }
-#endif
     gc_mark_obj(lisp_package_vector);
     gc_mark_obj(lisp_init_function);
     gc_mark_obj(gc_object_watcher);
@@ -459,6 +413,7 @@ void execute_full_mark_phase()
     do {
         lispobj ptr = gc_dequeue();
         gc_dcheck(ptr != 0);
+        stray_pointer_source_obj = ptr;
         if (!listp(ptr))
             trace_object(native_pointer(ptr));
         else
@@ -466,6 +421,7 @@ void execute_full_mark_phase()
     } while (scav_queue.head_block->count ||
              (test_weak_triggers(pointer_survived_gc_yet, gc_mark_obj) &&
               scav_queue.head_block->count));
+    stray_pointer_source_obj = 0;
 
 #if HAVE_GETRUSAGE
     getrusage(RUSAGE_SELF, &after);
@@ -474,8 +430,8 @@ void execute_full_mark_phase()
              (a.field.tv_usec-b.field.tv_usec)) / 1000000.0
     if (gencgc_verbose)
         fprintf(stderr,
-                "[Mark phase: %d pages used, HT-count=%d, ET=%f+%f sys+usr]\n",
-                (int)(page_table_pages - free_page), mark_bits.count,
+                "[Mark phase: %d pages used, ET=%f+%f sys+usr]\n",
+                (int)(page_table_pages - free_page),
                 timediff(before, after, ru_stime), timediff(before, after, ru_utime));
 #endif
 }
@@ -511,35 +467,15 @@ static void local_smash_weak_pointers()
     weak_vectors = 0;
 }
 
-__attribute__((unused)) static char *fillerp(lispobj* where)
-{
-    page_index_t page;
-    if (where[0] | where[1])
-        return "cons";
-    if ((page = find_page_index(where)) >= 0 && page_single_obj_p(page))
-        return "cons (largeobj filler)";
-    return "cons (filler)";
-}
-
-static FILE *sweeplog;
-static int sweep_mode = 1;
-
-# define NOTE_GARBAGE(gen,addr,nwords,tally,erase) \
-  { tally[gen] += nwords; \
-    if (sweep_mode & 2) /* print before erasing */ \
-     fprintf(sweeplog, "%5d %d #x%"OBJ_FMTX": %"OBJ_FMTX" %"OBJ_FMTX"\n", \
-             (int)nwords, gen, compute_lispobj(addr), \
-             addr[0], addr[1]); \
-    if (sweep_mode & 1) { erase; } }
-
 #ifndef LISP_FEATURE_IMMOBILE_SPACE
 #undef immobile_obj_gen_bits
 #define immobile_obj_gen_bits(x) (lose("No page index?"),0)
 #else
-static void sweep_fixedobj_pages(long *zeroed)
+static void sweep_fixedobj_pages()
 {
     low_page_index_t page;
-
+    uword_t space_base = FIXEDOBJ_SPACE_START;
+    sword_t bitmap_index_bias = fixedobj_index_bit_bias;
     for (page = FIXEDOBJ_RESERVED_PAGES ; ; ++page) {
         lispobj *obj = fixedobj_page_address(page);
         if (obj >= fixedobj_free_pointer)
@@ -551,47 +487,44 @@ static void sweep_fixedobj_pages(long *zeroed)
         lispobj *limit = (lispobj*)((char*)obj + IMMOBILE_CARD_BYTES - obj_spacing);
         for ( ; obj <= limit ; obj = (lispobj*)((char*)obj + obj_spacing) ) {
             lispobj header = *obj;
-            uword_t markbit = boxedobj_widetag_to_markbit(header_widetag(header));
-            if (fixnump(header)) { // is a hole
-            } else if (header & markbit) { // live object
-                *obj = header ^ markbit;
-            } else {
-                NOTE_GARBAGE(immobile_obj_gen_bits(obj), obj, nwords, zeroed,
-                             memset(obj, 0, nwords * N_WORD_BYTES));
-            }
+            if (fixnump(header)) continue; // is a hole
+            uword_t index = bitmap_index_bias + (((uword_t)obj - space_base) >> (1+WORD_SHIFT));
+            uword_t livep =
+                fullcgcmarks[index / N_WORD_BITS] & ((uword_t)1 << (index % N_WORD_BITS));
+            if (!livep) memset(obj, 0, nwords * N_WORD_BYTES);
         }
     }
-    // The reserved fixedobj page has the vector of primitive object layouts.
-    lispobj* obj = fixedobj_page_address(0);
-    lispobj* limit = fixedobj_page_address(FIXEDOBJ_RESERVED_PAGES);
-    while (obj < limit) {
-        lispobj header = *obj;
-        uword_t markbit = boxedobj_widetag_to_markbit(header_widetag(header));
-        if (header & markbit) *obj = header ^ markbit;
-        obj += headerobj_size(obj);
-    }
 }
 #endif
 
-/* Overwrite exactly 1 object wit non-pointer words of some sort.
+/* Overwrite exactly 1 object with non-pointer words of some sort.
  * This eliminates tenured garbage in pseudo-static-generation,
  * and does NOT strive to to write as few words as possible,
  * unlike deposit_filler() which tries to be efficient */
 static void clobber_headered_object(lispobj* addr, sword_t nwords)
 {
+    // FIXME: clobbering an object on single-object pages should free entire pages
     page_index_t page = find_page_index(addr);
     if (page < 0) { // code space
-        struct code* code  = (struct code*)addr;
-        if (!filler_obj_p((lispobj*)code)) {
-            code->boxed_size = 0;
-            code->header = (nwords << CODE_HEADER_SIZE_SHIFT)
-                           | CODE_HEADER_WIDETAG;
-            memset(addr+2, 0, (nwords - 2) * N_WORD_BYTES);
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+        extern lispobj codeblob_freelist;
+        if (widetag_of(addr) == CODE_HEADER_WIDETAG) {
+            // OAOO violation - like sweep_immobile_text()
+            assign_widetag(addr, FILLER_WIDETAG);
+            ((char*)addr)[2] = 0; // clear the TRACED flag
+            ((char*)addr)[3] = 0; // clear the WRITTEN flag and the generation
+            // add to list only if it is above tlsf_mem_start
+            // (below it will never by utilized by the TLSF allocator)
+            if (addr >= tlsf_mem_start) {
+                addr[1] = codeblob_freelist; // push into to-be-freed list
+                codeblob_freelist = (lispobj)addr;
+            }
         }
-    } else if (page_table[page].type == PAGE_TYPE_CODE) {
+#endif
+    } else if ((SINGLE_OBJECT_FLAG|page_table[page].type) == (SINGLE_OBJECT_FLAG|PAGE_TYPE_CODE)) {
         // Code pages don't want (0 . 0) fillers, otherwise heap checking
         // gets an error: "object @ 0x..... is non-code on code page"
-        addr[0] = (nwords - 1) << N_WIDETAG_BITS | FILLER_WIDETAG;
+        addr[0] = make_filler_header(nwords);
         addr[1] = 0;
     } else {
         memset(addr, 0, nwords * N_WORD_BYTES);
@@ -602,40 +535,43 @@ static uword_t sweep(lispobj* where, lispobj* end,
                      __attribute__((unused)) uword_t arg)
 {
     sword_t nwords;
-
+    uword_t space_base = DYNAMIC_SPACE_START;
+    sword_t bitmap_index_bias = 0;
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    if (find_page_index(where) < 0) {
+        gc_assert(find_text_page_index(where) >= 0);
+        space_base = TEXT_SPACE_START;
+        bitmap_index_bias = text_index_bit_bias;
+    }
+#endif
     for ( ; where < end ; where += nwords ) {
         lispobj word = *where;
-        int livep = 0;
+        uword_t index = bitmap_index_bias + (((uword_t)where - space_base) >> (1+WORD_SHIFT));
+        uword_t livep =
+            fullcgcmarks[index / N_WORD_BITS] & ((uword_t)1 << (index % N_WORD_BITS));
         if (is_header(word)) {
             nwords = headerobj_size2(where, word);
-            lispobj markbit = general_widetag_to_markbit(header_widetag(word));
-            if (word & markbit) livep = 1, *where = word ^ markbit;
-            if (!livep) clobber_headered_object(where, nwords);
+            if (!livep && header_widetag(word) != FILLER_WIDETAG)
+                clobber_headered_object(where, nwords);
         } else {
             nwords = 2;
-            livep = cons_markedp((lispobj)where);
             if (!livep) where[0] = where[1] = (uword_t)-1;
         }
     }
     return 0;
 }
 
-// sweep_mode: 1 = erase, 2 = print, 3 = both
-void toggle_print_garbage(char *filename, int enable)
-{
-    if (enable) {
-        if (sweeplog) {
-          fprintf(stderr,"Erasing previous sweep log file\n");
-          fclose(sweeplog);
-        }
-        sweeplog = fopen(filename, "w");
-        sweep_mode = enable < 0 ? 2 : 3;
-        fprintf(stderr, "Set sweep mode to %d\n", sweep_mode);
-    } else {
-        fclose(sweeplog);
-        fprintf(stderr, "Sweep log closed\n");
-        sweep_mode = 1;
-    }
+void dispose_markbits() {
+    os_deallocate((void*)fullcgcmarks, markbits_size);
+    fullcgcmarks = 0; markbits_size = 0;
+    page_index_t page;
+    // Give back all private-use pages and indicate need-to-zero
+    for (page = free_page; page < page_table_pages; ++page) {
+        gc_assert((page_table[page].type & PAGE_TYPE_MASK) == PAGE_TYPE_UNBOXED);
+        gc_assert(!page_bytes_used(page));
+        set_page_need_to_zero(page, 1);
+        page_table[page].type = FREE_PAGE_FLAG;
+      }
 }
 
 void execute_full_sweep_phase()
@@ -645,16 +581,23 @@ void execute_full_sweep_phase()
     local_smash_weak_pointers();
     gc_dispose_private_pages();
     cull_weak_hash_tables(alivep_funs);
+    scan_finalizers();
 
     memset(words_zeroed, 0, sizeof words_zeroed);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    if (sweeplog) fprintf(sweeplog, "-- fixedobj space --\n");
-    sweep_fixedobj_pages(words_zeroed);
-    if (sweeplog) fprintf(sweeplog, "-- varyobj space --\n");
-    sweep((lispobj*)VARYOBJ_SPACE_START, varyobj_free_pointer,
+    sweep_fixedobj_pages();
+    sweep((lispobj*)TEXT_SPACE_START, text_space_highwatermark,
           (uword_t)words_zeroed);
+    // Recompute generation masks for text space
+    int npages = (ALIGN_UP((uword_t)text_space_highwatermark, IMMOBILE_CARD_BYTES)
+                  - TEXT_SPACE_START) / IMMOBILE_CARD_BYTES;
+    memset(text_page_genmask, 0, npages);
+    lispobj* where = (lispobj*)TEXT_SPACE_START;
+    for ( ; where < text_space_highwatermark ; where += object_size(where) )
+        if (widetag_of(where) == CODE_HEADER_WIDETAG)
+            text_page_genmask[find_text_page_index(where)]
+                |= (1 << immobile_obj_gen_bits(where));
 #endif
-    if (sweeplog) fprintf(sweeplog, "-- dynamic space --\n");
     walk_generation(sweep, -1, (uword_t)words_zeroed);
     if (gencgc_verbose) {
         fprintf(stderr, "[Sweep phase: ");
@@ -663,24 +606,5 @@ void execute_full_sweep_phase()
             fprintf(stderr, "%ld%s", words_zeroed[i], i?"+":"");
         fprintf(stderr, " words zeroed]\n");
     }
-#ifdef LISP_FEATURE_USE_CONS_REGION
-    page_index_t page;
-    for (page = 0; page < next_free_page; ++page)
-        if (page_table[page].type == PAGE_TYPE_CONS) {
-            char* base = page_address(page);
-            char* bits = base + CONS_PAGE_USABLE_BYTES;
-            char* end  = base + GENCGC_PAGE_BYTES;
-            memset(bits, 0, end-bits);
-        }
-#else
-    hopscotch_destroy(&mark_bits);
-#endif
-
-    if (sweeplog)
-        fflush(sweeplog);
-
-    for (free_page = next_free_page; free_page < page_table_pages; ++free_page) {
-        set_page_need_to_zero(free_page, 1);
-        page_table[free_page].type = FREE_PAGE_FLAG;
-      }
+    dispose_markbits();
 }
