@@ -32,6 +32,7 @@
 #include "interr.h"
 #include "lispregs.h"
 #include "runtime.h"
+#include "genesis/cons.h"
 #include "genesis/static-symbols.h"
 #include "genesis/fdefn.h"
 
@@ -197,7 +198,6 @@ futex_init()
         futex_private_supported_p = 1;
     } else {
         futex_private_supported_p = 0;
-        SHOW("No futex private suppport\n");
     }
 }
 
@@ -225,9 +225,23 @@ futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
     struct mutex* m = (void*)((char*)lock_word - offsetof(struct mutex,state));
     char *name = m->name != NIL ? (char*)VECTOR(m->name)->data : "(unnamed)";
 #endif
-  if (sec<0) {
+    if (sec<0) { // unbounded wait
       lisp_mutex_event1("start futex wait", name);
-      t = sys_futex(lock_word, futex_wait_op(), oldval, 0);
+      // a mutex is profiled if its %NAME is a cons. %NAME is 1 word past the lock_word
+      lispobj name = ((uword_t*)lock_word)[1];
+      if (listp(name) && name != NIL && fixnump(CONS(name)->cdr)) {
+            struct timespec start_time, end_time;
+            clock_gettime(CLOCK_MONOTONIC, &start_time);
+            t = sys_futex(lock_word, futex_wait_op(), oldval, 0);
+            clock_gettime(CLOCK_MONOTONIC, &end_time);
+            // if my calculation is correct, this won't overflow for 146 years
+            // (most-positive-fixnum / nanoseconds-per-year)
+            long delta = (end_time.tv_nsec - start_time.tv_nsec)
+                         + (end_time.tv_sec - start_time.tv_sec) * 1000000000;
+            __sync_fetch_and_add(&CONS(name)->cdr, make_fixnum(delta));
+      } else {
+            t = sys_futex(lock_word, futex_wait_op(), oldval, 0);
+      }
   }
   else {
       timeout.tv_sec = sec;
@@ -389,9 +403,13 @@ sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
 #endif
 
 #ifdef LISP_FEATURE_GENCGC
-    if (gencgc_handle_wp_violation(addr)) return;
+    if (gencgc_handle_wp_violation(context, addr)) return;
 #else
     if (cheneygc_handle_wp_violation(context, addr)) return;
+#endif
+    extern int diagnose_arena_fault(os_context_t*,char*);
+#ifdef LISP_FEATURE_SYSTEM_TLABS
+    if (diagnose_arena_fault(context, addr)) return;
 #endif
     if (!handle_guard_page_triggered(context, addr))
             sbcl_fallback_sigsegv_handler(signal, info, context);
