@@ -208,6 +208,36 @@ int sb_GetTID()
     // from PID_MAX + 2 (100001) to INT_MAX. So casting to int is safe.
     return (int)id;
 }
+#elif defined __OpenBSD__
+int sb_GetTID()
+{
+    return getthrid();
+}
+#elif defined __APPLE__
+// follows is what the apple pthreads implementation does...
+#define __TSD_MACH_THREAD_SELF 3
+#if defined(__x86_64__)
+__attribute__((always_inline)) static inline uintptr_t _os_tsd_get_direct(unsigned long slot) {
+        uintptr_t ret;
+        __asm__ volatile ("mov %%gs:%1, %0" : "=r" (ret) : "m" (*(void **)(slot * sizeof(void *))));
+        return ret;
+}
+#else // __x86_64__
+__attribute__((always_inline)) static inline uintptr_t _os_tsd_get_direct(unsigned long slot) {
+        uintptr_t *base;
+#if defined(__arm__)
+        __asm__("mrc p15, 0, %0, c13, c0, 3\n"
+                "bic %0, %0, #0x3\n" : "=r" (base));
+#elif defined(__arm64__)
+        __asm__ ("mrs %0, TPIDRRO_EL0" : "=r" (base));
+#endif
+        return base[slot];
+}
+#endif
+uint32_t sb_GetTID(void) {
+        // actually a 32-bit id; truncation is harmless
+        return (uint32_t)_os_tsd_get_direct(__TSD_MACH_THREAD_SELF);
+}
 #else
 #define sb_GetTID() 0
 #endif
@@ -342,7 +372,7 @@ void create_main_lisp_thread(lispobj function) {
 #if defined THREADS_USING_GCSIGNAL && \
     (defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_RISCV)
     /* SIG_STOP_FOR_GC defaults to blocked on PPC? */
-    unblock_gc_signals();
+    unblock_gc_stop_signal();
 #endif
     link_thread(th);
     th->os_kernel_tid = get_nonzero_tid();
@@ -672,7 +702,7 @@ static void attach_os_thread(init_thread_data *scribble)
     /* new-lisp-thread-trampoline doesn't like when the GC signal is blocked */
     /* FIXME: could be done using a single call to pthread_sigmask
        together with blocking the deferrable signals above. */
-    unblock_gc_signals();
+    unblock_gc_stop_signal();
 #endif
 
     th->os_kernel_tid = get_nonzero_tid();
@@ -775,12 +805,9 @@ extern void funcall_alien_callback(lispobj arg1, lispobj arg2, lispobj arg0,
   __attribute__((sysv_abi));
 #endif
 
-void
-#ifdef LISP_FEATURE_DARWIN
 /* This function's address is assigned into a static symbol's value slot,
  * so it has to look like a fixnum. lp#1991485 */
-__attribute__((aligned(8)))
-#endif
+void __attribute__((aligned(8)))
 callback_wrapper_trampoline(
 #if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
     /* On the x86oid backends, the assembly wrapper happens to not pass
