@@ -115,8 +115,6 @@ static void set_seh_frame(void *frame)
 }
 #endif
 
-#if defined(LISP_FEATURE_SB_THREAD)
-
 void alloc_gc_page()
 {
     gc_assert(VirtualAlloc(GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
@@ -158,8 +156,6 @@ void unmap_gc_page()
                              PAGE_NOACCESS, &oldProt));
 }
 
-#endif
-
 /* This feature has already saved me more development time than it
  * took to implement.  In its current state, ``dynamic RT<->core
  * linking'' is a protocol of initialization of C runtime and Lisp
@@ -208,7 +204,7 @@ void unmap_gc_page()
  * bundle'' that rolls up your patch, redumps and -- presto -- 100MiB
  * program is fixed by sending and loading a 50KiB thingie.
  *
- * However, until LISP_FEATURE_LINKAGE_TABLE, if your bug were fixed
+ * However, until LISP_FEATURE_ALIEN_LINKAGE_TABLE, if your bug were fixed
  * by modifying two lines of _C_ sources, a customer described above
  * had to be ready to receive and reinstall a new 100MiB
  * executable. With the aid of code below, deploying such a fix
@@ -238,15 +234,6 @@ void unmap_gc_page()
  * ``global quiesce point'' that is generally required for this kind
  * of worldwide revolution -- around collect_garbage.
  *
- * What's almost unnoticeable from the C side (where you are now, dear
- * reader): using the same style for all linking is beautiful. I tried
- * to leave old-style linking code in place for the sake of
- * _non-linkage-table_ platforms (they probably don't have -ldl or its
- * equivalent, like LL/GPA, at all) -- but i did it usually by moving
- * the entire `old style' code under #-linkage-table and
- * refactoring the `new style' branch, instead of cutting the tail
- * piecemeal and increasing #+-ifdeffery amount & the world enthropy.
- *
  * If we look at the majority of the ``new style'' code units, it's a
  * common thing to observe how #+-ifdeffery _vanishes_ instead of
  * multiplying: #-sb-xc, #+sb-xc-host and #-sb-xc-host end up
@@ -454,7 +441,6 @@ void* os_dlsym_default(char* name)
     return result;
 }
 
-#if defined(LISP_FEATURE_SB_THREAD)
 /* We want to get a slot in TIB that (1) is available at constant
    offset, (2) is our private property, so libraries wouldn't legally
    override it, (3) contains something predefined for threads created
@@ -561,7 +547,6 @@ int os_preinit(char *argv[], char *envp[])
 #endif
     return 0;
 }
-#endif  /* LISP_FEATURE_SB_THREAD */
 
 int os_number_of_processors = 1;
 
@@ -723,6 +708,65 @@ set_up_win64_seh_thunk(size_t page_size)
 
 static LARGE_INTEGER lisp_init_time;
 static double qpcMultiplier;
+DWORD win32_page_size = 0;
+ULONG win32_stack_guarantee = 0;
+
+void
+win32_set_stack_guarantee()
+{
+    /* 64 KB appears to be enough to run the SWANK debugger + (FACT 1000). 32 KB
+     * is surprisingly not enough for the debugger. */
+    ULONG request = 64*1024;
+    if (!SetThreadStackGuarantee(&request)) {
+        fprintf(stderr, "ERROR: SetThreadStackGuarantee failed: 0x%lx.\n",
+                GetLastError());
+        fflush(stderr);
+    }
+
+    /* Store stack guarantee size the first time around so that
+     * win32_reset_stack_overflow_guard_page() may restore the stack guard. */
+    if (!win32_stack_guarantee)
+        SetThreadStackGuarantee(&win32_stack_guarantee);
+}
+
+/*
+ * The usual way to do this would be to invoke _resetstkoflw(). However, it
+ * refuses to re-establish the stack overflow guard page when the stack
+ * pointer is still within the stack area reserved by SetThreadStackGuarantee().
+ *
+ * This is invoked when SB-KERNEL:HANDLE-WIN32-EXCEPTION is unwound after
+ * handling a STACK_OVERFLOW_EXCEPTION.
+ */
+void
+win32_reset_stack_overflow_guard_page() {
+    struct thread *self = get_sb_vm_thread();
+
+    /* this is similar to CONTROL_STACK_RETURN_GUARD_PAGE on other platforms,
+     * but Windows handles the page faults on its own.
+     *
+     * From experimentation, it seems that as long as set up a guard region
+     * somewhere below the stack guarantee, Windows will manage to raise a
+     * STACK_OVERFLOW_EXCEPTION appropriately next time we exhaust the stack.
+     * Furthermore, if we reprotect sufficiently far away from the stack
+     * guarantee, user code can get away with modifying the stack while
+     * unwinding from a stack overflow condition without retriggering the guard
+     * page. See test (:EXHAUST :WRITE-TO-STACK-ON-UNWIND). */
+#define WIN32_STACK_GUARD_SLACK (2*win32_stack_guarantee + win32_page_size)
+    void *stack_guard_start = CONTROL_STACK_GUARD_PAGE(self);
+    fprintf(stderr, "INFO: Reprotecting control stack guard (0x%p+0x%x)\n",
+            stack_guard_start, WIN32_STACK_GUARD_SLACK);
+    fflush(stderr);
+
+    DWORD oldprot;
+    VirtualProtect(stack_guard_start + WIN32_STACK_GUARD_SLACK,
+                   /* a single page would probably be enough, but let's cargo
+                    * cult Windows and use guarantee size + 1 page. */
+                   win32_stack_guarantee + win32_page_size,
+                   PAGE_READWRITE | PAGE_GUARD,
+                   &oldprot);
+
+    self->state_word.control_stack_guard_page_protected = 1;
+}
 
 void os_init()
 {
@@ -741,6 +785,7 @@ void os_init()
 
     SYSTEM_INFO system_info;
     GetSystemInfo(&system_info);
+    win32_page_size = system_info.dwPageSize;
     os_vm_page_size = system_info.dwPageSize > BACKEND_PAGE_BYTES?
         system_info.dwPageSize : BACKEND_PAGE_BYTES;
     os_number_of_processors = system_info.dwNumberOfProcessors;
@@ -763,15 +808,9 @@ uword_t get_monotonic_time()
 }
 #endif
 
-static os_vm_address_t os_validate_nocommit(int,os_vm_address_t,os_vm_size_t);
-
 os_vm_address_t
-os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id)
+os_alloc_gc_space(int space_id, int attributes, os_vm_address_t addr, os_vm_size_t len)
 {
-    // Reserving the dynamic space doesn't commit it.
-    if (space_id == DYNAMIC_CORE_SPACE_ID && (attributes & MOVABLE))
-        return os_validate_nocommit(attributes, addr, len);
-
     if (!addr) {
         int protection = attributes & IS_GUARD_PAGE ? PAGE_NOACCESS : PAGE_EXECUTE_READWRITE;
         os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, protection);
@@ -779,7 +818,10 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id
         return actual;
     }
 
-    os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+    // Reserving the dynamic space doesn't commit it.
+    DWORD commit =
+      (space_id == DYNAMIC_CORE_SPACE_ID && (attributes & MOVABLE)) ? 0 : MEM_COMMIT;
+    os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE|commit, PAGE_EXECUTE_READWRITE);
 
     if (!actual) {
         if (!(attributes & MOVABLE)) {
@@ -790,29 +832,7 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id
             return 0;
         }
 
-        actual = VirtualAlloc(NULL, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE);
-        gc_assert(actual);
-    }
-
-    return actual;
-}
-
-/* Used to allocate the dynamic space, as it may be very large.
- * Dynamically comitted by gc_alloc_new_region() or gc_alloc_large() */
-static os_vm_address_t
-os_validate_nocommit(int attributes, os_vm_address_t addr, os_vm_size_t len)
-{
-    os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
-
-    if (!actual) {
-        if (!(attributes & MOVABLE)) {
-            fprintf(stderr,
-                    "VirtualAlloc: wanted %lu bytes at %p, actually mapped at %p\n",
-                    (unsigned long) len, addr, actual);
-            fflush(stderr);
-            return 0;
-        }
-        actual = VirtualAlloc(NULL, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
+        actual = VirtualAlloc(NULL, len, MEM_RESERVE|commit, PAGE_EXECUTE_READWRITE);
         gc_assert(actual);
     }
 
@@ -829,12 +849,6 @@ void os_decommit_mem(os_vm_address_t addr,  os_vm_size_t len) {
     gc_assert(VirtualFree(addr, len, MEM_DECOMMIT));
 }
 
-void
-os_invalidate(os_vm_address_t addr, os_vm_size_t len)
-{
-    gc_assert(VirtualFree(addr, 0, MEM_RELEASE));
-}
-
 /*
  * load_core_bytes() is called to load a chunk of the core file into memory.
  *
@@ -947,7 +961,6 @@ handle_breakpoint_trap(os_context_t *ctx, struct thread* self)
      * 'kind' value (eg trap_Cerror). */
     unsigned trap = *(unsigned char *)OS_CONTEXT_PC(ctx);
 
-#ifdef LISP_FEATURE_SB_THREAD
     /* Before any other trap handler: gc_safepoint ensures that
        inner alloc_sap for passing the context won't trap on
        pseudo-atomic. */
@@ -959,7 +972,6 @@ handle_breakpoint_trap(os_context_t *ctx, struct thread* self)
         thread_interrupted(ctx);
         return 0;
     }
-#endif
 
     /* This is just for info in case the monitor wants to print an
      * approximation. */
@@ -967,13 +979,23 @@ handle_breakpoint_trap(os_context_t *ctx, struct thread* self)
         (lispobj *)*os_context_sp_addr(ctx);
 
     WITH_GC_AT_SAFEPOINTS_ONLY() {
-#if defined(LISP_FEATURE_SB_THREAD)
         block_blockable_signals(&ctx->sigmask);
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+        if (trap == trap_UndefinedFunction) {
+            lispobj* fdefn = (lispobj*)(OS_CONTEXT_PC(ctx) & ~LOWTAG_MASK);
+            if (fdefn && widetag_of(fdefn) == FDEFN_WIDETAG) {
+                // Return to undefined-tramp
+                OS_CONTEXT_PC(ctx) = (uword_t)((struct fdefn*)fdefn)->raw_addr;
+                // with RAX containing the FDEFN
+                *os_context_register_addr(ctx,reg_RAX) =
+                    make_lispobj(fdefn, OTHER_POINTER_LOWTAG);
+            }
+        } else
 #endif
-        handle_trap(ctx, trap);
-#if defined(LISP_FEATURE_SB_THREAD)
+        {
+            handle_trap(ctx, trap);
+        }
         thread_sigmask(SIG_SETMASK,&ctx->sigmask,NULL);
-#endif
     }
 
     /* Done, we're good to go! */
@@ -1071,18 +1093,13 @@ signal_internal_error_or_lose(os_context_t *ctx,
          * aren't supposed to happen during cold init or reinit
          * anyway. */
 
-#if defined(LISP_FEATURE_SB_THREAD)
         block_blockable_signals(&ctx->sigmask);
-#endif
         fake_foreign_function_call(ctx);
 
         WITH_GC_AT_SAFEPOINTS_ONLY() {
             DX_ALLOC_SAP(context_sap, ctx);
             DX_ALLOC_SAP(exception_record_sap, exception_record);
-
-#if defined(LISP_FEATURE_SB_THREAD)
             thread_sigmask(SIG_SETMASK, &ctx->sigmask, NULL);
-#endif
 
             /* The exception system doesn't automatically clear pending
              * exceptions, so we lose as soon as we execute any FP
@@ -1094,9 +1111,7 @@ signal_internal_error_or_lose(os_context_t *ctx,
         }
         /* If Lisp doesn't nlx, we need to put things back. */
         undo_fake_foreign_function_call(ctx);
-#if defined(LISP_FEATURE_SB_THREAD)
         thread_sigmask(SIG_SETMASK, &ctx->sigmask, NULL);
-#endif
         /* FIXME: HANDLE-WIN32-EXCEPTION should be allowed to decline */
         return;
     }
@@ -1189,6 +1204,17 @@ handle_exception_ex(EXCEPTION_RECORD *exception_record,
     int rc;
     EXCEPTION_DISPOSITION disp = ExceptionContinueExecution;
     switch (code) {
+    case EXCEPTION_STACK_OVERFLOW:
+    {
+        void *sp = voidreg(win32_context, sp);
+        fprintf(stderr, "INFO: Caught stack overflow exception (sp=0x%p); "
+                        "proceed with caution.\n", sp);
+        fflush(stderr);
+        self->state_word.control_stack_guard_page_protected = 0;
+        rc = -1;
+        break;
+    }
+
     case EXCEPTION_ACCESS_VIOLATION:
         rc = handle_access_violation(ctx, exception_record, fault_address, self);
         if (rc && continue_search_on_unhandled_access_violation) {
@@ -1280,6 +1306,7 @@ veh(EXCEPTION_POINTERS *ep)
     BOOL from_lisp =
         (rip >= DYNAMIC_SPACE_START && rip < DYNAMIC_SPACE_START+dynamic_space_size) ||
         (rip >= READ_ONLY_SPACE_START && rip < READ_ONLY_SPACE_END) ||
+        immobile_space_p(rip) ||
         (rip >= STATIC_SPACE_START && rip < (uword_t)static_space_free_pointer);
 
     if (code == EXCEPTION_ACCESS_VIOLATION ||
@@ -1382,7 +1409,6 @@ socket_input_available(HANDLE socket, long time, long utime)
         return 0;
 }
 
-#ifdef LISP_FEATURE_SB_THREAD
 /* Atomically mark current thread as (probably) doing synchronous I/O
  * on handle, if no cancellation is requested yet (and return TRUE),
  * otherwise clear thread's I/O cancellation flag and return false.
@@ -1424,9 +1450,6 @@ io_end_interruptible(HANDLE handle)
       return -1;                           \
     }                                      \
     RUN_BODY_ONCE(xx, io_end_interruptible(handle))
-#else
-#define WITH_INTERRUPTIBLE_IO(handle)
-#endif
 
 int console_handle_p(HANDLE handle)
 {
@@ -1434,7 +1457,7 @@ int console_handle_p(HANDLE handle)
     return GetFileType(handle) == FILE_TYPE_CHAR &&
         GetConsoleMode(handle, &mode);
 }
-#ifdef LISP_FEATURE_SB_THREAD
+
 /*
  * (AK writes:)
  *
@@ -1685,7 +1708,6 @@ win32_maybe_interrupt_io(void* thread)
     }
     return done;
 }
-#endif
 
 static const LARGE_INTEGER zero_large_offset = {.QuadPart = 0LL};
 
@@ -1931,31 +1953,21 @@ char *os_get_runtime_executable_path()
 DWORD
 win32_wait_object_or_signal(HANDLE waitFor)
 {
-#ifdef LISP_FEATURE_SB_THREAD
     struct thread *self = get_sb_vm_thread();
     HANDLE handles[] = {waitFor, thread_private_events(self,1)};
     return
         WaitForMultipleObjects(2,handles, FALSE, INFINITE);
-#else
-    return WaitForSingleObject(waitFor, INFINITE);
-#endif
 }
 
 DWORD
 win32_wait_for_multiple_objects_or_signal(HANDLE *handles, DWORD count)
 {
-#ifdef LISP_FEATURE_SB_THREAD
     struct thread *self = get_sb_vm_thread();
     handles[count] = thread_private_events(self,1);
     return
         WaitForMultipleObjects(count + 1, handles, FALSE, INFINITE);
-#else
-    return
-        WaitForMultipleObjects(count, handles, FALSE, INFINITE);
-#endif
 }
 
-#ifdef LISP_FEATURE_SB_THREAD
 /*
  * Portability glue for win32 waitable timers.
  *
@@ -1998,7 +2010,6 @@ os_cancel_wtimer(HANDLE handle)
 {
     CancelWaitableTimer(handle);
 }
-#endif
 
 #ifdef LISP_FEATURE_SB_FUTEX
 int
@@ -2054,30 +2065,7 @@ int sb_pthr_kill(struct thread* thread, int signum)
     return 0;
 }
 
-int sigpending(sigset_t *set)
-{
-    struct extra_thread_data* data = thread_extra_data(get_sb_vm_thread());
-    *set = InterlockedCompareExchange((volatile LONG*)&data->pending_signal_set,
-                                      0, 0);
-    return 0;
-}
-
 /* Signals */
-struct sigaction signal_handlers[NSIG];
-
-/* Never called for now */
-int sigaction(int signum, const struct sigaction* act, struct sigaction* oldact)
-{
-  struct sigaction newact = *act;
-  if (oldact)
-    *oldact = signal_handlers[signum];
-  if (!(newact.sa_flags & SA_SIGINFO)) {
-      newact.sa_sigaction = (typeof(newact.sa_sigaction))newact.sa_handler;
-  }
-  signal_handlers[signum] = newact;
-  return 0;
-}
-
 int sched_yield()
 {
   /* http://stackoverflow.com/questions/1383943/switchtothread-vs-sleep1
