@@ -655,26 +655,26 @@ lispobj decode_fdefn_rawfun(struct fdefn* fdefn) {
     return entrypoint_taggedptr((uword_t)fdefn->raw_addr);
 }
 
+#ifdef LISP_FEATURE_SB_THREAD
 #include "genesis/vector.h"
 #define LOCK_PREFIX 0xF0
 #undef SHOW_PC_RECORDING
 
-extern unsigned int alloc_profile_n_counters;
 extern unsigned int max_alloc_point_counters;
-#ifdef LISP_FEATURE_SB_THREAD
+
 #ifdef LISP_FEATURE_WIN32
 extern CRITICAL_SECTION alloc_profiler_lock;
 #else
 extern pthread_mutex_t alloc_profiler_lock;
 #endif
-#endif
 
-static unsigned int claim_index(int qty)
+
+static unsigned int claim_index(int qty) // qty is 1 or 2
 {
     static boolean warning_issued;
-    unsigned int index = alloc_profile_n_counters;
-    alloc_profile_n_counters += qty;
-    if (alloc_profile_n_counters <= max_alloc_point_counters)
+    unsigned int index = fixnum_value(SYMBOL(N_PROFILE_SITES)->value);
+    SYMBOL(N_PROFILE_SITES)->value += make_fixnum(qty);
+    if (fixnum_value(SYMBOL(N_PROFILE_SITES)->value) <= max_alloc_point_counters)
        return index;
     if (!warning_issued) {
        fprintf(stderr, "allocation profile buffer overflowed\n");
@@ -817,6 +817,7 @@ allocation_tracker_sized(uword_t* sp)
     int __attribute__((unused)) ret = mutex_release(&alloc_profiler_lock);
     gc_assert(ret);
 }
+#endif
 
 __attribute__((sysv_abi)) lispobj call_into_lisp(lispobj fun, lispobj *args, int nargs) {
     extern lispobj call_into_lisp_(lispobj, lispobj *, int, struct thread *)
