@@ -53,6 +53,17 @@ arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
  * want to get to, and on OS, which determines how we get to it.)
  */
 
+void visit_context_registers(void (*proc)(os_context_register_t,int), os_context_t *context)
+{
+    proc(os_context_pc(context), 1);
+    proc(*os_context_register_addr(context,reg_EAX), 1);
+    proc(*os_context_register_addr(context,reg_ECX), 1);
+    proc(*os_context_register_addr(context,reg_EDX), 1);
+    proc(*os_context_register_addr(context,reg_EBX), 1);
+    proc(*os_context_register_addr(context,reg_ESI), 1);
+    proc(*os_context_register_addr(context,reg_EDI), 1);
+}
+
 int *
 os_context_flags_addr(os_context_t *context)
 {
@@ -69,8 +80,6 @@ os_context_flags_addr(os_context_t *context)
     return &context->uc_mcontext.mc_eflags;
 #elif defined __OpenBSD__
     return &context->sc_eflags;
-#elif defined LISP_FEATURE_DARWIN
-    return (int *)(&context->uc_mcontext->SS.EFLAGS);
 #elif defined __NetBSD__
     return &(context->uc_mcontext.__gregs[_REG_EFL]);
 #elif defined LISP_FEATURE_WIN32
@@ -289,12 +298,10 @@ sigtrap_handler(int signal, siginfo_t *info, os_context_t *context)
 
 void
 sigill_handler(int signal, siginfo_t *siginfo, os_context_t *context) {
-#ifndef LISP_FEATURE_MACH_EXCEPTION_HANDLER
     if (*(unsigned short *)OS_CONTEXT_PC(context) == UD2_INST) {
         OS_CONTEXT_PC(context) += 2;
         return sigtrap_handler(signal, siginfo, context);
     }
-#endif
     fake_foreign_function_call(context);
     lose("Unhandled SIGILL at %p.", (void*)OS_CONTEXT_PC(context));
 }
@@ -313,7 +320,7 @@ arch_install_interrupt_handlers()
      * OS I haven't tested on?) and we have to go back to the old CMU
      * CL way, I hope there will at least be a comment to explain
      * why.. -- WHN 2001-06-07 */
-#if !defined(LISP_FEATURE_WIN32) && !defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER)
+#ifndef LISP_FEATURE_WIN32
     ll_install_handler(SIGILL , sigill_handler);
     ll_install_handler(SIGTRAP, sigtrap_handler);
 #endif
@@ -366,7 +373,7 @@ void
 arch_write_linkage_table_entry(int index, void *target_addr, int datap)
 {
     // 'volatile' works around a spurious GCC warning
-    volatile char *reloc_addr = (char*)LINKAGE_TABLE_SPACE_START + index * LINKAGE_TABLE_ENTRY_SIZE;
+    volatile char *reloc_addr = (char*)ALIEN_LINKAGE_TABLE_SPACE_START + index * ALIEN_LINKAGE_TABLE_ENTRY_SIZE;
     if (datap) {
         *(unsigned long *)reloc_addr = (unsigned long)target_addr;
         return;
